@@ -17,6 +17,7 @@
 
 #include <gtest/gtest.h>
 #include <ignition/msgs/int32.pb.h>
+#include <ignition/utilities/ExtraTestMacros.hh>
 
 #include <memory>
 
@@ -29,15 +30,18 @@
 #include "ignition/gazebo/components/Name.hh"
 #include "ignition/gazebo/EntityComponentManager.hh"
 
+#include "../test/helpers/EnvTestFixture.hh"
+
 using namespace ignition;
 using namespace gazebo;
 
 //////////////////////////////////////////////////
-class ComponentTest : public ::testing::Test
+class ComponentTest : public InternalFixture<::testing::Test>
 {
   protected: void SetUp() override
   {
-    common::Console::SetVerbosity(4);
+    InternalFixture::SetUp();
+    common::setenv("IGN_DEBUG_COMPONENT_FACTORY", "true");
   }
 };
 
@@ -58,7 +62,8 @@ TEST_F(ComponentTest, ComponentCanBeCopiedAfterDefaultCtor)
 }
 
 //////////////////////////////////////////////////
-TEST_F(ComponentTest, DataByMove)
+// See https://github.com/ignitionrobotics/ign-gazebo/issues/1175
+TEST_F(ComponentTest, IGN_UTILS_TEST_DISABLED_ON_WIN32(DataByMove))
 {
   auto factory = components::Factory::Instance();
 
