@@ -568,6 +568,25 @@ msgs::Light ignition::gazebo::convert(const sdf::Light &_in)
   out.set_spot_inner_angle(_in.SpotInnerAngle().Radian());
   out.set_spot_outer_angle(_in.SpotOuterAngle().Radian());
   out.set_spot_falloff(_in.SpotFalloff());
+
+  {
+    // todo(ahcorde) Use the field is_light_off in light.proto from
+    // Garden on.
+    auto header = out.mutable_header()->add_data();
+    header->set_key("isLightOn");
+    std::string *value = header->add_value();
+    *value = std::to_string(_in.LightOn());
+  }
+
+  {
+    // todo(ahcorde) Use the field visualize_visual in light.proto from
+    // Garden on.
+    auto header = out.mutable_header()->add_data();
+    header->set_key("visualizeVisual");
+    std::string *value = header->add_value();
+    *value = std::to_string(_in.Visualize());
+  }
+
   if (_in.Type() == sdf::LightType::POINT)
     out.set_type(msgs::Light_LightType_POINT);
   else if (_in.Type() == sdf::LightType::SPOT)
@@ -597,6 +616,43 @@ sdf::Light ignition::gazebo::convert(const msgs::Light &_in)
   out.SetSpotInnerAngle(math::Angle(_in.spot_inner_angle()));
   out.SetSpotOuterAngle(math::Angle(_in.spot_outer_angle()));
   out.SetSpotFalloff(_in.spot_falloff());
+
+  // todo(ahcorde) Use the field is_light_off in light.proto from
+  // Garden on.
+  bool visualizeVisual = true;
+  for (int i = 0; i < _in.header().data_size(); ++i)
+  {
+    for (int j = 0;
+        j < _in.header().data(i).value_size(); ++j)
+    {
+      if (_in.header().data(i).key() ==
+          "visualizeVisual")
+      {
+        visualizeVisual = ignition::math::parseInt(
+          _in.header().data(i).value(0));
+      }
+    }
+  }
+  out.SetVisualize(visualizeVisual);
+
+  // todo(ahcorde) Use the field is_light_off in light.proto from
+  // Garden on.
+  bool isLightOn = true;
+  for (int i = 0; i < _in.header().data_size(); ++i)
+  {
+    for (int j = 0;
+        j < _in.header().data(i).value_size(); ++j)
+    {
+      if (_in.header().data(i).key() ==
+          "isLightOn")
+      {
+        isLightOn = ignition::math::parseInt(
+          _in.header().data(i).value(0));
+      }
+    }
+  }
+  out.SetLightOn(isLightOn);
+
   if (_in.type() == msgs::Light_LightType_POINT)
     out.SetType(sdf::LightType::POINT);
   else if (_in.type() == msgs::Light_LightType_SPOT)
@@ -1263,8 +1319,8 @@ sdf::Sensor ignition::gazebo::convert(const msgs::Sensor &_in)
     if (_in.has_camera())
     {
       sensor.SetHorizontalFov(_in.camera().horizontal_fov());
-      sensor.SetImageWidth(_in.camera().image_size().x());
-      sensor.SetImageHeight(_in.camera().image_size().y());
+      sensor.SetImageWidth(static_cast<int>(_in.camera().image_size().x()));
+      sensor.SetImageHeight(static_cast<int>(_in.camera().image_size().y()));
       sensor.SetPixelFormatStr(_in.camera().image_format());
       sensor.SetNearClip(_in.camera().near_clip());
       sensor.SetFarClip(_in.camera().far_clip());
@@ -1496,3 +1552,141 @@ math::AxisAlignedBox ignition::gazebo::convert(const msgs::AxisAlignedBox &_in)
   out.Max() = msgs::Convert(_in.max_corner());
   return out;
 }
+
+//////////////////////////////////////////////////
+template<>
+IGNITION_GAZEBO_VISIBLE
+msgs::ParticleEmitter ignition::gazebo::convert(const sdf::ParticleEmitter &_in)
+{
+  msgs::ParticleEmitter out;
+  out.set_name(_in.Name());
+  switch (_in.Type())
+  {
+    default:
+    case sdf::ParticleEmitterType::POINT:
+      out.set_type(msgs::ParticleEmitter::POINT);
+      break;
+    case sdf::ParticleEmitterType::BOX:
+      out.set_type(msgs::ParticleEmitter::BOX);
+      break;
+    case sdf::ParticleEmitterType::CYLINDER:
+      out.set_type(msgs::ParticleEmitter::CYLINDER);
+      break;
+    case sdf::ParticleEmitterType::ELLIPSOID:
+      out.set_type(msgs::ParticleEmitter::ELLIPSOID);
+      break;
+  }
+
+  msgs::Set(out.mutable_pose(), _in.RawPose());
+  msgs::Set(out.mutable_size(), _in.Size());
+  msgs::Set(out.mutable_particle_size(), _in.ParticleSize());
+  out.mutable_rate()->set_data(_in.Rate());
+  out.mutable_duration()->set_data(_in.Duration());
+  out.mutable_emitting()->set_data(_in.Emitting());
+  out.mutable_lifetime()->set_data(_in.Lifetime());
+  if (_in.Material())
+  {
+    out.mutable_material()->CopyFrom(convert<msgs::Material>(*_in.Material()));
+  }
+  out.mutable_min_velocity()->set_data(_in.MinVelocity());
+  out.mutable_max_velocity()->set_data(_in.MaxVelocity());
+  msgs::Set(out.mutable_color_start(), _in.ColorStart());
+  msgs::Set(out.mutable_color_end(), _in.ColorEnd());
+  out.mutable_scale_rate()->set_data(_in.ScaleRate());
+  out.mutable_color_range_image()->set_data(_in.ColorRangeImage());
+
+  if (!_in.ColorRangeImage().empty())
+  {
+    std::string path = asFullPath(_in.ColorRangeImage(), _in.FilePath());
+
+    common::SystemPaths systemPaths;
+    systemPaths.SetFilePathEnv(kResourcePathEnv);
+    std::string absolutePath = systemPaths.FindFile(path);
+
+    if (!absolutePath.empty())
+      out.mutable_color_range_image()->set_data(absolutePath);
+  }
+
+  /// \todo(nkoenig) Modify the particle_emitter.proto file to
+  /// have a topic field.
+  if (!_in.Topic().empty())
+  {
+    auto header = out.mutable_header()->add_data();
+    header->set_key("topic");
+    header->add_value(_in.Topic());
+  }
+
+  // todo(anyone) Use particle_scatter_ratio in particle_emitter.proto from
+  // Fortress on.
+  auto header = out.mutable_header()->add_data();
+  header->set_key("particle_scatter_ratio");
+  std::string *value = header->add_value();
+  *value = std::to_string(_in.ScatterRatio());
+
+  return out;
+}
+
+//////////////////////////////////////////////////
+template<>
+IGNITION_GAZEBO_VISIBLE
+sdf::ParticleEmitter ignition::gazebo::convert(const msgs::ParticleEmitter &_in)
+{
+  sdf::ParticleEmitter out;
+  out.SetName(_in.name());
+  switch (_in.type())
+  {
+    default:
+    case msgs::ParticleEmitter::POINT:
+      out.SetType(sdf::ParticleEmitterType::POINT);
+      break;
+    case msgs::ParticleEmitter::BOX:
+      out.SetType(sdf::ParticleEmitterType::BOX);
+      break;
+    case msgs::ParticleEmitter::CYLINDER:
+      out.SetType(sdf::ParticleEmitterType::CYLINDER);
+      break;
+    case msgs::ParticleEmitter::ELLIPSOID:
+      out.SetType(sdf::ParticleEmitterType::ELLIPSOID);
+      break;
+  }
+  out.SetRawPose(msgs::Convert(_in.pose()));
+  out.SetSize(msgs::Convert(_in.size()));
+  out.SetParticleSize(msgs::Convert(_in.particle_size()));
+  out.SetMinVelocity(msgs::Convert(_in.min_velocity()));
+  out.SetMaxVelocity(msgs::Convert(_in.max_velocity()));
+  out.SetColorStart(msgs::Convert(_in.color_start()));
+  out.SetColorEnd(msgs::Convert(_in.color_end()));
+
+  if (_in.has_material())
+  {
+    out.SetMaterial(convert<sdf::Material>(_in.material()));
+  }
+
+  if (_in.has_rate())
+    out.SetRate(_in.rate().data());
+  if (_in.has_duration())
+    out.SetDuration(_in.duration().data());
+  if (_in.has_emitting())
+    out.SetEmitting(_in.emitting().data());
+  if (_in.has_lifetime())
+    out.SetLifetime(_in.lifetime().data());
+  if (_in.has_scale_rate())
+    out.SetScaleRate(_in.scale_rate().data());
+  if (_in.has_color_range_image())
+    out.SetColorRangeImage(_in.color_range_image().data());
+
+  for (int i = 0; i < _in.header().data_size(); ++i)
+  {
+    auto data = _in.header().data(i);
+    if (data.key() == "topic" && data.value_size() > 0)
+    {
+      out.SetTopic(data.value(0));
+    }
+    else if (data.key() == "particle_scatter_ratio" && data.value_size() > 0)
+    {
+      out.SetScatterRatio(std::stof(data.value(0)));
+    }
+  }
+
+  return out;
+}
