@@ -111,6 +111,31 @@ void LevelManager::ReadLevelPerformerInfo()
   this->runner->entityCompMgr.CreateComponent(this->worldEntity,
       components::Physics(*physics));
 
+  // Populate physics options that aren't accessible outside the Element()
+  // See https://github.com/osrf/sdformat/issues/508
+  if (physics->Element() && physics->Element()->HasElement("dart"))
+  {
+    auto dartElem = physics->Element()->GetElement("dart");
+
+    if (dartElem->HasElement("collision_detector"))
+    {
+      auto collisionDetector =
+          dartElem->Get<std::string>("collision_detector");
+
+      this->runner->entityCompMgr.CreateComponent(worldEntity,
+          components::PhysicsCollisionDetector(collisionDetector));
+    }
+    if (dartElem->HasElement("solver") &&
+        dartElem->GetElement("solver")->HasElement("solver_type"))
+    {
+      auto solver =
+          dartElem->GetElement("solver")->Get<std::string>("solver_type");
+
+      this->runner->entityCompMgr.CreateComponent(worldEntity,
+          components::PhysicsSolver(solver));
+    }
+  }
+
   this->runner->entityCompMgr.CreateComponent(this->worldEntity,
       components::MagneticField(this->runner->sdfWorld->MagneticField()));
 
@@ -161,7 +186,7 @@ void LevelManager::ReadLevelPerformerInfo()
 
   sdf::ElementPtr pluginElem;
   // Get the ignition::gazebo plugin element
-  for (auto plugin = worldElem->GetElement("plugin"); plugin;
+  for (auto plugin = worldElem->FindElement("plugin"); plugin;
        plugin = plugin->GetNextElement("plugin"))
   {
     if (plugin->Get<std::string>("name") == kPluginName)
