@@ -20,9 +20,10 @@
 #include <string>
 
 #include <gtest/gtest.h>
+#include <ignition/utilities/ExtraTestMacros.hh>
 
-#include "gz/sim/Server.hh"
-#include "gz/sim/test_config.hh"  // NOLINT(build/include)
+#include "ignition/gazebo/Server.hh"
+#include "ignition/gazebo/test_config.hh"  // NOLINT(build/include)
 
 static const std::string kIgnModelCommand(
     std::string(BREW_RUBY) + std::string(IGN_PATH) + "/ign model ");
@@ -72,7 +73,8 @@ std::string customExecStr(std::string _cmd)
 
 /////////////////////////////////////////////////
 // Test `ign model` command when no Gazebo server is running.
-TEST(ModelCommandAPI, NoServerRunning)
+// See https://github.com/ignitionrobotics/ign-gazebo/issues/1175
+TEST(ModelCommandAPI, IGN_UTILS_TEST_DISABLED_ON_WIN32(NoServerRunning))
 {
   const std::string cmd = kIgnModelCommand + "--list ";
   const std::string output = customExecStr(cmd);
@@ -85,14 +87,14 @@ TEST(ModelCommandAPI, NoServerRunning)
 
 /////////////////////////////////////////////////
 // Tests `ign model` command.
-TEST(ModelCommandAPI, Commands)
+TEST(ModelCommandAPI, IGN_UTILS_TEST_DISABLED_ON_WIN32(Commands))
 {
-  gz::sim::ServerConfig serverConfig;
+  ignition::gazebo::ServerConfig serverConfig;
   // Using an static model to avoid any movements in the simulation.
   serverConfig.SetSdfFile(std::string(PROJECT_SOURCE_PATH) +
                           "/test/worlds/static_diff_drive_vehicle.sdf");
 
-  gz::sim::Server server(serverConfig);
+  ignition::gazebo::Server server(serverConfig);
   // Run at least one iteration before continuing to guarantee correctly set up.
   ASSERT_TRUE(server.Run(true, 5, false));
   // Run without blocking.
