@@ -57,6 +57,7 @@
 #include "ignition/gazebo/components/Name.hh"
 #include "ignition/gazebo/components/ParentLinkName.hh"
 #include "ignition/gazebo/components/ParentEntity.hh"
+#include <ignition/gazebo/components/ParticleEmitter.hh>
 #include "ignition/gazebo/components/Physics.hh"
 #include "ignition/gazebo/components/Pose.hh"
 #include "ignition/gazebo/components/RgbdCamera.hh"
@@ -279,6 +280,31 @@ Entity SdfEntityCreator::CreateEntities(const sdf::World *_world)
   this->dataPtr->ecm->CreateComponent(worldEntity,
       components::Physics(*physics));
 
+  // Populate physics options that aren't accessible outside the Element()
+  // See https://github.com/osrf/sdformat/issues/508
+  if (physics->Element() && physics->Element()->HasElement("dart"))
+  {
+    auto dartElem = physics->Element()->GetElement("dart");
+
+    if (dartElem->HasElement("collision_detector"))
+    {
+      auto collisionDetector =
+          dartElem->Get<std::string>("collision_detector");
+
+      this->dataPtr->ecm->CreateComponent(worldEntity,
+          components::PhysicsCollisionDetector(collisionDetector));
+    }
+    if (dartElem->HasElement("solver") &&
+        dartElem->GetElement("solver")->HasElement("solver_type"))
+    {
+      auto solver =
+          dartElem->GetElement("solver")->Get<std::string>("solver_type");
+
+      this->dataPtr->ecm->CreateComponent(worldEntity,
+          components::PhysicsSolver(solver));
+    }
+  }
+
   // MagneticField
   this->dataPtr->ecm->CreateComponent(worldEntity,
       components::MagneticField(_world->MagneticField()));
@@ -536,6 +562,16 @@ Entity SdfEntityCreator::CreateEntities(const sdf::Link *_link)
     this->SetParent(sensorEntity, linkEntity);
   }
 
+  // Particle emitters
+  for (uint64_t emitterIndex = 0; emitterIndex  < _link->ParticleEmitterCount();
+       ++emitterIndex)
+  {
+    auto emitter = _link->ParticleEmitterByIndex(emitterIndex);
+    auto emitterEntity = this->CreateEntities(emitter);
+
+    this->SetParent(emitterEntity, linkEntity);
+  }
+
   return linkEntity;
 }
 
@@ -671,6 +707,25 @@ Entity SdfEntityCreator::CreateEntities(const sdf::Visual *_visual)
   return visualEntity;
 }
 
+//////////////////////////////////////////////////
+Entity SdfEntityCreator::CreateEntities(const sdf::ParticleEmitter *_emitter)
+{
+  IGN_PROFILE("SdfEntityCreator::CreateEntities(sdf::ParticleEmitter)");
+
+  // Entity
+  Entity emitterEntity = this->dataPtr->ecm->CreateEntity();
+
+  // Components
+  this->dataPtr->ecm->CreateComponent(emitterEntity,
+      components::ParticleEmitter(convert<msgs::ParticleEmitter>(*_emitter)));
+  this->dataPtr->ecm->CreateComponent(emitterEntity,
+      components::Pose(ResolveSdfPose(_emitter->SemanticPose())));
+  this->dataPtr->ecm->CreateComponent(emitterEntity,
+      components::Name(_emitter->Name()));
+
+  return emitterEntity;
+}
+
 //////////////////////////////////////////////////
 Entity SdfEntityCreator::CreateEntities(const sdf::Collision *_collision)
 {
