@@ -22,6 +22,7 @@
 #include <ignition/common/Util.hh>
 #include <ignition/math/Rand.hh>
 #include <ignition/transport/Node.hh>
+#include <ignition/utilities/ExtraTestMacros.hh>
 #include <sdf/Mesh.hh>
 
 #include "ignition/gazebo/components/AxisAlignedBox.hh"
@@ -45,7 +46,13 @@ using namespace ignition::gazebo;
 using namespace std::chrono_literals;
 
 /////////////////////////////////////////////////
-TEST_P(ServerFixture, DefaultServerConfig)
+class ServerFixture : public InternalFixture<::testing::TestWithParam<int>>
+{
+};
+
+/////////////////////////////////////////////////
+// See https://github.com/ignitionrobotics/ign-gazebo/issues/1175
+TEST_P(ServerFixture, IGN_UTILS_TEST_DISABLED_ON_WIN32(DefaultServerConfig))
 {
   ignition::gazebo::ServerConfig serverConfig;
   EXPECT_TRUE(serverConfig.SdfFile().empty());
@@ -153,7 +160,7 @@ TEST_P(ServerFixture, ServerConfigPluginInfo)
 }
 
 /////////////////////////////////////////////////
-TEST_P(ServerFixture, ServerConfigRealPlugin)
+TEST_P(ServerFixture, IGN_UTILS_TEST_DISABLED_ON_WIN32(ServerConfigRealPlugin))
 {
   // Start server
   ServerConfig serverConfig;
@@ -204,7 +211,8 @@ TEST_P(ServerFixture, ServerConfigRealPlugin)
 }
 
 /////////////////////////////////////////////////
-TEST_P(ServerFixture, ServerConfigSensorPlugin)
+TEST_P(ServerFixture,
+       IGN_UTILS_TEST_DISABLED_ON_WIN32(ServerConfigSensorPlugin))
 {
   // Start server
   ServerConfig serverConfig;
@@ -255,7 +263,7 @@ TEST_P(ServerFixture, ServerConfigSensorPlugin)
 }
 
 /////////////////////////////////////////////////
-TEST_P(ServerFixture, SdfServerConfig)
+TEST_P(ServerFixture, IGN_UTILS_TEST_DISABLED_ON_WIN32(SdfServerConfig))
 {
   ignition::gazebo::ServerConfig serverConfig;
 
@@ -288,7 +296,7 @@ TEST_P(ServerFixture, SdfServerConfig)
 }
 
 /////////////////////////////////////////////////
-TEST_P(ServerFixture, ServerConfigLogRecord)
+TEST_P(ServerFixture, IGN_UTILS_TEST_DISABLED_ON_WIN32(ServerConfigLogRecord))
 {
   auto logPath = common::joinPaths(
       std::string(PROJECT_BINARY_PATH), "test_log_path");
@@ -327,7 +335,8 @@ TEST_P(ServerFixture, ServerConfigLogRecord)
 }
 
 /////////////////////////////////////////////////
-TEST_P(ServerFixture, ServerConfigLogRecordCompress)
+TEST_P(ServerFixture,
+       IGN_UTILS_TEST_DISABLED_ON_WIN32(ServerConfigLogRecordCompress))
 {
   auto logPath = common::joinPaths(
       std::string(PROJECT_BINARY_PATH), "test_log_path");
@@ -478,7 +487,7 @@ TEST_P(ServerFixture, RunNonBlocking)
 }
 
 /////////////////////////////////////////////////
-TEST_P(ServerFixture, RunOnceUnpaused)
+TEST_P(ServerFixture, IGN_UTILS_TEST_DISABLED_ON_WIN32(RunOnceUnpaused))
 {
   gazebo::Server server;
   EXPECT_FALSE(server.Running());
@@ -525,7 +534,7 @@ TEST_P(ServerFixture, RunOnceUnpaused)
 }
 
 /////////////////////////////////////////////////
-TEST_P(ServerFixture, RunOncePaused)
+TEST_P(ServerFixture, IGN_UTILS_TEST_DISABLED_ON_WIN32(RunOncePaused))
 {
   gazebo::Server server;
   EXPECT_FALSE(server.Running());
@@ -615,7 +624,65 @@ TEST_P(ServerFixture, SigInt)
 }
 
 /////////////////////////////////////////////////
-TEST_P(ServerFixture, AddSystemWhileRunning)
+TEST_P(ServerFixture, ServerControlStop)
+{
+  // Test that the server correctly reacts to requests on /server_control
+  // service with `stop` set to either false or true.
+
+  gazebo::Server server;
+  EXPECT_FALSE(server.Running());
+  EXPECT_FALSE(*server.Running(0));
+
+  // Run forever, non-blocking.
+  server.Run(false, 0, false);
+
+  IGN_SLEEP_MS(500);
+
+  EXPECT_TRUE(server.Running());
+  EXPECT_TRUE(*server.Running(0));
+
+  transport::Node node;
+  msgs::ServerControl req;
+  msgs::Boolean res;
+  bool result{false};
+  bool executed{false};
+  int sleep{0};
+  int maxSleep{30};
+
+  // first, call with stop = false; the server should keep running
+  while (!executed && sleep < maxSleep)
+  {
+    igndbg << "Requesting /server_control" << std::endl;
+    executed = node.Request("/server_control", req, 100, res, result);
+    sleep++;
+  }
+  EXPECT_TRUE(executed);
+  EXPECT_TRUE(result);
+  EXPECT_FALSE(res.data());
+
+  IGN_SLEEP_MS(500);
+
+  EXPECT_TRUE(server.Running());
+  EXPECT_TRUE(*server.Running(0));
+
+  // now call with stop = true; the server should stop
+  req.set_stop(true);
+
+  igndbg << "Requesting /server_control" << std::endl;
+  executed = node.Request("/server_control", req, 100, res, result);
+
+  EXPECT_TRUE(executed);
+  EXPECT_TRUE(result);
+  EXPECT_TRUE(res.data());
+
+  IGN_SLEEP_MS(500);
+
+  EXPECT_FALSE(server.Running());
+  EXPECT_FALSE(*server.Running(0));
+}
+
+/////////////////////////////////////////////////
+TEST_P(ServerFixture, IGN_UTILS_TEST_DISABLED_ON_WIN32(AddSystemWhileRunning))
 {
   ignition::gazebo::ServerConfig serverConfig;
 
@@ -637,6 +704,7 @@ TEST_P(ServerFixture, AddSystemWhileRunning)
 
   EXPECT_EQ(3u, *server.SystemCount());
 
+  // Add system from plugin
   gazebo::SystemLoader systemLoader;
   auto mockSystemPlugin = systemLoader.LoadPlugin("libMockSystem.so",
       "ignition::gazebo::MockSystem", nullptr);
@@ -647,6 +715,13 @@ TEST_P(ServerFixture, AddSystemWhileRunning)
   EXPECT_FALSE(result.value());
   EXPECT_EQ(3u, *server.SystemCount());
 
+  // Add system pointer
+  auto mockSystem = std::make_shared<MockSystem>();
+  result = server.AddSystem(mockSystem);
+  ASSERT_TRUE(result.has_value());
+  EXPECT_FALSE(result.value());
+  EXPECT_EQ(3u, *server.SystemCount());
+
   // Stop the server
   std::raise(SIGTERM);
 
@@ -655,7 +730,7 @@ TEST_P(ServerFixture, AddSystemWhileRunning)
 }
 
 /////////////////////////////////////////////////
-TEST_P(ServerFixture, AddSystemAfterLoad)
+TEST_P(ServerFixture, IGN_UTILS_TEST_DISABLED_ON_WIN32(AddSystemAfterLoad))
 {
   ignition::gazebo::ServerConfig serverConfig;
 
@@ -666,28 +741,55 @@ TEST_P(ServerFixture, AddSystemAfterLoad)
   EXPECT_FALSE(server.Running());
   EXPECT_FALSE(*server.Running(0));
 
+  // Add system from plugin
   gazebo::SystemLoader systemLoader;
   auto mockSystemPlugin = systemLoader.LoadPlugin("libMockSystem.so",
       "ignition::gazebo::MockSystem", nullptr);
   ASSERT_TRUE(mockSystemPlugin.has_value());
 
-  EXPECT_EQ(3u, *server.SystemCount());
-  EXPECT_TRUE(*server.AddSystem(mockSystemPlugin.value()));
-  EXPECT_EQ(4u, *server.SystemCount());
-
   auto system = mockSystemPlugin.value()->QueryInterface<gazebo::System>();
   EXPECT_NE(system, nullptr);
   auto mockSystem = dynamic_cast<gazebo::MockSystem*>(system);
-  EXPECT_NE(mockSystem, nullptr);
+  ASSERT_NE(mockSystem, nullptr);
 
+  EXPECT_EQ(3u, *server.SystemCount());
+  EXPECT_EQ(0u, mockSystem->configureCallCount);
+
+  EXPECT_TRUE(*server.AddSystem(mockSystemPlugin.value()));
+
+  EXPECT_EQ(4u, *server.SystemCount());
+  EXPECT_EQ(1u, mockSystem->configureCallCount);
+
+  // Add system pointer
+  auto mockSystemLocal = std::make_shared<MockSystem>();
+  EXPECT_EQ(0u, mockSystemLocal->configureCallCount);
+
+  EXPECT_TRUE(server.AddSystem(mockSystemLocal));
+  EXPECT_EQ(5u, *server.SystemCount());
+  EXPECT_EQ(1u, mockSystemLocal->configureCallCount);
+
+  // Check that update callbacks are called
   server.SetUpdatePeriod(1us);
   EXPECT_EQ(0u, mockSystem->preUpdateCallCount);
   EXPECT_EQ(0u, mockSystem->updateCallCount);
   EXPECT_EQ(0u, mockSystem->postUpdateCallCount);
+  EXPECT_EQ(0u, mockSystemLocal->preUpdateCallCount);
+  EXPECT_EQ(0u, mockSystemLocal->updateCallCount);
+  EXPECT_EQ(0u, mockSystemLocal->postUpdateCallCount);
   server.Run(true, 1, false);
   EXPECT_EQ(1u, mockSystem->preUpdateCallCount);
   EXPECT_EQ(1u, mockSystem->updateCallCount);
   EXPECT_EQ(1u, mockSystem->postUpdateCallCount);
+  EXPECT_EQ(1u, mockSystemLocal->preUpdateCallCount);
+  EXPECT_EQ(1u, mockSystemLocal->updateCallCount);
+  EXPECT_EQ(1u, mockSystemLocal->postUpdateCallCount);
+
+  // Add to inexistent world
+  auto result = server.AddSystem(mockSystemPlugin.value(), 100);
+  EXPECT_FALSE(result.has_value());
+
+  result = server.AddSystem(mockSystemLocal, 100);
+  EXPECT_FALSE(result.has_value());
 }
 
 /////////////////////////////////////////////////
@@ -702,7 +804,7 @@ TEST_P(ServerFixture, Seed)
 }
 
 /////////////////////////////////////////////////
-TEST_P(ServerFixture, ResourcePath)
+TEST_P(ServerFixture, IGN_UTILS_TEST_DISABLED_ON_WIN32(ResourcePath))
 {
   ignition::common::setenv("IGN_GAZEBO_RESOURCE_PATH",
          (std::string(PROJECT_SOURCE_PATH) + "/test/worlds:" +
