@@ -103,8 +103,15 @@ SimulationRunner::SimulationRunner(const sdf::World *_world,
   // So to get a given RTF, our desired period is:
   //
   // period = step_size / RTF
-  this->updatePeriod = std::chrono::nanoseconds(
-      static_cast<int>(this->stepSize.count() / this->desiredRtf));
+  if (this->desiredRtf < 1e-9)
+  {
+    this->updatePeriod = 0ms;
+  }
+  else
+  {
+    this->updatePeriod = std::chrono::nanoseconds(
+        static_cast<int>(this->stepSize.count() / this->desiredRtf));
+  }
 
   this->pauseConn = this->eventMgr.Connect<events::Pause>(
       std::bind(&SimulationRunner::SetPaused, this, std::placeholders::_1));
@@ -355,20 +362,32 @@ void SimulationRunner::UpdatePhysicsParams()
   if (newStepSize != this->stepSize ||
       std::abs(newRTF - this->desiredRtf) > eps)
   {
-    this->SetStepSize(
-      std::chrono::duration_cast<std::chrono::steady_clock::duration>(
-        newStepSize));
-    this->desiredRtf = newRTF;
-    this->updatePeriod = std::chrono::nanoseconds(
-        static_cast<int>(this->stepSize.count() / this->desiredRtf));
-
-    this->simTimes.clear();
-    this->realTimes.clear();
-    // Update physics components
-    physicsComp->Data().SetMaxStepSize(physicsParams.max_step_size());
-    physicsComp->Data().SetRealTimeFactor(newRTF);
-    this->entityCompMgr.SetChanged(worldEntity, components::Physics::typeId,
-        ComponentState::OneTimeChange);
+    bool updated = false;
+    // Make sure the values are valid before setting them
+    if (newStepSize.count() > 0.0)
+    {
+      this->SetStepSize(
+        std::chrono::duration_cast<std::chrono::steady_clock::duration>(
+          newStepSize));
+      physicsComp->Data().SetMaxStepSize(physicsParams.max_step_size());
+      updated = true;
+    }
+    if (newRTF > 0.0)
+    {
+      this->desiredRtf = newRTF;
+      this->updatePeriod = std::chrono::nanoseconds(
+          static_cast<int>(this->stepSize.count() / this->desiredRtf));
+      physicsComp->Data().SetRealTimeFactor(newRTF);
+      updated = true;
+    }
+    if (updated)
+    {
+      this->simTimes.clear();
+      this->realTimes.clear();
+      // Set as OneTimeChange to make sure the update is not missed
+      this->entityCompMgr.SetChanged(worldEntity, components::Physics::typeId,
+          ComponentState::OneTimeChange);
+    }
   }
   this->entityCompMgr.RemoveComponent<components::PhysicsCmd>(worldEntity);
 }
@@ -421,28 +440,61 @@ void SimulationRunner::PublishStats()
     this->rootClockPub.Publish(clockMsg);
 }
 
-/////////////////////////////////////////////////
-void SimulationRunner::AddSystem(const SystemPluginPtr &_system)
+//////////////////////////////////////////////////
+void SimulationRunner::AddSystem(const SystemPluginPtr &_system,
+      std::optional<Entity> _entity,
+      std::optional<std::shared_ptr<const sdf::Element>> _sdf)
 {
+  this->AddSystemImpl(SystemInternal(_system), _entity, _sdf);
+}
+
+//////////////////////////////////////////////////
+void SimulationRunner::AddSystem(
+      const std::shared_ptr<System> &_system,
+      std::optional<Entity> _entity,
+      std::optional<std::shared_ptr<const sdf::Element>> _sdf)
+{
+  this->AddSystemImpl(SystemInternal(_system), _entity, _sdf);
+}
+
+//////////////////////////////////////////////////
+void SimulationRunner::AddSystemImpl(
+      SystemInternal _system,
+      std::optional<Entity> _entity,
+      std::optional<std::shared_ptr<const sdf::Element>> _sdf)
+{
+  // Call configure
+  if (_system.configure)
+  {
+    // Default to world entity and SDF
+    auto entity = _entity.has_value() ? _entity.value()
+        : worldEntity(this->entityCompMgr);
+    auto sdf = _sdf.has_value() ? _sdf.value() : this->sdfWorld->Element();
+
+    _system.configure->Configure(
+        entity, sdf,
+        this->entityCompMgr,
+        this->eventMgr);
+  }
+
+  // Update callbacks will be handled later, add to queue
   std::lock_guard<std::mutex> lock(this->pendingSystemsMutex);
   this->pendingSystems.push_back(_system);
 }
 
 /////////////////////////////////////////////////
-void SimulationRunner::AddSystemToRunner(const SystemPluginPtr &_system)
+void SimulationRunner::AddSystemToRunner(SystemInternal _system)
 {
-  this->systems.push_back(SystemInternal(_system));
+  this->systems.push_back(_system);
 
-  const auto &system = this->systems.back();
+  if (_system.preupdate)
+    this->systemsPreupdate.push_back(_system.preupdate);
 
-  if (system.preupdate)
-    this->systemsPreupdate.push_back(system.preupdate);
+  if (_system.update)
+    this->systemsUpdate.push_back(_system.update);
 
-  if (system.update)
-    this->systemsUpdate.push_back(system.update);
-
-  if (system.postupdate)
-    this->systemsPostupdate.push_back(system.postupdate);
+  if (_system.postupdate)
+    this->systemsPostupdate.push_back(_system.postupdate);
 }
 
 /////////////////////////////////////////////////
@@ -461,7 +513,6 @@ void SimulationRunner::ProcessSystemQueue()
   {
     this->AddSystemToRunner(system);
   }
-
   this->pendingSystems.clear();
 
   // If additional systems were added, recreate the worker threads.
@@ -471,9 +522,9 @@ void SimulationRunner::ProcessSystemQueue()
       << this->systemsPostupdate.size() + 1 << std::endl;
 
     this->postUpdateStartBarrier =
-      std::make_unique<Barrier>(this->systemsPostupdate.size() + 1);
+      std::make_unique<Barrier>(this->systemsPostupdate.size() + 1u);
     this->postUpdateStopBarrier =
-      std::make_unique<Barrier>(this->systemsPostupdate.size() + 1);
+      std::make_unique<Barrier>(this->systemsPostupdate.size() + 1u);
 
     this->postUpdateThreadsRunning = true;
     int id = 0;
@@ -824,18 +875,10 @@ void SimulationRunner::LoadPlugin(const Entity _entity,
     system = this->systemLoader->LoadPlugin(_fname, _name, _sdf);
   }
 
-  // System correctly loaded from library, try to configure
+  // System correctly loaded from library
   if (system)
   {
-    auto systemConfig = system.value()->QueryInterface<ISystemConfigure>();
-    if (systemConfig != nullptr)
-    {
-      systemConfig->Configure(_entity, _sdf,
-          this->entityCompMgr,
-          this->eventMgr);
-    }
-
-    this->AddSystem(system.value());
+    this->AddSystem(system.value(), _entity, _sdf);
     igndbg << "Loaded system [" << _name
            << "] for entity [" << _entity << "]" << std::endl;
   }
@@ -952,7 +995,7 @@ void SimulationRunner::LoadLoggingPlugins(const ServerConfig &_config)
 void SimulationRunner::LoadPlugins(const Entity _entity,
     const sdf::ElementPtr &_sdf)
 {
-  sdf::ElementPtr pluginElem = _sdf->GetElement("plugin");
+  sdf::ElementPtr pluginElem = _sdf->FindElement("plugin");
   while (pluginElem)
   {
     auto filename = pluginElem->Get<std::string>("filename");
@@ -960,8 +1003,7 @@ void SimulationRunner::LoadPlugins(const Entity _entity,
     // No error message for the 'else' case of the following 'if' statement
     // because SDF create a default <plugin> element even if it's not
     // specified. An error message would result in spamming
-    // the console. \todo(nkoenig) Fix SDF should so that elements are not
-    // automatically added.
+    // the console.
     if (filename != "__default__" && name != "__default__")
     {
       this->LoadPlugin(_entity, filename, name, pluginElem);
