@@ -28,23 +28,21 @@
 #include "ignition/gazebo/components/Model.hh"
 #include "ignition/gazebo/components/Name.hh"
 #include "ignition/gazebo/components/ParentEntity.hh"
+#include "ignition/gazebo/components/ParticleEmitter.hh"
 #include "ignition/gazebo/components/Sensor.hh"
 #include "ignition/gazebo/components/Visual.hh"
 #include "ignition/gazebo/components/World.hh"
 #include "ignition/gazebo/EntityComponentManager.hh"
 #include "ignition/gazebo/Util.hh"
 
+#include "helpers/EnvTestFixture.hh"
+
 using namespace ignition;
 using namespace gazebo;
 
 /// \brief Tests for Util.hh
-class UtilTest : public ::testing::Test
+class UtilTest : public InternalFixture<::testing::Test>
 {
-  // Documentation inherited
-  protected: void SetUp() override
-  {
-    common::Console::SetVerbosity(4);
-  }
 };
 
 /////////////////////////////////////////////////
@@ -69,6 +67,7 @@ TEST_F(UtilTest, ScopedName)
 
   // World
   auto worldEntity = ecm.CreateEntity();
+  EXPECT_EQ(kNullEntity, gazebo::worldEntity(ecm));
   EXPECT_EQ(kNullEntity, gazebo::worldEntity(worldEntity, ecm));
   ecm.CreateComponent(worldEntity, components::World());
   ecm.CreateComponent(worldEntity, components::Name("world_name"));
@@ -212,6 +211,7 @@ TEST_F(UtilTest, ScopedName)
     "world_name::actorD_name");
 
   // World entity
+  EXPECT_EQ(worldEntity, gazebo::worldEntity(ecm));
   EXPECT_EQ(worldEntity, gazebo::worldEntity(worldEntity, ecm));
   EXPECT_EQ(worldEntity, gazebo::worldEntity(lightAEntity, ecm));
   EXPECT_EQ(worldEntity, gazebo::worldEntity(modelBEntity, ecm));
@@ -229,6 +229,88 @@ TEST_F(UtilTest, ScopedName)
   EXPECT_EQ(kNullEntity, gazebo::worldEntity(kNullEntity, ecm));
 }
 
+/////////////////////////////////////////////////
+TEST_F(UtilTest, EntitiesFromScopedName)
+{
+  EntityComponentManager ecm;
+
+  // banana 1
+  //  - orange 2
+  //    - plum 3
+  //      - grape 4
+  //        - pear 5
+  //          - plum 6
+  //  - grape 7
+  //    - pear 8
+  //      - plum 9
+  //        - pear 10
+  //  - grape 11
+  //    - pear 12
+  //      - orange 13
+  //        - orange 14
+  //    - pear 15
+
+  auto createEntity = [&ecm](const std::string &_name, Entity _parent) -> Entity
+  {
+    auto res = ecm.CreateEntity();
+    ecm.CreateComponent(res, components::Name(_name));
+    ecm.CreateComponent(res, components::ParentEntity(_parent));
+    return res;
+  };
+
+  auto banana1 = ecm.CreateEntity();
+  ecm.CreateComponent(banana1, components::Name("banana"));
+
+  auto orange2 = createEntity("orange", banana1);
+  auto plum3 = createEntity("plum", orange2);
+  auto grape4 = createEntity("grape", plum3);
+  auto pear5 = createEntity("pear", grape4);
+  auto plum6 = createEntity("plum", pear5);
+  auto grape7 = createEntity("grape", banana1);
+  auto pear8 = createEntity("pear", grape7);
+  auto plum9 = createEntity("plum", pear8);
+  auto pear10 = createEntity("pear", plum9);
+  auto grape11 = createEntity("grape", banana1);
+  auto pear12 = createEntity("pear", grape11);
+  auto orange13 = createEntity("orange", pear12);
+  auto orange14 = createEntity("orange", orange13);
+  auto pear15 = createEntity("pear", grape11);
+
+  auto checkEntities = [&ecm](const std::string &_scopedName,
+      Entity _relativeTo, const std::unordered_set<Entity> &_result,
+      const std::string &_delim)
+  {
+    auto res = gazebo::entitiesFromScopedName(_scopedName, ecm, _relativeTo,
+        _delim);
+    EXPECT_EQ(_result.size(), res.size()) << _scopedName;
+
+    for (auto it : _result)
+    {
+      EXPECT_NE(res.find(it), res.end()) << it << "  " << _scopedName;
+    }
+  };
+
+  checkEntities("watermelon", kNullEntity, {}, "::");
+  checkEntities("banana", kNullEntity, {banana1}, "::");
+  checkEntities("orange", kNullEntity, {orange2, orange13, orange14}, ":");
+  checkEntities("banana::orange", kNullEntity, {orange2}, "::");
+  checkEntities("banana::grape", kNullEntity, {grape7, grape11}, "::");
+  checkEntities("grape/pear", kNullEntity, {pear5, pear8, pear12, pear15}, "/");
+  checkEntities("grape...pear...plum", kNullEntity, {plum6, plum9}, "...");
+  checkEntities(
+      "banana::orange::plum::grape::pear::plum", kNullEntity, {plum6}, "::");
+  checkEntities(
+      "banana::orange::kiwi::grape::pear::plum", kNullEntity, {}, "::");
+  checkEntities("orange+orange", kNullEntity, {orange14}, "+");
+  checkEntities("orange", banana1, {orange2}, "::");
+  checkEntities("grape", banana1, {grape7, grape11}, "::");
+  checkEntities("orange", orange2, {}, "::");
+  checkEntities("orange", orange13, {orange14}, "::");
+  checkEntities("grape::pear::plum", plum3, {plum6}, "::");
+  checkEntities("pear", grape11, {pear12, pear15}, "==");
+  checkEntities("plum=pear", pear8, {pear10}, "=");
+}
+
 /////////////////////////////////////////////////
 TEST_F(UtilTest, EntityTypeId)
 {
@@ -272,6 +354,10 @@ TEST_F(UtilTest, EntityTypeId)
   entity = ecm.CreateEntity();
   ecm.CreateComponent(entity, components::Actor());
   EXPECT_EQ(components::Actor::typeId, entityTypeId(entity, ecm));
+
+  entity = ecm.CreateEntity();
+  ecm.CreateComponent(entity, components::ParticleEmitter());
+  EXPECT_EQ(components::ParticleEmitter::typeId, entityTypeId(entity, ecm));
 }
 
 /////////////////////////////////////////////////
@@ -317,6 +403,10 @@ TEST_F(UtilTest, EntityTypeStr)
   entity = ecm.CreateEntity();
   ecm.CreateComponent(entity, components::Actor());
   EXPECT_EQ("actor", entityTypeStr(entity, ecm));
+
+  entity = ecm.CreateEntity();
+  ecm.CreateComponent(entity, components::ParticleEmitter());
+  EXPECT_EQ("particle_emitter", entityTypeStr(entity, ecm));
 }
 
 /////////////////////////////////////////////////
@@ -510,3 +600,118 @@ TEST_F(UtilTest, ValidTopic)
   EXPECT_EQ("not_bad", validTopic({fixable, invalid, good}));
   EXPECT_EQ("good", validTopic({invalid, good, fixable}));
 }
+
+/////////////////////////////////////////////////
+TEST_F(UtilTest, TopicFromScopedName)
+{
+  EntityComponentManager ecm;
+
+  // world
+  //  - modelA
+  //    - linkA
+  //    - modelB
+  //      - linkB
+  //        - emitterB
+  //  - modelC
+
+  // World
+  auto worldEntity = ecm.CreateEntity();
+  ecm.CreateComponent(worldEntity, components::World());
+  ecm.CreateComponent(worldEntity, components::Name("world_name"));
+
+  // Model A
+  auto modelAEntity = ecm.CreateEntity();
+  ecm.CreateComponent(modelAEntity, components::Model());
+  ecm.CreateComponent(modelAEntity, components::Name("modelA_name"));
+  ecm.CreateComponent(modelAEntity, components::ParentEntity(worldEntity));
+
+  // Link A - Child of Model A
+  auto linkAEntity = ecm.CreateEntity();
+  ecm.CreateComponent(linkAEntity, components::Link());
+  ecm.CreateComponent(linkAEntity, components::Name("linkA_name"));
+  ecm.CreateComponent(linkAEntity, components::ParentEntity(modelAEntity));
+
+  // Model B - nested inside Model A
+  auto modelBEntity = ecm.CreateEntity();
+  ecm.CreateComponent(modelBEntity, components::Model());
+  ecm.CreateComponent(modelBEntity, components::Name("modelB_name"));
+  ecm.CreateComponent(modelBEntity, components::ParentEntity(modelAEntity));
+
+  // Link B - child of Model B
+  auto linkBEntity = ecm.CreateEntity();
+  ecm.CreateComponent(linkBEntity, components::Link());
+  ecm.CreateComponent(linkBEntity, components::Name("linkB_name"));
+  ecm.CreateComponent(linkBEntity, components::ParentEntity(modelBEntity));
+
+  // Emitter B - child of Link B
+  auto emitterBEntity = ecm.CreateEntity();
+  ecm.CreateComponent(emitterBEntity, components::ParticleEmitter());
+  ecm.CreateComponent(emitterBEntity, components::Name("emitterB_name"));
+  ecm.CreateComponent(emitterBEntity, components::ParentEntity(linkBEntity));
+
+  // Model C
+  auto modelCEntity = ecm.CreateEntity();
+  ecm.CreateComponent(modelCEntity, components::Model());
+  ecm.CreateComponent(modelCEntity, components::Name("modelC_name"));
+  ecm.CreateComponent(modelCEntity, components::ParentEntity(worldEntity));
+
+  std::string testName = "/model/modelA_name";
+  std::string worldName = "/world/world_name";
+  // model A, link A, model B, link B and visual B should have
+  // model A as the top level model
+  EXPECT_EQ(testName, topicFromScopedName(modelAEntity, ecm));
+  EXPECT_EQ(worldName + testName,
+      topicFromScopedName(modelAEntity, ecm, false));
+
+  testName += "/link/linkA_name";
+  EXPECT_EQ(testName, topicFromScopedName(linkAEntity, ecm));
+  EXPECT_EQ(worldName + testName, topicFromScopedName(linkAEntity, ecm, false));
+
+  testName = "/model/modelA_name/model/modelB_name";
+  EXPECT_EQ(testName, topicFromScopedName(modelBEntity, ecm));
+  EXPECT_EQ(worldName + testName,
+      topicFromScopedName(modelBEntity, ecm, false));
+
+  testName +="/link/linkB_name";
+  EXPECT_EQ(testName, topicFromScopedName(linkBEntity, ecm));
+  EXPECT_EQ(worldName + testName, topicFromScopedName(linkBEntity, ecm, false));
+
+  testName += "/particle_emitter/emitterB_name";
+  EXPECT_EQ(testName,
+      topicFromScopedName(emitterBEntity, ecm));
+  EXPECT_EQ(worldName + testName,
+      topicFromScopedName(emitterBEntity, ecm, false));
+
+  testName = "/model/modelC_name";
+  EXPECT_EQ(testName, topicFromScopedName(modelCEntity, ecm));
+  EXPECT_EQ(worldName + testName,
+      topicFromScopedName(modelCEntity, ecm, false));
+
+  EXPECT_TRUE(topicFromScopedName(worldEntity, ecm).empty());
+  EXPECT_EQ(worldName, topicFromScopedName(worldEntity, ecm, false));
+}
+
+/////////////////////////////////////////////////
+TEST_F(UtilTest, EnableComponent)
+{
+  EntityComponentManager ecm;
+
+  auto entity1 = ecm.CreateEntity();
+  EXPECT_EQ(nullptr, ecm.Component<components::Name>(entity1));
+
+  // Enable
+  EXPECT_TRUE(enableComponent<components::Name>(ecm, entity1));
+  EXPECT_NE(nullptr, ecm.Component<components::Name>(entity1));
+
+  // Enabling again makes no changes
+  EXPECT_FALSE(enableComponent<components::Name>(ecm, entity1, true));
+  EXPECT_NE(nullptr, ecm.Component<components::Name>(entity1));
+
+  // Disable
+  EXPECT_TRUE(enableComponent<components::Name>(ecm, entity1, false));
+  EXPECT_EQ(nullptr, ecm.Component<components::Name>(entity1));
+
+  // Disabling again makes no changes
+  EXPECT_FALSE(enableComponent<components::Name>(ecm, entity1, false));
+  EXPECT_EQ(nullptr, ecm.Component<components::Name>(entity1));
+}
