@@ -21,28 +21,28 @@
 #include <vector>
 #include <map>
 
-#include <gz/msgs/serialized.pb.h>
-#include <gz/msgs/stringmsg.pb.h>
+#include <ignition/msgs/serialized.pb.h>
+#include <ignition/msgs/stringmsg.pb.h>
 
-#include <gz/common/Console.hh>
-#include <gz/common/Filesystem.hh>
-#include <gz/sim/EntityComponentManager.hh>
-#include <gz/sim/components/ChildLinkName.hh>
-#include <gz/sim/components/Inertial.hh>
-#include <gz/sim/components/Joint.hh>
-#include <gz/sim/components/JointAxis.hh>
-#include <gz/sim/components/JointType.hh>
-#include <gz/sim/components/Link.hh>
-#include <gz/sim/components/Model.hh>
-#include <gz/sim/components/Name.hh>
-#include <gz/sim/components/ParentEntity.hh>
-#include <gz/sim/components/ParentLinkName.hh>
-#include <gz/sim/components/Pose.hh>
-#include <gz/sim/components/World.hh>
-#include <gz/transport/Node.hh>
+#include <ignition/common/Console.hh>
+#include <ignition/common/Filesystem.hh>
+#include <ignition/gazebo/EntityComponentManager.hh>
+#include <ignition/gazebo/components/ChildLinkName.hh>
+#include <ignition/gazebo/components/Inertial.hh>
+#include <ignition/gazebo/components/Joint.hh>
+#include <ignition/gazebo/components/JointAxis.hh>
+#include <ignition/gazebo/components/JointType.hh>
+#include <ignition/gazebo/components/Link.hh>
+#include <ignition/gazebo/components/Model.hh>
+#include <ignition/gazebo/components/Name.hh>
+#include <ignition/gazebo/components/ParentEntity.hh>
+#include <ignition/gazebo/components/ParentLinkName.hh>
+#include <ignition/gazebo/components/Pose.hh>
+#include <ignition/gazebo/components/World.hh>
+#include <ignition/transport/Node.hh>
 
-using namespace gz;
-using namespace gz::sim;
+using namespace ignition;
+using namespace gazebo;
 
 //////////////////////////////////////////////////
 /// \brief Get the name of the world being used by calling
@@ -330,7 +330,7 @@ void printJoints(const uint64_t _modelEntity,
 }
 
 //////////////////////////////////////////////////
-extern "C" IGNITION_GAZEBO_VISIBLE void cmdModelList()
+extern "C" void cmdModelList()
 {
   EntityComponentManager ecm{};
   if (!populateECM(ecm))
@@ -364,7 +364,7 @@ extern "C" IGNITION_GAZEBO_VISIBLE void cmdModelList()
 }
 
 //////////////////////////////////////////////////
-extern "C" IGNITION_GAZEBO_VISIBLE void cmdModelInfo(
+extern "C" void cmdModelInfo(
     const char *_modelName, int _pose, const char *_linkName,
     const char *_jointName)
 {
