@@ -17,37 +17,41 @@
 
 #include <gtest/gtest.h>
 
-#include <gz/msgs/stringmsg.pb.h>
+#include <ignition/msgs/stringmsg.pb.h>
 
-#include <gz/common/Console.hh>
-#include <gz/common/StringUtils.hh>
-#include <gz/common/Util.hh>
-#include <gz/gui/Dialog.hh>
-#include <gz/gui/MainWindow.hh>
-#include <gz/transport/Node.hh>
-#include <gz/utilities/ExtraTestMacros.hh>
+#include <ignition/common/Console.hh>
+#include <ignition/common/StringUtils.hh>
+#include <ignition/common/Util.hh>
+#include <ignition/transport/Node.hh>
+#include <ignition/utilities/ExtraTestMacros.hh>
 
-#include "gz/sim/gui/Gui.hh"
-#include "gz/sim/test_config.hh"
-#include "QuickStartHandler.hh"
+#include "ignition/gazebo/gui/Gui.hh"
+#include "ignition/gazebo/test_config.hh"
+
+#include "../../test/helpers/EnvTestFixture.hh"
 
 int gg_argc = 1;
 char **gg_argv = new char *[gg_argc];
 
-using namespace gz;
-using namespace ignition::gazebo::gui;
+using namespace ignition;
+using namespace gazebo;
+
+class GuiTest : public InternalFixture<::testing::Test>
+{
+};
 
 /////////////////////////////////////////////////
-// https://github.com/gazebosim/gz-sim/issues/8
-TEST(GuiTest, IGN_UTILS_TEST_DISABLED_ON_MAC(PathManager))
+// https://github.com/ignitionrobotics/ign-gazebo/issues/8
+// See https://github.com/ignitionrobotics/ign-gazebo/issues/1175
+TEST_F(GuiTest, IGN_UTILS_TEST_ENABLED_ONLY_ON_LINUX(PathManager))
 {
   common::Console::SetVerbosity(4);
   igndbg << "Start test" << std::endl;
 
-  common::setenv("IGN_GAZEBO_RESOURCE_PATH",
+  ignition::common::setenv("IGN_GAZEBO_RESOURCE_PATH",
          "/from_env:/tmp/more_env");
-  common::setenv("SDF_PATH", "");
-  common::setenv("IGN_FILE_PATH", "");
+  ignition::common::setenv("SDF_PATH", "");
+  ignition::common::setenv("IGN_FILE_PATH", "");
   igndbg << "Environment set" << std::endl;
 
   transport::Node node;
@@ -87,7 +91,7 @@ TEST(GuiTest, IGN_UTILS_TEST_DISABLED_ON_MAC(PathManager))
   node.Advertise("/gazebo/resource_paths/get", pathsCb);
   igndbg << "Paths advertised" << std::endl;
 
-  auto app = createGui(gg_argc, gg_argv, nullptr);
+  auto app = ignition::gazebo::gui::createGui(gg_argc, gg_argv, nullptr);
   EXPECT_NE(nullptr, app);
   igndbg << "GUI created" << std::endl;
 
@@ -163,102 +167,3 @@ TEST(GuiTest, IGN_UTILS_TEST_DISABLED_ON_MAC(PathManager))
   }
 }
 
-/////////////////////////////////////////////////
-TEST(GuiTest, IGN_UTILS_TEST_ENABLED_ONLY_ON_LINUX(QuickStart))
-{
-  common::Console::SetVerbosity(4);
-  igndbg << "Start test" << std::endl;
-
-  transport::Node node;
-
-  // Worlds callback
-  bool worldsCalled{false};
-  std::function<bool(msgs::StringMsg_V &)> worldsCb =
-      [&worldsCalled](msgs::StringMsg_V &_res)
-      {
-        _res.add_data("world_name");
-        worldsCalled = true;
-        return true;
-      };
-  node.Advertise("/gazebo/worlds", worldsCb);
-  igndbg << "Worlds advertised" << std::endl;
-
-  // Starting world callback
-  bool startingWorldCalled{false};
-  std::function<void(const msgs::StringMsg &)> topicCb =
-      [&startingWorldCalled](const auto &_msg)
-      {
-        EXPECT_EQ("banana", _msg.data());
-        startingWorldCalled = true;
-      };
-
-  std::string topic{"/gazebo/starting_world"};
-  node.Subscribe(topic, topicCb);
-  igndbg << "Subscribed to [" << topic << "]" << std::endl;
-
-  // Custom config
-  // TODO(chapulina) Make it not Linux-specific
-  std::string configFilePath{"/tmp/quick_start_test.config"};
-  std::ofstream configFile(configFilePath);
-  configFile << "<window><dialog_on_exit>false</dialog_on_exit></window>" <<
-      "<plugin filename='Publisher' name='Publisher'/>";
-  configFile.close();
-
-  // Thread to check and close quick start dialog
-  std::thread checkingThread([&]()
-  {
-    igndbg << "Started checking thread" << std::endl;
-    for (int sleep = 0;
-        (nullptr == gui::App( ) ||
-         gui::App()->allWindows().empty() ||
-         !gui::App()->allWindows()[0]->isVisible())
-        && sleep < 30; ++sleep)
-    {
-      std::this_thread::sleep_for(std::chrono::milliseconds(100));
-    }
-    ASSERT_NE(nullptr, gui::App());
-    ASSERT_EQ(1, gui::App()->allWindows().count());
-    igndbg << "Found app" << std::endl;
-
-    auto handler = gui::App()->Engine()->findChild<QuickStartHandler *>();
-    ASSERT_NE(nullptr, handler);
-
-    EXPECT_EQ(GZ_DISTRIBUTION, handler->Distribution());
-    EXPECT_EQ(IGNITION_GAZEBO_VERSION_FULL, handler->SimVersion());
-    EXPECT_TRUE(handler->ShowAgain());
-
-    handler->SetStartingWorld("banana");
-    EXPECT_EQ("banana", handler->StartingWorld());
-
-    gui::App()->allWindows()[0]->close();
-
-    // Close main window
-    for (int sleep = 0;
-        (nullptr == gui::App()->findChild<gui::MainWindow *>() ||
-        !gui::App()->findChild<gui::MainWindow *>()->QuickWindow()->isVisible())
-        && sleep < 30; ++sleep)
-    {
-      std::this_thread::sleep_for(std::chrono::milliseconds(100));
-    }
-    auto win = gui::App()->findChild<gui::MainWindow *>();
-    ASSERT_NE(nullptr, win);
-    EXPECT_TRUE(win->QuickWindow()->isVisible());
-    win->QuickWindow()->close();
-  });
-
-  auto app = createGui(gg_argc, gg_argv,
-      configFilePath.c_str() /* _guiConfig */,
-      nullptr /* _defaultGuiConfig */,
-      true /* _loadPluginsFromSdf */,
-      nullptr /* _sdfFile */,
-      true /* _waitGui */);
-  EXPECT_NE(nullptr, app);
-  igndbg << "GUI created" << std::endl;
-
-  EXPECT_TRUE(worldsCalled);
-  EXPECT_TRUE(startingWorldCalled);
-
-  app->exec();
-  checkingThread.join();
-}
-
