@@ -25,25 +25,27 @@
 #include <string>
 #include <vector>
 
+#include <ignition/common/Console.hh>
 #include <ignition/gui/Application.hh>
 #include <ignition/gui/GuiEvents.hh>
 #include <ignition/gui/MainWindow.hh>
-#include <ignition/common/Console.hh>
 #include <ignition/plugin/Register.hh>
-#include <ignition/transport/Node.hh>
-#include <ignition/rendering/Visual.hh>
 #include <ignition/rendering/Geometry.hh>
 #include <ignition/rendering/Material.hh>
+#include <ignition/rendering/RenderEngine.hh>
 #include <ignition/rendering/RenderTypes.hh>
 #include <ignition/rendering/RenderingIface.hh>
-#include <ignition/rendering/RenderEngine.hh>
 #include <ignition/rendering/Scene.hh>
+#include <ignition/rendering/Visual.hh>
 #include <ignition/rendering/WireBox.hh>
-#include "ignition/gazebo/components/World.hh"
-#include "ignition/gazebo/components/Name.hh"
+#include <ignition/transport/Node.hh>
 
 #include "ignition/gazebo/EntityComponentManager.hh"
+#include "ignition/gazebo/components/Name.hh"
+#include "ignition/gazebo/components/World.hh"
 #include "ignition/gazebo/gui/GuiEvents.hh"
+#include "ignition/gazebo/rendering/RenderUtil.hh"
+
 #include "AlignTool.hh"
 
 namespace ignition::gazebo
@@ -89,6 +91,9 @@ namespace ignition::gazebo
 
     /// \brief The map of the original transparency values for the nodes.
     public: std::map<std::string, double> originalTransparency;
+
+    /// \brief Pointer to the scene.
+    public: rendering::ScenePtr scene{nullptr};
   };
 }
 
@@ -342,52 +347,9 @@ void AlignTool::Align()
   if (this->dataPtr->currentState == AlignState::NONE)
     return;
 
-  auto loadedEngNames = rendering::loadedEngines();
-  if (loadedEngNames.empty())
-  {
-    ignerr << "Internal error: engine should be loaded at this point."
-      << std::endl;
-    return;
-  }
-
-  // Assume there is only one engine loaded
-  auto engineName = loadedEngNames[0];
-  if (loadedEngNames.size() > 1)
-  {
-    ignwarn << "Found more than one loaded engine "
-      "- using " << engineName << "." << std::endl;
-  }
-  auto engine = rendering::engine(engineName);
-
-  if (!engine)
-  {
-    ignerr << "Internal error: failed to load engine [" << engineName
-      << "]. Align tool plugin won't work." << std::endl;
-    return;
-  }
-
-  if (engine->SceneCount() == 0)
-  {
-    ignerr<< "Internal error: no scenes are available with the loaded engine."
-      << std::endl;
-    return;
-  }
-  // assume there is only one scene
   // load scene
-  auto scene = engine->SceneByIndex(0);
-
-  if (!scene)
-  {
-    ignerr << "Internal error: scene is null." << std::endl;
-    return;
-  }
-
-  if (!scene->IsInitialized() || scene->VisualCount() == 0)
-  {
-    ignerr << "Internal error: scene is either not initialized "
-      "or there are no visuals within it." << std::endl;
-    return;
-  }
+  if (!this->dataPtr->scene)
+    this->dataPtr->scene = rendering::sceneFromFirstRenderEngine();
 
   // Get current list of selected entities
   std::vector<ignition::rendering::VisualPtr> selectedList;
@@ -395,9 +357,10 @@ void AlignTool::Align()
 
   for (const auto &entityId : this->dataPtr->selectedEntities)
   {
-    for (auto i = 0u; i < scene->VisualCount(); ++i)
+    for (auto i = 0u; i < this->dataPtr->scene->VisualCount(); ++i)
     {
-      ignition::rendering::VisualPtr vis = scene->VisualByIndex(i);
+      ignition::rendering::VisualPtr vis =
+        this->dataPtr->scene->VisualByIndex(i);
       if (!vis)
         continue;
 
@@ -405,7 +368,7 @@ void AlignTool::Align()
           static_cast<int>(entityId))
       {
         // Check here to see if visual is top level or not, continue if not
-        rendering::VisualPtr topLevelVis = this->TopLevelVisual(scene, vis);
+        auto topLevelVis = this->TopLevelVisual(this->dataPtr->scene, vis);
         if (topLevelVis != vis)
           continue;
 
@@ -459,7 +422,8 @@ void AlignTool::Align()
     ignition::math::Vector3d max = box.Max();
 
     // Check here to see if visual is top level or not, continue if not
-    rendering::VisualPtr topLevelVis = this->TopLevelVisual(scene, vis);
+    rendering::VisualPtr topLevelVis = this->TopLevelVisual(
+        this->dataPtr->scene, vis);
     if (topLevelVis != vis)
       continue;
 
