@@ -17,22 +17,25 @@
 
 #include <iostream>
 #include <regex>
+#include <QColorDialog>
 #include <ignition/common/Console.hh>
 #include <ignition/common/Profiler.hh>
 #include <ignition/gui/Application.hh>
 #include <ignition/gui/MainWindow.hh>
 #include <ignition/plugin/Register.hh>
-#include <ignition/transport/Node.hh>
 
 #include "ignition/gazebo/components/Actor.hh"
 #include "ignition/gazebo/components/AngularAcceleration.hh"
 #include "ignition/gazebo/components/AngularVelocity.hh"
+#include "ignition/gazebo/components/BatterySoC.hh"
 #include "ignition/gazebo/components/CastShadows.hh"
+#include "ignition/gazebo/components/CenterOfVolume.hh"
 #include "ignition/gazebo/components/ChildLinkName.hh"
 #include "ignition/gazebo/components/Collision.hh"
 #include "ignition/gazebo/components/Factory.hh"
 #include "ignition/gazebo/components/Gravity.hh"
 #include "ignition/gazebo/components/Joint.hh"
+#include "ignition/gazebo/components/LaserRetro.hh"
 #include "ignition/gazebo/components/Level.hh"
 #include "ignition/gazebo/components/Light.hh"
 #include "ignition/gazebo/components/LightCmd.hh"
@@ -42,6 +45,7 @@
 #include "ignition/gazebo/components/LinearVelocitySeed.hh"
 #include "ignition/gazebo/components/Link.hh"
 #include "ignition/gazebo/components/MagneticField.hh"
+#include "ignition/gazebo/components/Material.hh"
 #include "ignition/gazebo/components/Model.hh"
 #include "ignition/gazebo/components/Name.hh"
 #include "ignition/gazebo/components/ParentEntity.hh"
@@ -49,19 +53,24 @@
 #include "ignition/gazebo/components/Performer.hh"
 #include "ignition/gazebo/components/PerformerAffinity.hh"
 #include "ignition/gazebo/components/Physics.hh"
-#include "ignition/gazebo/components/Pose.hh"
-#include "ignition/gazebo/components/PoseCmd.hh"
+#include "ignition/gazebo/components/PhysicsEnginePlugin.hh"
+#include "ignition/gazebo/components/RenderEngineGuiPlugin.hh"
+#include "ignition/gazebo/components/RenderEngineServerPlugin.hh"
 #include "ignition/gazebo/components/SelfCollide.hh"
 #include "ignition/gazebo/components/Sensor.hh"
 #include "ignition/gazebo/components/SourceFilePath.hh"
 #include "ignition/gazebo/components/Static.hh"
+#include "ignition/gazebo/components/ThreadPitch.hh"
+#include "ignition/gazebo/components/Transparency.hh"
 #include "ignition/gazebo/components/Visual.hh"
+#include "ignition/gazebo/components/Volume.hh"
 #include "ignition/gazebo/components/WindMode.hh"
 #include "ignition/gazebo/components/World.hh"
 #include "ignition/gazebo/EntityComponentManager.hh"
 #include "ignition/gazebo/gui/GuiEvents.hh"
 
 #include "ComponentInspector.hh"
+#include "Pose3d.hh"
 
 namespace ignition::gazebo
 {
@@ -96,32 +105,26 @@ namespace ignition::gazebo
 
     /// \brief Transport node for making command requests
     public: transport::Node node;
+
+    /// \brief A map of component types to the function used to update it.
+    public: std::map<ComponentTypeId, inspector::UpdateViewCb>
+        updateViewCbs;
+
+    /// \brief Handles all components displayed as a 3D pose.
+    public: std::unique_ptr<inspector::Pose3d> pose3d;
   };
 }
 
 using namespace ignition;
 using namespace gazebo;
 
-//////////////////////////////////////////////////
-template<>
-void ignition::gazebo::setData(QStandardItem *_item, const math::Pose3d &_data)
-{
-  _item->setData(QString("Pose3d"),
-      ComponentsModel::RoleNames().key("dataType"));
-  _item->setData(QList({
-    QVariant(_data.Pos().X()),
-    QVariant(_data.Pos().Y()),
-    QVariant(_data.Pos().Z()),
-    QVariant(_data.Rot().Roll()),
-    QVariant(_data.Rot().Pitch()),
-    QVariant(_data.Rot().Yaw())
-  }), ComponentsModel::RoleNames().key("data"));
-}
-
 //////////////////////////////////////////////////
 template<>
 void ignition::gazebo::setData(QStandardItem *_item, const msgs::Light &_data)
 {
+  if (nullptr == _item)
+    return;
+
   int lightType = -1;
   if (_data.type() == msgs::Light::POINT)
   {
@@ -136,6 +139,36 @@ void ignition::gazebo::setData(QStandardItem *_item, const msgs::Light &_data)
     lightType = 2;
   }
 
+  bool visualizeVisual = true;
+  for (int i = 0; i < _data.header().data_size(); ++i)
+  {
+    for (int j = 0;
+        j < _data.header().data(i).value_size(); ++j)
+    {
+      if (_data.header().data(i).key() ==
+          "visualizeVisual")
+      {
+        visualizeVisual = ignition::math::parseInt(
+          _data.header().data(i).value(0));
+      }
+    }
+  }
+
+  bool isLightOn = true;
+  for (int i = 0; i < _data.header().data_size(); ++i)
+  {
+    for (int j = 0;
+        j < _data.header().data(i).value_size(); ++j)
+    {
+      if (_data.header().data(i).key() ==
+          "isLightOn")
+      {
+        isLightOn = ignition::math::parseInt(
+          _data.header().data(i).value(0));
+      }
+    }
+  }
+
   _item->setData(QString("Light"),
       ComponentsModel::RoleNames().key("dataType"));
   _item->setData(QList({
@@ -159,7 +192,9 @@ void ignition::gazebo::setData(QStandardItem *_item, const msgs::Light &_data)
     QVariant(_data.spot_outer_angle()),
     QVariant(_data.spot_falloff()),
     QVariant(_data.intensity()),
-    QVariant(lightType)
+    QVariant(lightType),
+    QVariant(isLightOn),
+    QVariant(visualizeVisual)
   }), ComponentsModel::RoleNames().key("data"));
 }
 
@@ -168,6 +203,9 @@ template<>
 void ignition::gazebo::setData(QStandardItem *_item,
     const math::Vector3d &_data)
 {
+  if (nullptr == _item)
+    return;
+
   _item->setData(QString("Vector3d"),
       ComponentsModel::RoleNames().key("dataType"));
   _item->setData(QList({
@@ -181,6 +219,9 @@ void ignition::gazebo::setData(QStandardItem *_item,
 template<>
 void ignition::gazebo::setData(QStandardItem *_item, const std::string &_data)
 {
+  if (nullptr == _item)
+    return;
+
   _item->setData(QString("String"),
       ComponentsModel::RoleNames().key("dataType"));
   _item->setData(QString::fromStdString(_data),
@@ -192,6 +233,9 @@ template<>
 void ignition::gazebo::setData(QStandardItem *_item,
     const std::ostringstream &_data)
 {
+  if (nullptr == _item)
+    return;
+
   _item->setData(QString("Raw"),
       ComponentsModel::RoleNames().key("dataType"));
   _item->setData(QString::fromStdString(_data.str()),
@@ -202,6 +246,9 @@ void ignition::gazebo::setData(QStandardItem *_item,
 template<>
 void ignition::gazebo::setData(QStandardItem *_item, const bool &_data)
 {
+  if (nullptr == _item)
+    return;
+
   _item->setData(QString("Boolean"),
       ComponentsModel::RoleNames().key("dataType"));
   _item->setData(_data, ComponentsModel::RoleNames().key("data"));
@@ -211,15 +258,28 @@ void ignition::gazebo::setData(QStandardItem *_item, const bool &_data)
 template<>
 void ignition::gazebo::setData(QStandardItem *_item, const int &_data)
 {
+  if (nullptr == _item)
+    return;
+
   _item->setData(QString("Integer"),
       ComponentsModel::RoleNames().key("dataType"));
   _item->setData(_data, ComponentsModel::RoleNames().key("data"));
 }
 
+//////////////////////////////////////////////////
+template<>
+void ignition::gazebo::setData(QStandardItem *_item, const Entity &_data)
+{
+  setData(_item, static_cast<int>(_data));
+}
+
 //////////////////////////////////////////////////
 template<>
 void ignition::gazebo::setData(QStandardItem *_item, const double &_data)
 {
+  if (nullptr == _item)
+    return;
+
   _item->setData(QString("Float"),
       ComponentsModel::RoleNames().key("dataType"));
   _item->setData(_data, ComponentsModel::RoleNames().key("data"));
@@ -229,6 +289,9 @@ void ignition::gazebo::setData(QStandardItem *_item, const double &_data)
 template<>
 void ignition::gazebo::setData(QStandardItem *_item, const sdf::Physics &_data)
 {
+  if (nullptr == _item)
+    return;
+
   _item->setData(QString("Physics"),
       ComponentsModel::RoleNames().key("dataType"));
   _item->setData(QList({
@@ -237,9 +300,45 @@ void ignition::gazebo::setData(QStandardItem *_item, const sdf::Physics &_data)
   }), ComponentsModel::RoleNames().key("data"));
 }
 
+//////////////////////////////////////////////////
+template<>
+void ignition::gazebo::setData(QStandardItem *_item,
+    const sdf::Material &_data)
+{
+  if (nullptr == _item)
+    return;
+
+  _item->setData(QString("Material"),
+      ComponentsModel::RoleNames().key("dataType"));
+  _item->setData(QList({
+    QVariant(_data.Ambient().R() * 255),
+    QVariant(_data.Ambient().G() * 255),
+    QVariant(_data.Ambient().B() * 255),
+    QVariant(_data.Ambient().A() * 255),
+    QVariant(_data.Diffuse().R() * 255),
+    QVariant(_data.Diffuse().G() * 255),
+    QVariant(_data.Diffuse().B() * 255),
+    QVariant(_data.Diffuse().A() * 255),
+    QVariant(_data.Specular().R() * 255),
+    QVariant(_data.Specular().G() * 255),
+    QVariant(_data.Specular().B() * 255),
+    QVariant(_data.Specular().A() * 255),
+    QVariant(_data.Emissive().R() * 255),
+    QVariant(_data.Emissive().G() * 255),
+    QVariant(_data.Emissive().B() * 255),
+    QVariant(_data.Emissive().A() * 255)
+  }), ComponentsModel::RoleNames().key("data"));
+
+  // TODO(anyone) Only shows colors of material,
+  // need to add others (e.g., pbr)
+}
+
 //////////////////////////////////////////////////
 void ignition::gazebo::setUnit(QStandardItem *_item, const std::string &_unit)
 {
+  if (nullptr == _item)
+    return;
+
   _item->setData(QString::fromStdString(_unit),
       ComponentsModel::RoleNames().key("unit"));
 }
@@ -333,6 +432,7 @@ ComponentInspector::ComponentInspector()
   : GuiSystem(), dataPtr(std::make_unique<ComponentInspectorPrivate>())
 {
   qRegisterMetaType<ignition::gazebo::ComponentTypeId>();
+  qRegisterMetaType<Entity>("Entity");
 }
 
 /////////////////////////////////////////////////
@@ -350,6 +450,9 @@ void ComponentInspector::LoadConfig(const tinyxml2::XMLElement *)
   // Connect model
   this->Context()->setContextProperty(
       "ComponentsModel", &this->dataPtr->componentsModel);
+
+  // Type-specific handlers
+  this->dataPtr->pose3d = std::make_unique<inspector::Pose3d>(this);
 }
 
 //////////////////////////////////////////////////
@@ -449,12 +552,7 @@ void ComponentInspector::Update(const UpdateInfo &,
     // Add component to list
     else
     {
-      // TODO(louise) Blocking here is not the best idea
-      QMetaObject::invokeMethod(&this->dataPtr->componentsModel,
-          "AddComponentType",
-          Qt::BlockingQueuedConnection,
-          Q_RETURN_ARG(QStandardItem *, item),
-          Q_ARG(ignition::gazebo::ComponentTypeId, typeId));
+      item = this->dataPtr->componentsModel.AddComponentType(typeId);
     }
 
     item->setData(QString::number(this->dataPtr->entity),
@@ -495,6 +593,13 @@ void ComponentInspector::Update(const UpdateInfo &,
       if (comp)
         setData(item, comp->Data());
     }
+    else if (typeId == components::BatterySoC::typeId)
+    {
+      auto comp = _ecm.Component<components::BatterySoC>(
+          this->dataPtr->entity);
+      if (comp)
+        setData(item, comp->Data());
+    }
     else if (typeId == components::CastShadows::typeId)
     {
       auto comp = _ecm.Component<components::CastShadows>(
@@ -502,6 +607,16 @@ void ComponentInspector::Update(const UpdateInfo &,
       if (comp)
         setData(item, comp->Data());
     }
+    else if (typeId == components::CenterOfVolume::typeId)
+    {
+      auto comp = _ecm.Component<components::CenterOfVolume>(
+          this->dataPtr->entity);
+      if (comp)
+      {
+        setData(item, comp->Data());
+        setUnit(item, "m");
+      }
+    }
     else if (typeId == components::ChildLinkName::typeId)
     {
       auto comp = _ecm.Component<components::ChildLinkName>(
@@ -518,6 +633,12 @@ void ComponentInspector::Update(const UpdateInfo &,
         setUnit(item, "m/s\u00B2");
       }
     }
+    else if (typeId == components::LaserRetro::typeId)
+    {
+      auto comp = _ecm.Component<components::LaserRetro>(this->dataPtr->entity);
+      if (comp)
+        setData(item, comp->Data());
+    }
     else if (typeId == components::LinearAcceleration::typeId)
     {
       auto comp = _ecm.Component<components::LinearAcceleration>(
@@ -601,9 +722,31 @@ void ComponentInspector::Update(const UpdateInfo &,
       if (comp)
         setData(item, comp->Data());
     }
-    else if (typeId == components::Pose::typeId)
+    else if (typeId == components::PhysicsCollisionDetector::typeId)
     {
-      auto comp = _ecm.Component<components::Pose>(this->dataPtr->entity);
+      auto comp = _ecm.Component<components::PhysicsCollisionDetector>(
+          this->dataPtr->entity);
+      if (comp)
+        setData(item, comp->Data());
+    }
+    else if (typeId == components::PhysicsSolver::typeId)
+    {
+      auto comp = _ecm.Component<components::PhysicsSolver>(
+          this->dataPtr->entity);
+      if (comp)
+        setData(item, comp->Data());
+    }
+    else if (typeId == components::RenderEngineGuiPlugin::typeId)
+    {
+      auto comp = _ecm.Component<components::RenderEngineGuiPlugin>(
+          this->dataPtr->entity);
+      if (comp)
+        setData(item, comp->Data());
+    }
+    else if (typeId == components::RenderEngineServerPlugin::typeId)
+    {
+      auto comp = _ecm.Component<components::RenderEngineServerPlugin>(
+          this->dataPtr->entity);
       if (comp)
         setData(item, comp->Data());
     }
@@ -634,6 +777,33 @@ void ComponentInspector::Update(const UpdateInfo &,
       if (comp)
         setData(item, comp->Data());
     }
+    else if (typeId == components::ThreadPitch::typeId)
+    {
+      auto comp = _ecm.Component<components::ThreadPitch>(
+          this->dataPtr->entity);
+      if (comp)
+      {
+        setData(item, comp->Data());
+        setUnit(item, "m");
+      }
+    }
+    else if (typeId == components::Transparency::typeId)
+    {
+      auto comp = _ecm.Component<components::Transparency>(
+          this->dataPtr->entity);
+      if (comp)
+        setData(item, comp->Data());
+    }
+    else if (typeId == components::Volume::typeId)
+    {
+      auto comp = _ecm.Component<components::Volume>(
+          this->dataPtr->entity);
+      if (comp)
+      {
+        setData(item, comp->Data());
+        setUnit(item, "m\u00B3");
+      }
+    }
     else if (typeId == components::WindMode::typeId)
     {
       auto comp = _ecm.Component<components::WindMode>(this->dataPtr->entity);
@@ -650,6 +820,16 @@ void ComponentInspector::Update(const UpdateInfo &,
         setUnit(item, "rad/s\u00B2");
       }
     }
+    else if (typeId == components::WorldAngularVelocity::typeId)
+    {
+      auto comp = _ecm.Component<components::WorldAngularVelocity>(
+          this->dataPtr->entity);
+      if (comp)
+      {
+        setData(item, comp->Data());
+        setUnit(item, "rad/s");
+      }
+    }
     else if (typeId == components::WorldLinearVelocity::typeId)
     {
       auto comp = _ecm.Component<components::WorldLinearVelocity>(
@@ -670,18 +850,19 @@ void ComponentInspector::Update(const UpdateInfo &,
         setUnit(item, "m/s");
       }
     }
-    else if (typeId == components::WorldPose::typeId)
+    else if (this->dataPtr->updateViewCbs.find(typeId) !=
+          this->dataPtr->updateViewCbs.end())
     {
-      auto comp = _ecm.Component<components::WorldPose>(this->dataPtr->entity);
-      if (comp)
-        setData(item, comp->Data());
+      this->dataPtr->updateViewCbs[typeId](_ecm, item);
     }
-    else if (typeId == components::WorldPoseCmd::typeId)
+    else if (typeId == components::Material::typeId)
     {
-      auto comp = _ecm.Component<components::WorldPoseCmd>(
-          this->dataPtr->entity);
+      auto comp = _ecm.Component<components::Material>(this->dataPtr->entity);
       if (comp)
+      {
+        this->SetType("material");
         setData(item, comp->Data());
+      }
     }
   }
 
@@ -699,6 +880,13 @@ void ComponentInspector::Update(const UpdateInfo &,
   }
 }
 
+/////////////////////////////////////////////////
+void ComponentInspector::AddUpdateViewCb(ComponentTypeId _id,
+    inspector::UpdateViewCb _cb)
+{
+  this->dataPtr->updateViewCbs[_id] = _cb;
+}
+
 /////////////////////////////////////////////////
 bool ComponentInspector::eventFilter(QObject *_obj, QEvent *_event)
 {
@@ -729,13 +917,13 @@ bool ComponentInspector::eventFilter(QObject *_obj, QEvent *_event)
 }
 
 /////////////////////////////////////////////////
-int ComponentInspector::Entity() const
+Entity ComponentInspector::GetEntity() const
 {
   return this->dataPtr->entity;
 }
 
 /////////////////////////////////////////////////
-void ComponentInspector::SetEntity(const int &_entity)
+void ComponentInspector::SetEntity(const Entity &_entity)
 {
   // If nothing is selected, display world properties
   if (_entity == kNullEntity)
@@ -788,26 +976,6 @@ void ComponentInspector::SetPaused(bool _paused)
   this->PausedChanged();
 }
 
-/////////////////////////////////////////////////
-void ComponentInspector::OnPose(double _x, double _y, double _z, double _roll,
-    double _pitch, double _yaw)
-{
-  std::function<void(const ignition::msgs::Boolean &, const bool)> cb =
-      [](const ignition::msgs::Boolean &/*_rep*/, const bool _result)
-  {
-    if (!_result)
-        ignerr << "Error setting pose" << std::endl;
-  };
-
-  ignition::msgs::Pose req;
-  req.set_id(this->dataPtr->entity);
-  msgs::Set(req.mutable_position(), math::Vector3d(_x, _y, _z));
-  msgs::Set(req.mutable_orientation(), math::Quaterniond(_roll, _pitch, _yaw));
-  auto poseCmdService = "/world/" + this->dataPtr->worldName
-      + "/set_pose";
-  this->dataPtr->node.Request(poseCmdService, req, cb);
-}
-
 /////////////////////////////////////////////////
 void ComponentInspector::OnLight(
   double _rSpecular, double _gSpecular, double _bSpecular, double _aSpecular,
@@ -815,7 +983,8 @@ void ComponentInspector::OnLight(
   double _attRange, double _attLinear, double _attConstant,
   double _attQuadratic, bool _castShadows, double _directionX,
   double _directionY, double _directionZ, double _innerAngle,
-  double _outerAngle, double _falloff, double _intensity, int _type)
+  double _outerAngle, double _falloff, double _intensity, int _type,
+  bool _isLightOn, bool _visualizeVisual)
 {
   std::function<void(const ignition::msgs::Boolean &, const bool)> cb =
       [](const ignition::msgs::Boolean &/*_rep*/, const bool _result)
@@ -825,6 +994,23 @@ void ComponentInspector::OnLight(
   };
 
   ignition::msgs::Light req;
+  {
+    // todo(ahcorde) Use the field is_light_off in light.proto from
+    // Garden on.
+    auto header = req.mutable_header()->add_data();
+    header->set_key("isLightOn");
+    std::string *value = header->add_value();
+    *value = std::to_string(_isLightOn);
+  }
+  {
+    // todo(ahcorde) Use the field visualize_visual in light.proto from
+    // Garden on.
+    auto header = req.mutable_header()->add_data();
+    header->set_key("visualizeVisual");
+    std::string *value = header->add_value();
+    *value = std::to_string(_visualizeVisual);
+  }
+
   req.set_name(this->dataPtr->entityName);
   req.set_id(this->dataPtr->entity);
   ignition::msgs::Set(req.mutable_diffuse(),
@@ -893,12 +1079,114 @@ void ComponentInspector::OnPhysics(double _stepSize, double _realTimeFactor)
   this->dataPtr->node.Request(physicsCmdService, req, cb);
 }
 
+/////////////////////////////////////////////////
+void ComponentInspector::OnMaterialColor(
+  double _rAmbient, double _gAmbient, double _bAmbient, double _aAmbient,
+  double _rDiffuse, double _gDiffuse, double _bDiffuse, double _aDiffuse,
+  double _rSpecular, double _gSpecular, double _bSpecular, double _aSpecular,
+  double _rEmissive, double _gEmissive, double _bEmissive, double _aEmissive,
+  QString _type, QColor _currColor)
+{
+  // when type is not empty, open qt color dialog
+  std::string type = _type.toStdString();
+  if (!type.empty())
+  {
+    QColor newColor = QColorDialog::getColor(
+        _currColor, nullptr, "Pick a color",
+        {QColorDialog::DontUseNativeDialog, QColorDialog::ShowAlphaChannel});
+
+    // returns if the user hits cancel
+    if (!newColor.isValid())
+      return;
+
+    if (type == "ambient")
+    {
+      _rAmbient = newColor.red();
+      _gAmbient = newColor.green();
+      _bAmbient = newColor.blue();
+      _aAmbient = newColor.alpha();
+    }
+    else if (type == "diffuse")
+    {
+      _rDiffuse = newColor.red();
+      _gDiffuse = newColor.green();
+      _bDiffuse = newColor.blue();
+      _aDiffuse = newColor.alpha();
+    }
+    else if (type == "specular")
+    {
+      _rSpecular = newColor.red();
+      _gSpecular = newColor.green();
+      _bSpecular = newColor.blue();
+      _aSpecular = newColor.alpha();
+    }
+    else if (type == "emissive")
+    {
+      _rEmissive = newColor.red();
+      _gEmissive = newColor.green();
+      _bEmissive = newColor.blue();
+      _aEmissive = newColor.alpha();
+    }
+    else
+    {
+      ignerr << "Invalid material type: " << type << std::endl;
+      return;
+    }
+  }
+
+  std::function<void(const ignition::msgs::Boolean &, const bool)> cb =
+      [](const ignition::msgs::Boolean &/*_rep*/, const bool _result)
+  {
+    if (!_result)
+      ignerr << "Error setting material color configuration"
+             << " on visual" << std::endl;
+  };
+
+  msgs::Visual req;
+  req.set_id(this->dataPtr->entity);
+
+  msgs::Set(req.mutable_material()->mutable_ambient(),
+    math::Color(_rAmbient / 255.0, _gAmbient / 255.0,
+      _bAmbient / 255.0, _aAmbient / 255.0));
+  msgs::Set(req.mutable_material()->mutable_diffuse(),
+    math::Color(_rDiffuse / 255.0, _gDiffuse / 255.0,
+      _bDiffuse / 255.0, _aDiffuse / 255.0));
+  msgs::Set(req.mutable_material()->mutable_specular(),
+    math::Color(_rSpecular / 255.0, _gSpecular / 255.0,
+      _bSpecular / 255.0, _aSpecular / 255.0));
+  msgs::Set(req.mutable_material()->mutable_emissive(),
+    math::Color(_rEmissive / 255.0, _gEmissive / 255.0,
+      _bEmissive / 255.0, _aEmissive / 255.0));
+
+  auto materialCmdService = "/world/" + this->dataPtr->worldName
+      + "/visual_config";
+  materialCmdService = transport::TopicUtils::AsValidTopic(materialCmdService);
+  if (materialCmdService.empty())
+  {
+    ignerr << "Invalid material command service topic provided" << std::endl;
+    return;
+  }
+  this->dataPtr->node.Request(materialCmdService, req, cb);
+}
+
 /////////////////////////////////////////////////
 bool ComponentInspector::NestedModel() const
 {
   return this->dataPtr->nestedModel;
 }
 
+/////////////////////////////////////////////////
+const std::string &ComponentInspector::WorldName() const
+{
+  return this->dataPtr->worldName;
+}
+
+/////////////////////////////////////////////////
+transport::Node &ComponentInspector::TransportNode()
+{
+  return this->dataPtr->node;
+}
+
 // Register this plugin
 IGNITION_ADD_PLUGIN(ignition::gazebo::ComponentInspector,
                     ignition::gui::Plugin)
