@@ -19,9 +19,9 @@
 #include <ignition/gui/Application.hh>
 #include <ignition/gui/GuiEvents.hh>
 #include <ignition/gui/MainWindow.hh>
-#include <ignition/plugin/Register.hh>
 #include <ignition/math/Color.hh>
 #include <ignition/math/Pose3.hh>
+#include <ignition/plugin/Register.hh>
 #include <ignition/rendering.hh>
 
 #include "GridConfig.hh"
@@ -43,7 +43,7 @@ namespace ignition::gazebo
     math::Pose3d pose{math::Pose3d::Zero};
 
     /// \brief Default color of grid
-    math::Color color{math::Color(0.7, 0.7, 0.7, 1.0)};
+    math::Color color{math::Color(0.7f, 0.7f, 0.7f, 1.0f)};
 
     /// \brief Default visible state
     bool visible{true};
@@ -149,42 +149,7 @@ void GridConfig::UpdateGrid()
 /////////////////////////////////////////////////
 void GridConfig::LoadGrid()
 {
-  auto loadedEngNames = rendering::loadedEngines();
-  if (loadedEngNames.empty())
-    return;
-
-  // assume there is only one engine loaded
-  auto engineName = loadedEngNames[0];
-  if (loadedEngNames.size() > 1)
-  {
-    igndbg << "More than one engine is available. "
-      << "Grid config plugin will use engine ["
-        << engineName << "]" << std::endl;
-  }
-  auto engine = rendering::engine(engineName);
-  if (!engine)
-  {
-    ignerr << "Internal error: failed to load engine [" << engineName
-      << "]. Grid plugin won't work." << std::endl;
-    return;
-  }
-
-  if (engine->SceneCount() == 0)
-    return;
-
-  // assume there is only one scene
-  // load scene
-  auto scene = engine->SceneByIndex(0);
-  if (!scene)
-  {
-    ignerr << "Internal error: scene is null." << std::endl;
-    return;
-  }
-
-  if (!scene->IsInitialized() || nullptr == scene->RootVisual())
-  {
-    return;
-  }
+  auto scene = rendering::sceneFromFirstRenderEngine();
 
   // load grid
   // if gridPtr found, load the existing gridPtr to class
