@@ -33,21 +33,12 @@
 #include <ignition/transport/Publisher.hh>
 #include <ignition/utils/SuppressWarning.hh>
 
-#include "ignition/gazebo/EntityComponentManager.hh"
 #include "ignition/gazebo/gui/GuiEvents.hh"
 
 namespace ignition::gazebo
 {
   class ShapesPrivate
   {
-    /// \brief Ignition communication node.
-    public: transport::Node node;
-
-    /// \brief Mutex to protect mode
-    public: std::mutex mutex;
-
-    /// \brief Transform control service name
-    public: std::string service;
   };
 }
 
@@ -113,6 +104,11 @@ void Shapes::OnMode(const QString &_mode)
                                              "<size>1 1 1</size>"
                                            "</box>"
                                          "</geometry>"
+                                         "<material>"
+                                           "<ambient>0.3 0.3 0.3 1</ambient>"
+                                           "<diffuse>0.7 0.7 0.7 1</diffuse>"
+                                           "<specular>1 1 1 1</specular>"
+                                         "</material>"
                                        "</visual>"
                                      "</link>"
                                    "</model>"
@@ -149,6 +145,11 @@ void Shapes::OnMode(const QString &_mode)
                                              "<radius>0.5</radius>"
                                            "</sphere>"
                                          "</geometry>"
+                                         "<material>"
+                                           "<ambient>0.3 0.3 0.3 1</ambient>"
+                                           "<diffuse>0.7 0.7 0.7 1</diffuse>"
+                                           "<specular>1 1 1 1</specular>"
+                                         "</material>"
                                        "</visual>"
                                      "</link>"
                                    "</model>"
@@ -187,6 +188,11 @@ void Shapes::OnMode(const QString &_mode)
                                              "<length>1.0</length>"
                                            "</cylinder>"
                                          "</geometry>"
+                                         "<material>"
+                                           "<ambient>0.3 0.3 0.3 1</ambient>"
+                                           "<diffuse>0.7 0.7 0.7 1</diffuse>"
+                                           "<specular>1 1 1 1</specular>"
+                                         "</material>"
                                        "</visual>"
                                      "</link>"
                                    "</model>"
@@ -225,6 +231,11 @@ void Shapes::OnMode(const QString &_mode)
                                              "<length>0.6</length>"
                                            "</capsule>"
                                          "</geometry>"
+                                         "<material>"
+                                           "<ambient>0.3 0.3 0.3 1</ambient>"
+                                           "<diffuse>0.7 0.7 0.7 1</diffuse>"
+                                           "<specular>1 1 1 1</specular>"
+                                         "</material>"
                                        "</visual>"
                                      "</link>"
                                    "</model>"
@@ -261,6 +272,11 @@ void Shapes::OnMode(const QString &_mode)
                                              "<radii>0.2 0.3 0.5</radii>"
                                            "</ellipsoid>"
                                          "</geometry>"
+                                         "<material>"
+                                           "<ambient>0.3 0.3 0.3 1</ambient>"
+                                           "<diffuse>0.7 0.7 0.7 1</diffuse>"
+                                           "<specular>1 1 1 1</specular>"
+                                         "</material>"
                                        "</visual>"
                                      "</link>"
                                    "</model>"
