@@ -27,9 +27,6 @@
 #include <ignition/fuel_tools/ClientConfig.hh>
 #include <ignition/fuel_tools/Result.hh>
 #include <ignition/fuel_tools/WorldIdentifier.hh>
-#include <ignition/msgs/stringmsg.pb.h>
-#include <ignition/transport/Node.hh>
-#include <sdf/Console.hh>
 
 #include "ignition/gazebo/config.hh"
 #include "ignition/gazebo/Server.hh"
@@ -37,58 +34,48 @@
 
 #include "ignition/gazebo/gui/Gui.hh"
 
-using namespace ignition;
-
 //////////////////////////////////////////////////
-extern "C" IGNITION_GAZEBO_VISIBLE char *ignitionGazeboVersion()
+extern "C" char *ignitionGazeboVersion()
 {
   return strdup(IGNITION_GAZEBO_VERSION_FULL);
 }
 
 //////////////////////////////////////////////////
-extern "C" IGNITION_GAZEBO_VISIBLE char *gazeboVersionHeader()
+extern "C" char *gazeboVersionHeader()
 {
   return strdup(IGNITION_GAZEBO_VERSION_HEADER);
 }
 
 //////////////////////////////////////////////////
-extern "C" IGNITION_GAZEBO_VISIBLE void cmdVerbosity(
+extern "C" void cmdVerbosity(
     const char *_verbosity)
 {
-  int verbosity = std::atoi(_verbosity);
-  common::Console::SetVerbosity(verbosity);
-
-  // SDFormat only has 2 levels: quiet / loud. Let sim users suppress all SDF
-  // console output with zero verbosity.
-  if (verbosity == 0)
-  {
-    sdf::Console::Instance()->SetQuiet(true);
-  }
+  ignition::common::Console::SetVerbosity(std::atoi(_verbosity));
 }
 
 //////////////////////////////////////////////////
-extern "C" IGNITION_GAZEBO_VISIBLE const char *worldInstallDir()
+extern "C" const char *worldInstallDir()
 {
   return IGN_GAZEBO_WORLD_INSTALL_DIR;
 }
 
 //////////////////////////////////////////////////
-extern "C" IGNITION_GAZEBO_VISIBLE const char *findFuelResource(
+extern "C" const char *findFuelResource(
     char *_pathToResource)
 {
   std::string path;
   std::string worldPath;
-  fuel_tools::FuelClient fuelClient;
+  ignition::fuel_tools::FuelClient fuelClient;
 
   // Attempt to find cached copy, and then attempt download
-  if (fuelClient.CachedWorld(common::URI(_pathToResource), path))
+  if (fuelClient.CachedWorld(ignition::common::URI(_pathToResource), path))
   {
     ignmsg << "Cached world found." << std::endl;
     worldPath = path;
   }
   // cppcheck-suppress syntaxError
-  else if (fuel_tools::Result result =
-    fuelClient.DownloadWorld(common::URI(_pathToResource), path);
+  else if (ignition::fuel_tools::Result result =
+    fuelClient.DownloadWorld(ignition::common::URI(_pathToResource), path);
     result)
   {
     ignmsg << "Successfully downloaded world from fuel." << std::endl;
@@ -101,20 +88,20 @@ extern "C" IGNITION_GAZEBO_VISIBLE const char *findFuelResource(
     return "";
   }
 
-  if (!common::exists(worldPath))
+  if (!ignition::common::exists(worldPath))
     return "";
 
 
   // Find the first sdf file in the world path for now, the later intention is
   // to load an optional world config file first and if that does not exist,
   // continue to load the first sdf file found as done below
-  for (common::DirIter file(worldPath);
-       file != common::DirIter(); ++file)
+  for (ignition::common::DirIter file(worldPath);
+       file != ignition::common::DirIter(); ++file)
   {
     std::string current(*file);
-    if (common::isFile(current))
+    if (ignition::common::isFile(current))
     {
-      std::string fileName = common::basename(current);
+      std::string fileName = ignition::common::basename(current);
       std::string::size_type fileExtensionIndex = fileName.rfind(".");
       std::string fileExtension = fileName.substr(fileExtensionIndex + 1);
 
@@ -128,51 +115,22 @@ extern "C" IGNITION_GAZEBO_VISIBLE const char *findFuelResource(
 }
 
 //////////////////////////////////////////////////
-extern "C" IGNITION_GAZEBO_VISIBLE int runServer(const char *_sdfString,
+extern "C" int runServer(const char *_sdfString,
     int _iterations, int _run, float _hz, int _levels, const char *_networkRole,
     int _networkSecondaries, int _record, const char *_recordPath,
     int _recordResources, int _logOverwrite, int _logCompress,
     const char *_playback, const char *_physicsEngine,
     const char *_renderEngineServer, const char *_renderEngineGui,
-    const char *_file, const char *_recordTopics, int _waitGui)
+    const char *_file, const char *_recordTopics)
 {
-  std::string startingWorldPath{""};
-  gazebo::ServerConfig serverConfig;
-
-  // Lock until the starting world is received from Gui
-  if (_waitGui == 1)
-  {
-    transport::Node node;
-    std::condition_variable condition;
-    std::mutex mutex;
-
-    // Create a subscriber just so we can check when the message has propagated
-    std::function<void(const msgs::StringMsg &)> topicCb =
-        [&startingWorldPath, &mutex, &condition](const auto &_msg)
-        {
-          std::unique_lock<std::mutex> lock(mutex);
-          startingWorldPath = _msg.data();
-          condition.notify_all();
-        };
-
-    std::string topic{"/gazebo/starting_world"};
-    std::unique_lock<std::mutex> lock(mutex);
-    igndbg << "Subscribing to [" << topic << "]." << std::endl;
-    node.Subscribe(topic, topicCb);
-    igndbg << "Waiting for a world to be set from the GUI..." << std::endl;
-    condition.wait(lock);
-    ignmsg << "Received world [" << startingWorldPath << "] from the GUI."
-          << std::endl;
-    igndbg << "Unsubscribing from [" << topic << "]." << std::endl;
-    node.Unsubscribe(topic);
-  }
+  ignition::gazebo::ServerConfig serverConfig;
 
   // Path for logs
   std::string recordPathMod = serverConfig.LogRecordPath();
 
   // Path for compressed log, used to check for duplicates
   std::string cmpPath = std::string(recordPathMod);
-  if (!std::string(1, cmpPath.back()).compare(common::separator("")))
+  if (!std::string(1, cmpPath.back()).compare(ignition::common::separator("")))
   {
     // Remove the separator at end of path
     cmpPath = cmpPath.substr(0, cmpPath.length() - 1);
@@ -200,7 +158,7 @@ extern "C" IGNITION_GAZEBO_VISIBLE int runServer(const char *_sdfString,
 
       // Update compressed file path to name of recording directory path
       cmpPath = std::string(recordPathMod);
-      if (!std::string(1, cmpPath.back()).compare(common::separator(
+      if (!std::string(1, cmpPath.back()).compare(ignition::common::separator(
         "")))
       {
         // Remove the separator at end of path
@@ -209,8 +167,8 @@ extern "C" IGNITION_GAZEBO_VISIBLE int runServer(const char *_sdfString,
       cmpPath += ".zip";
 
       // Check if path or compressed file with same prefix exists
-      if (common::exists(recordPathMod) ||
-        common::exists(cmpPath))
+      if (ignition::common::exists(recordPathMod) ||
+        ignition::common::exists(cmpPath))
       {
         // Overwrite if flag specified
         if (_logOverwrite > 0)
@@ -218,15 +176,15 @@ extern "C" IGNITION_GAZEBO_VISIBLE int runServer(const char *_sdfString,
           bool recordMsg = false;
           bool cmpMsg = false;
           // Remove files before initializing console log files on top of them
-          if (common::exists(recordPathMod))
+          if (ignition::common::exists(recordPathMod))
           {
             recordMsg = true;
-            common::removeAll(recordPathMod);
+            ignition::common::removeAll(recordPathMod);
           }
-          if (common::exists(cmpPath))
+          if (ignition::common::exists(cmpPath))
           {
             cmpMsg = true;
-            common::removeFile(cmpPath);
+            ignition::common::removeFile(cmpPath);
           }
 
           // Create log file before printing any messages so they can be logged
@@ -253,7 +211,7 @@ extern "C" IGNITION_GAZEBO_VISIBLE int runServer(const char *_sdfString,
         {
           // Remove the separator at end of path
           if (!std::string(1, recordPathMod.back()).compare(
-            common::separator("")))
+            ignition::common::separator("")))
           {
             recordPathMod = recordPathMod.substr(0, recordPathMod.length()
               - 1);
@@ -264,8 +222,8 @@ extern "C" IGNITION_GAZEBO_VISIBLE int runServer(const char *_sdfString,
 
           // Keep renaming until path does not exist for both directory and
           // compressed file
-          while (common::exists(recordPathMod) ||
-            common::exists(cmpPath))
+          while (ignition::common::exists(recordPathMod) ||
+            ignition::common::exists(cmpPath))
           {
             recordPathMod = recordOrigPrefix +  "(" + std::to_string(count++) +
               ")";
@@ -273,7 +231,7 @@ extern "C" IGNITION_GAZEBO_VISIBLE int runServer(const char *_sdfString,
             cmpPath = std::string(recordPathMod);
             // Remove the separator at end of path
             if (!std::string(1, cmpPath.back()).compare(
-              common::separator("")))
+              ignition::common::separator("")))
             {
               cmpPath = cmpPath.substr(0, cmpPath.length() - 1);
             }
@@ -294,14 +252,6 @@ extern "C" IGNITION_GAZEBO_VISIBLE int runServer(const char *_sdfString,
       {
         ignLogInit(recordPathMod, "server_console.log");
       }
-      // TODO(anyone) In Ignition-D, to be moved to outside and after this
-      //   if-else statement, after all ignLogInit() calls have been finalized,
-      //   so that <path> in SDF will always be ignored in favor of logging both
-      //   console logs and LogRecord recordings to common::ignLogDirectory().
-      //   In Blueprint and Citadel, LogRecord will record to <path> if no
-      //   --record-path is specified on command line.
-      serverConfig.SetLogRecordPath(recordPathMod);
-      serverConfig.SetLogIgnoreSdfPath(true);
     }
     // Empty record path specified. Use default.
     else
@@ -310,11 +260,10 @@ extern "C" IGNITION_GAZEBO_VISIBLE int runServer(const char *_sdfString,
       ignLogInit(recordPathMod, "server_console.log");
       ignmsg << "Recording states to default path [" << recordPathMod << "]"
              << std::endl;
-
-      serverConfig.SetLogRecordPath(recordPathMod);
     }
+    serverConfig.SetLogRecordPath(recordPathMod);
 
-    std::vector<std::string> topics = common::split(
+    std::vector<std::string> topics = ignition::common::split(
         _recordTopics, ":");
     for (const std::string &topic : topics)
     {
@@ -343,13 +292,7 @@ extern "C" IGNITION_GAZEBO_VISIBLE int runServer(const char *_sdfString,
       return -1;
     }
   }
-
-  // This ensures if the server was run stand alone with a world from
-  // command line, the correct world would be loaded.
-  if(_waitGui == 1)
-    serverConfig.SetSdfFile(startingWorldPath);
-  else
-    serverConfig.SetSdfFile(_file);
+  serverConfig.SetSdfFile(_file);
 
   // Set the update rate.
   if (_hz > 0.0)
@@ -381,7 +324,7 @@ extern "C" IGNITION_GAZEBO_VISIBLE int runServer(const char *_sdfString,
     else
     {
       ignmsg << "Playing back states" << _playback << std::endl;
-      serverConfig.SetLogPlaybackPath(common::absPath(
+      serverConfig.SetLogPlaybackPath(ignition::common::absPath(
         std::string(_playback)));
     }
   }
@@ -402,7 +345,7 @@ extern "C" IGNITION_GAZEBO_VISIBLE int runServer(const char *_sdfString,
   }
 
   // Create the Gazebo server
-  gazebo::Server server(serverConfig);
+  ignition::gazebo::Server server(serverConfig);
 
   // Run the server
   server.Run(true, _iterations, _run == 0);
@@ -412,8 +355,7 @@ extern "C" IGNITION_GAZEBO_VISIBLE int runServer(const char *_sdfString,
 }
 
 //////////////////////////////////////////////////
-extern "C" IGNITION_GAZEBO_VISIBLE int runGui(
-  const char *_guiConfig, const char *_file, int _waitGui)
+extern "C" int runGui(const char *_guiConfig)
 {
   // argc and argv are going to be passed to a QApplication. The Qt
   // documentation has a warning about these:
@@ -426,6 +368,5 @@ extern "C" IGNITION_GAZEBO_VISIBLE int runGui(
   // be converted to a const char *. The const cast is here to prevent a warning
   // since we do need to pass a char* to runGui
   char *argv = const_cast<char *>("ign-gazebo-gui");
-  return gazebo::gui::runGui(
-    argc, &argv, _guiConfig, _file, _waitGui);
+  return ignition::gazebo::gui::runGui(argc, &argv, _guiConfig);
 }
