@@ -16,6 +16,7 @@
  */
 
 #include <map>
+#include <stack>
 #include <string>
 #include <tuple>
 #include <unordered_map>
@@ -75,6 +76,7 @@
 #include "ignition/gazebo/components/Transparency.hh"
 #include "ignition/gazebo/components/Visibility.hh"
 #include "ignition/gazebo/components/Visual.hh"
+#include "ignition/gazebo/components/VisualCmd.hh"
 #include "ignition/gazebo/components/World.hh"
 #include "ignition/gazebo/EntityComponentManager.hh"
 
@@ -120,11 +122,15 @@ class ignition::gazebo::RenderUtilPrivate
   //// \brief True to enable sky in the scene
   public: bool skyEnabled = false;
 
-  /// \brief Scene background color
-  public: math::Color backgroundColor = math::Color::Black;
+  /// \brief Scene background color. This is optional because a <scene> is
+  /// always present, which has a default background color value. This
+  /// backgroundColor variable is used to override the <scene> value.
+  public: std::optional<math::Color> backgroundColor = math::Color::Black;
 
-  /// \brief Ambient color
-  public: math::Color ambientLight = math::Color(1.0, 1.0, 1.0, 1.0);
+  /// \brief Ambient color. This is optional because an <scene> is always
+  /// present, which has a default ambient light value. This ambientLight
+  /// variable is used to override the <scene> value.
+  public: std::optional<math::Color> ambientLight;
 
   /// \brief Scene manager
   public: SceneManager sceneManager;
@@ -201,6 +207,25 @@ class ignition::gazebo::RenderUtilPrivate
   /// \brief A map of entity ids and light updates.
   public: std::vector<Entity> entityLightsCmdToDelete;
 
+  /// \brief A map of entity ids and visual updates.
+  public: std::map<Entity, msgs::Visual> entityVisuals;
+
+  /// \brief A vector of entity ids of VisualCmds to delete
+  public: std::vector<Entity> entityVisualsCmdToDelete;
+
+  /// \brief Visual material equality comparision function
+  /// TODO(anyone) Currently only checks for material colors equality,
+  /// need to extend to others (e.g., PbrMaterial)
+  public: std::function<bool(const sdf::Material &, const sdf::Material &)>
+          materialEql { [](const sdf::Material &_a, const sdf::Material &_b)
+            {
+              return
+                _a.Ambient() == _b.Ambient() &&
+                _a.Diffuse() == _b.Diffuse() &&
+                _a.Specular() == _b.Specular() &&
+                _a.Emissive() == _b.Emissive();
+            }};
+
   /// \brief A map of entity ids and actor transforms.
   public: std::map<Entity, std::map<std::string, math::Matrix4d>>
                           actorTransforms;
@@ -254,6 +279,7 @@ class ignition::gazebo::RenderUtilPrivate
           lightEql { [](const sdf::Light &_a, const sdf::Light &_b)
             {
              return
+                _a.Visualize() == _b.Visualize() &&
                 _a.Type() == _b.Type() &&
                 _a.Name() == _b.Name() &&
                 _a.Diffuse() == _b.Diffuse() &&
@@ -324,11 +350,55 @@ class ignition::gazebo::RenderUtilPrivate
   /// visible
   public: std::map<Entity, bool> viewingCollisions;
 
+  /// \brief A map of model entities and their corresponding children models
+  public: std::map<Entity, std::vector<Entity>> modelToModelEntities;
+
   /// \brief A map of entity id to thermal camera sensor configuration
   /// properties. The elements in the tuple are:
   /// <resolution, temperature range (min, max)>
-  public:std::unordered_map<Entity,
+  public: std::unordered_map<Entity,
       std::tuple<double, components::TemperatureRangeInfo>> thermalCameraData;
+
+  /// \brief A helper function that removes the sensor associated with an
+  /// entity, if an associated sensor exists. This should be called in
+  /// RenderUtil::Update.
+  /// \param[in] _entity The entity that should be checked for an associated
+  /// sensor.
+  public: void RemoveSensor(const Entity _entity);
+
+  /// \brief A helper function that removes the bounding box associated with an
+  /// entity, if an associated bounding box exists. This should be called in
+  /// RenderUtil::Update.
+  /// \param[in] _entity The entity that should be checked for an associated
+  /// bounding box.
+  public: void RemoveBoundingBox(const Entity _entity);
+
+  /// \brief A helper function for updating lights. This should be called in
+  /// RenderUtil::Update.
+  /// \param[in] _entityLights A map of entity IDs to their light updates.
+  public: void UpdateLights(
+              const std::unordered_map<Entity, msgs::Light> &_entityLights);
+
+  /// \brief A helper function for updating the thermal camera. This should be
+  /// called in RenderUtil::Update.
+  /// \param[in] _thermalCamData The thermal camera data that needs to be
+  /// updated.
+  /// \sa thermalCameraData
+  public: void UpdateThermalCamera(const std::unordered_map<Entity,
+    std::tuple<double, components::TemperatureRangeInfo>> &_thermalCamData);
+
+  /// \brief Helper function for updating animation. This should be called in
+  /// RenderUtil::Update.
+  /// \param[in] _actorAnimationData A map of entities to their animation update
+  /// data.
+  /// \param[in] _entityPoses A map of entity ids and pose updates.
+  /// \param[in] _trajectoryPoses A map of entity ids and trajectory
+  /// pose updates.
+  /// \sa actorManualSkeletonUpdate
+  public: void UpdateAnimation(const std::unordered_map<Entity,
+              AnimationUpdateData> &_actorAnimationData,
+              const std::unordered_map<Entity, math::Pose3d> &_entityPoses,
+              const std::unordered_map<Entity, math::Pose3d> &_trajectoryPoses);
 };
 
 //////////////////////////////////////////////////
@@ -450,6 +520,41 @@ void RenderUtil::UpdateECM(const UpdateInfo &/*_info*/,
         }
         return true;
       });
+
+  // visual commands
+  {
+    auto olderEntityVisualsCmdToDelete
+        = std::move(this->dataPtr->entityVisualsCmdToDelete);
+    this->dataPtr->entityVisualsCmdToDelete.clear();
+
+    // TODO(anyone) Currently only updates material colors,
+    // need to extend to others
+    _ecm.Each<components::VisualCmd>(
+      [&](const Entity &_entity,
+          const components::VisualCmd *_visualCmd) -> bool
+      {
+        this->dataPtr->entityVisuals[_entity] = _visualCmd->Data();
+        this->dataPtr->entityVisualsCmdToDelete.push_back(_entity);
+
+        auto materialComp = _ecm.Component<components::Material>(_entity);
+        if (materialComp)
+        {
+          msgs::Material materialMsg = _visualCmd->Data().material();
+          sdf::Material sdfMaterial = convert<sdf::Material>(materialMsg);
+
+          auto state =
+              materialComp->SetData(sdfMaterial, this->dataPtr->materialEql) ?
+              ComponentState::OneTimeChange : ComponentState::NoChange;
+          _ecm.SetChanged(_entity, components::Material::typeId, state);
+        }
+        return true;
+      });
+
+    for (const auto entity : olderEntityVisualsCmdToDelete)
+    {
+      _ecm.RemoveComponent<components::VisualCmd>(entity);
+    }
+  }
 }
 
 //////////////////////////////////////////////////
@@ -479,8 +584,29 @@ void RenderUtilPrivate::FindCollisionLinks(const EntityComponentManager &_ecm)
     if (_ecm.EntityMatches(entity,
           std::set<ComponentTypeId>{components::Model::typeId}))
     {
-      links = _ecm.EntitiesByComponents(components::ParentEntity(entity),
-                                        components::Link());
+      std::stack<Entity> modelStack;
+      modelStack.push(entity);
+
+      std::vector<Entity> childLinks, childModels;
+      while (!modelStack.empty())
+      {
+        Entity model = modelStack.top();
+        modelStack.pop();
+
+        childLinks = _ecm.EntitiesByComponents(components::ParentEntity(model),
+                                               components::Link());
+        links.insert(links.end(),
+                     childLinks.begin(),
+                     childLinks.end());
+
+        childModels =
+            _ecm.EntitiesByComponents(components::ParentEntity(model),
+                                      components::Model());
+        for (const auto &childModel : childModels)
+        {
+            modelStack.push(childModel);
+        }
+      }
     }
     else if (_ecm.EntityMatches(entity,
                 std::set<ComponentTypeId>{components::Link::typeId}))
@@ -540,6 +666,7 @@ void RenderUtil::Update()
   auto removeEntities = std::move(this->dataPtr->removeEntities);
   auto entityPoses = std::move(this->dataPtr->entityPoses);
   auto entityLights = std::move(this->dataPtr->entityLights);
+  auto entityVisuals = std::move(this->dataPtr->entityVisuals);
   auto trajectoryPoses = std::move(this->dataPtr->trajectoryPoses);
   auto actorTransforms = std::move(this->dataPtr->actorTransforms);
   auto actorAnimationData = std::move(this->dataPtr->actorAnimationData);
@@ -558,6 +685,7 @@ void RenderUtil::Update()
   this->dataPtr->removeEntities.clear();
   this->dataPtr->entityPoses.clear();
   this->dataPtr->entityLights.clear();
+  this->dataPtr->entityVisuals.clear();
   this->dataPtr->trajectoryPoses.clear();
   this->dataPtr->actorTransforms.clear();
   this->dataPtr->actorAnimationData.clear();
@@ -579,8 +707,16 @@ void RenderUtil::Update()
   // extend the sensor system to support mutliple scenes in the future
   for (auto &scene : newScenes)
   {
-    this->dataPtr->scene->SetAmbientLight(scene.Ambient());
-    this->dataPtr->scene->SetBackgroundColor(scene.Background());
+    // Only set the ambient color if the RenderUtil::SetBackgroundColor
+    // was not called.
+    if (!this->dataPtr->ambientLight)
+      this->dataPtr->scene->SetAmbientLight(scene.Ambient());
+
+    // Only set the background color if the RenderUtil::SetBackgroundColor
+    // was not called.
+    if (!this->dataPtr->backgroundColor)
+      this->dataPtr->scene->SetBackgroundColor(scene.Background());
+
     if (scene.Grid() && !this->dataPtr->enableSensors)
       this->ShowGrid();
     if (scene.Sky())
@@ -604,21 +740,8 @@ void RenderUtil::Update()
           this->dataPtr->selectedEntities.end());
       this->dataPtr->sceneManager.RemoveEntity(entity.first);
 
-      // delete associated sensor, if existing
-      auto sensorEntityIt = this->dataPtr->sensorEntities.find(entity.first);
-      if (sensorEntityIt != this->dataPtr->sensorEntities.end())
-      {
-        this->dataPtr->removeSensorCb(entity.first);
-        this->dataPtr->sensorEntities.erase(sensorEntityIt);
-      }
-
-      // delete associated bounding box, if existing
-      auto wireBoxIt = this->dataPtr->wireBoxes.find(entity.first);
-      if (wireBoxIt != this->dataPtr->wireBoxes.end())
-      {
-        this->dataPtr->scene->DestroyVisual(wireBoxIt->second->Parent());
-        this->dataPtr->wireBoxes.erase(wireBoxIt);
-      }
+      this->dataPtr->RemoveSensor(entity.first);
+      this->dataPtr->RemoveBoundingBox(entity.first);
     }
   }
 
@@ -669,18 +792,23 @@ void RenderUtil::Update()
       this->dataPtr->sceneManager.CreateLight(
           std::get<0>(light), std::get<1>(light), std::get<2>(light));
 
-      // create a new id for the light visual
-      auto attempts = 100000u;
-      for (auto i = 0u; i < attempts; ++i)
+      // TODO(anyone) This needs to be updated for when sensors and GUI use
+      // the same scene
+      // create a new id for the light visual, if we're not loading sensors
+      if (!this->dataPtr->enableSensors)
       {
-        Entity id = std::numeric_limits<uint64_t>::min() + i;
-        if (!this->dataPtr->sceneManager.HasEntity(id))
+        auto attempts = 100000u;
+        for (auto i = 0u; i < attempts; ++i)
         {
-          rendering::VisualPtr lightVisual =
-            this->dataPtr->sceneManager.CreateLightVisual(
-              id, std::get<1>(light), std::get<0>(light));
-          this->dataPtr->matchLightWithVisuals[std::get<0>(light)] = id;
-          break;
+          Entity id = std::numeric_limits<uint64_t>::min() + i;
+          if (!this->dataPtr->sceneManager.HasEntity(id))
+          {
+            rendering::VisualPtr lightVisual =
+              this->dataPtr->sceneManager.CreateLightVisual(
+                id, std::get<1>(light), std::get<0>(light));
+            this->dataPtr->matchLightWithVisuals[std::get<0>(light)] = id;
+            break;
+          }
         }
       }
     }
@@ -730,101 +858,8 @@ void RenderUtil::Update()
     }
   }
 
-  // update lights
-  {
-    IGN_PROFILE("RenderUtil::Update Lights");
-    for (const auto &light : entityLights) {
-      auto node = this->dataPtr->sceneManager.NodeById(light.first);
-      if (!node)
-        continue;
-      auto l = std::dynamic_pointer_cast<rendering::Light>(node);
-      if (l)
-      {
-        if (light.second.has_diffuse())
-        {
-          if (l->DiffuseColor() != msgs::Convert(light.second.diffuse()))
-            l->SetDiffuseColor(msgs::Convert(light.second.diffuse()));
-        }
-        if (light.second.has_specular())
-        {
-          if (l->SpecularColor() != msgs::Convert(light.second.specular()))
-          {
-            l->SetSpecularColor(msgs::Convert(light.second.specular()));
-          }
-        }
-        if (!ignition::math::equal(
-            l->AttenuationRange(),
-            static_cast<double>(light.second.range())))
-        {
-          l->SetAttenuationRange(light.second.range());
-        }
-        if (!ignition::math::equal(
-            l->AttenuationLinear(),
-            static_cast<double>(light.second.attenuation_linear())))
-        {
-          l->SetAttenuationLinear(light.second.attenuation_linear());
-        }
-        if (!ignition::math::equal(
-            l->AttenuationConstant(),
-            static_cast<double>(light.second.attenuation_constant())))
-        {
-          l->SetAttenuationConstant(light.second.attenuation_constant());
-        }
-        if (!ignition::math::equal(
-            l->AttenuationQuadratic(),
-            static_cast<double>(light.second.attenuation_quadratic())))
-        {
-          l->SetAttenuationQuadratic(light.second.attenuation_quadratic());
-        }
-        if (l->CastShadows() != light.second.cast_shadows())
-          l->SetCastShadows(light.second.cast_shadows());
-        if (!ignition::math::equal(
-            l->Intensity(),
-            static_cast<double>(light.second.intensity())))
-        {
-          l->SetIntensity(light.second.intensity());
-        }
-        auto lDirectional =
-          std::dynamic_pointer_cast<rendering::DirectionalLight>(node);
-        if (lDirectional)
-        {
-          if (light.second.has_direction())
-          {
-            if (lDirectional->Direction() !=
-                msgs::Convert(light.second.direction()))
-            {
-              lDirectional->SetDirection(
-                msgs::Convert(light.second.direction()));
-            }
-          }
-        }
-        auto lSpotLight =
-          std::dynamic_pointer_cast<rendering::SpotLight>(node);
-        if (lSpotLight)
-        {
-          if (light.second.has_direction())
-          {
-            if (lSpotLight->Direction() !=
-              msgs::Convert(light.second.direction()))
-            {
-              lSpotLight->SetDirection(
-                msgs::Convert(light.second.direction()));
-            }
-          }
-          if (lSpotLight->InnerAngle() != light.second.spot_inner_angle())
-            lSpotLight->SetInnerAngle(light.second.spot_inner_angle());
-          if (lSpotLight->OuterAngle() != light.second.spot_outer_angle())
-            lSpotLight->SetOuterAngle(light.second.spot_outer_angle());
-          if (!ignition::math::equal(
-              lSpotLight->Falloff(),
-              static_cast<double>(light.second.spot_falloff())))
-          {
-            lSpotLight->SetFalloff(light.second.spot_falloff());
-          }
-        }
-      }
-    }
-  }
+  this->dataPtr->UpdateLights(entityLights);
+
   // update entities' pose
   {
     IGN_PROFILE("RenderUtil::Update Poses");
@@ -900,96 +935,8 @@ void RenderUtil::Update()
     }
     else
     {
-      for (auto &it : actorAnimationData)
-      {
-        auto actorMesh = this->dataPtr->sceneManager.ActorMeshById(it.first);
-        auto actorVisual = this->dataPtr->sceneManager.NodeById(it.first);
-        auto actorSkel = this->dataPtr->sceneManager.ActorSkeletonById(
-            it.first);
-        if (!actorMesh || !actorVisual || !actorSkel)
-        {
-          ignerr << "Actor with Entity ID '" << it.first << "'. not found. "
-                 << "Skipping skeleton animation update." << std::endl;
-          continue;
-        }
-
-        AnimationUpdateData &animData = it.second;
-        if (!animData.valid)
-        {
-          ignerr << "invalid animation update data" << std::endl;
-          continue;
-        }
-        // Enable skeleton animation
-        if (!actorMesh->SkeletonAnimationEnabled(animData.animationName))
-        {
-          // disable all animations for this actor
-          for (unsigned int i = 0; i < actorSkel->AnimationCount(); ++i)
-          {
-            actorMesh->SetSkeletonAnimationEnabled(
-                actorSkel->Animation(i)->Name(), false, false, 0.0);
-          }
-
-          // enable requested animation
-          actorMesh->SetSkeletonAnimationEnabled(
-              animData.animationName, true, animData.loop);
-
-          // Set skeleton root node weight to zero so it is not affected by
-          // the animation being played. This is needed if trajectory animation
-          // is enabled. We need to let the trajectory animation set the
-          // position of the actor instead
-          common::SkeletonPtr skeleton =
-              this->dataPtr->sceneManager.ActorSkeletonById(it.first);
-          if (skeleton)
-          {
-            float rootBoneWeight = (animData.followTrajectory) ? 0.0 : 1.0;
-            std::unordered_map<std::string, float> weights;
-            weights[skeleton->RootNode()->Name()] = rootBoneWeight;
-            actorMesh->SetSkeletonWeights(weights);
-          }
-        }
-        // Update skeleton animation by setting animation time.
-        // Note that animation time is different from sim time. An actor can
-        // have multiple animations. Animation time is associated with
-        // current animation that is being played. It is also adjusted if
-        // interpotate_x is enabled.
-        actorMesh->UpdateSkeletonAnimation(animData.time);
-
-        // manually update root transform in order to sync with trajectory
-        // animation
-        if (animData.followTrajectory)
-        {
-          common::SkeletonPtr skeleton =
-              this->dataPtr->sceneManager.ActorSkeletonById(it.first);
-          std::map<std::string, math::Matrix4d> rootTf;
-          rootTf[skeleton->RootNode()->Name()] = animData.rootTransform;
-          actorMesh->SetSkeletonLocalTransforms(rootTf);
-        }
-
-        // update actor trajectory animation
-        math::Pose3d globalPose;
-        if (entityPoses.find(it.first) != entityPoses.end())
-        {
-          globalPose = entityPoses[it.first];
-        }
-
-        math::Pose3d trajPose;
-        // Trajectory from the ECS
-        if (trajectoryPoses.find(it.first) != trajectoryPoses.end())
-        {
-          trajPose = trajectoryPoses[it.first];
-        }
-        else
-        {
-          // trajectory from sdf script
-          common::PoseKeyFrame poseFrame(0.0);
-          if (animData.followTrajectory)
-            animData.trajectory.Waypoints()->InterpolatedKeyFrame(poseFrame);
-          trajPose.Pos() = poseFrame.Translation();
-          trajPose.Rot() = poseFrame.Rotation();
-        }
-
-        actorVisual->SetLocalPose(trajPose + globalPose);
-      }
+      this->dataPtr->UpdateAnimation(actorAnimationData, entityPoses,
+          trajectoryPoses);
     }
   }
 
@@ -1053,40 +1000,64 @@ void RenderUtil::Update()
     }
   }
 
-  // update thermal camera
-  for (const auto &thermal : this->dataPtr->thermalCameraData)
-  {
-    Entity id = thermal.first;
-    rendering::ThermalCameraPtr camera =
-        std::dynamic_pointer_cast<rendering::ThermalCamera>(
-        this->dataPtr->sceneManager.NodeById(id));
-    if (camera)
-    {
-      double resolution = std::get<0>(thermal.second);
+  this->dataPtr->UpdateThermalCamera(thermalCameraData);
 
-      if (resolution > 0.0)
+  // update visuals
+  // TODO(anyone) currently updates material colors of visual only,
+  // need to extend to other updates
+  {
+    IGN_PROFILE("RenderUtil::Update Visuals");
+    for (const auto &visual : entityVisuals)
+    {
+      if (!visual.second.has_material())
+        continue;
+
+      auto node = this->dataPtr->sceneManager.NodeById(visual.first);
+      if (!node)
+        continue;
+
+      auto vis = std::dynamic_pointer_cast<rendering::Visual>(node);
+      if (vis)
       {
-        camera->SetLinearResolution(resolution);
-      }
-      else
-      {
-        ignwarn << "Unable to set thermal camera temperature linear resolution."
-                << " Value must be greater than 0. Using the default value: "
-                << camera->LinearResolution() << ". " << std::endl;
-      }
-      double minTemp = std::get<1>(thermal.second).min.Kelvin();
-      double maxTemp = std::get<1>(thermal.second).max.Kelvin();
-      if (maxTemp >= minTemp)
-      {
-        camera->SetMinTemperature(minTemp);
-        camera->SetMaxTemperature(maxTemp);
-      }
-      else
-      {
-        ignwarn << "Unable to set thermal camera temperature range."
-                << "Max temperature must be greater or equal to min. "
-                << "Using the default values : [" << camera->MinTemperature()
-                << ", " << camera->MaxTemperature() << "]." << std::endl;
+        msgs::Material matMsg = visual.second.material();
+
+        // Geometry material
+        for (auto g = 0u; g < vis->GeometryCount(); ++g)
+        {
+          rendering::GeometryPtr geom = vis->GeometryByIndex(g);
+          rendering::MaterialPtr geomMat = geom->Material();
+          if (!geomMat)
+            continue;
+
+          math::Color color;
+          if (matMsg.has_ambient())
+          {
+            color = msgs::Convert(matMsg.ambient());
+            if (geomMat->Ambient() != color)
+              geomMat->SetAmbient(color);
+          }
+
+          if (matMsg.has_diffuse())
+          {
+            color = msgs::Convert(matMsg.diffuse());
+            if (geomMat->Diffuse() != color)
+              geomMat->SetDiffuse(color);
+          }
+
+          if (matMsg.has_specular())
+          {
+            color = msgs::Convert(matMsg.specular());
+            if (geomMat->Specular() != color)
+              geomMat->SetSpecular(color);
+          }
+
+          if (matMsg.has_emissive())
+          {
+            color = msgs::Convert(matMsg.emissive());
+            if (geomMat->Emissive() != color)
+              geomMat->SetEmissive(color);
+          }
+        }
       }
     }
   }
@@ -1154,6 +1125,7 @@ void RenderUtilPrivate::CreateRenderingEntities(
           this->newModels.push_back(
               std::make_tuple(_entity, model, _parent->Data(),
               _info.iterations));
+          this->modelToModelEntities[_parent->Data()].push_back(_entity);
           return true;
         });
 
@@ -1382,6 +1354,7 @@ void RenderUtilPrivate::CreateRenderingEntities(
           this->newModels.push_back(
               std::make_tuple(_entity, model, _parent->Data(),
               _info.iterations));
+          this->modelToModelEntities[_parent->Data()].push_back(_entity);
           return true;
         });
 
@@ -1653,8 +1626,13 @@ void RenderUtilPrivate::UpdateRenderingEntities(
         // Trajectory info from SDF so ign-rendering can calculate bone poses
         else
         {
-          this->actorAnimationData[_entity] =
-              this->sceneManager.ActorAnimationAt(_entity, this->simTime);
+          auto animData =
+            this->sceneManager.ActorAnimationAt(_entity, this->simTime);
+
+          if (animData.valid)
+          {
+            this->actorAnimationData[_entity] = animData;
+          }
         }
 
         // Trajectory pose set by other systems
@@ -1735,6 +1713,7 @@ void RenderUtilPrivate::RemoveRenderingEntities(
       {
         this->removeEntities[_entity] = _info.iterations;
         this->modelToLinkEntities.erase(_entity);
+        this->modelToModelEntities.erase(_entity);
         return true;
       });
 
@@ -1852,8 +1831,17 @@ void RenderUtil::Init()
         this->dataPtr->engine->CreateScene(this->dataPtr->sceneName);
     if (this->dataPtr->scene)
     {
-      this->dataPtr->scene->SetAmbientLight(this->dataPtr->ambientLight);
-      this->dataPtr->scene->SetBackgroundColor(this->dataPtr->backgroundColor);
+      if (this->dataPtr->ambientLight)
+      {
+        this->dataPtr->scene->SetAmbientLight(
+            *this->dataPtr->ambientLight);
+      }
+
+      if (this->dataPtr->backgroundColor)
+      {
+        this->dataPtr->scene->SetBackgroundColor(
+            *this->dataPtr->backgroundColor);
+      }
       this->dataPtr->scene->SetSkyEnabled(this->dataPtr->skyEnabled);
     }
   }
@@ -2055,9 +2043,15 @@ void RenderUtilPrivate::HighlightNode(const rendering::NodePtr &_node)
       white->SetEmissive(1.0, 1.0, 1.0);
     }
 
-    ignition::rendering::WireBoxPtr wireBox =
-      this->scene->CreateWireBox();
-    ignition::math::AxisAlignedBox aabb = vis->LocalBoundingBox();
+    auto aabb = vis->LocalBoundingBox();
+    if (aabb == math::AxisAlignedBox())
+    {
+      // Infinite bounding box, skip highlighting this node.
+      // This happens for Heightmaps, for example.
+      return;
+    }
+
+    auto wireBox = this->scene->CreateWireBox();
     wireBox->SetBox(aabb);
 
     // Create visual and add wire box
@@ -2102,10 +2096,323 @@ void RenderUtilPrivate::LowlightNode(const rendering::NodePtr &_node)
   }
 }
 
+/////////////////////////////////////////////////
+void RenderUtilPrivate::RemoveSensor(const Entity _entity)
+{
+  auto sensorEntityIt = this->sensorEntities.find(_entity);
+  if (sensorEntityIt != this->sensorEntities.end())
+  {
+    if (this->removeSensorCb)
+      this->removeSensorCb(_entity);
+    this->sensorEntities.erase(sensorEntityIt);
+  }
+}
+
+/////////////////////////////////////////////////
+void RenderUtilPrivate::RemoveBoundingBox(const Entity _entity)
+{
+  auto wireBoxIt = this->wireBoxes.find(_entity);
+  if (wireBoxIt != this->wireBoxes.end())
+  {
+    this->scene->DestroyVisual(wireBoxIt->second->Parent());
+    this->wireBoxes.erase(wireBoxIt);
+  }
+}
+
+/////////////////////////////////////////////////
+void RenderUtilPrivate::UpdateLights(
+    const std::unordered_map<Entity, msgs::Light> &_entityLights)
+{
+  IGN_PROFILE("RenderUtil::Update Lights");
+  for (const auto &light : _entityLights)
+  {
+    auto node = this->sceneManager.NodeById(light.first);
+    if (!node)
+      continue;
+    auto l = std::dynamic_pointer_cast<rendering::Light>(node);
+    if (l)
+    {
+      // todo(ahcorde) Use the field visualize_visual in light.proto from
+      // Garden on.
+      bool visualizeVisual = true;
+      for (int i = 0; i < light.second.header().data_size(); ++i)
+      {
+        for (int j = 0;
+            j < light.second.header().data(i).value_size(); ++j)
+        {
+          if (light.second.header().data(i).key() ==
+              "visualizeVisual")
+          {
+            visualizeVisual = ignition::math::parseInt(
+              light.second.header().data(i).value(0));
+          }
+        }
+      }
+
+      rendering::VisualPtr lightVisual =
+          this->sceneManager.VisualById(
+            this->matchLightWithVisuals[light.first]);
+      if (lightVisual)
+        lightVisual->SetVisible(visualizeVisual);
+
+      // todo(ahcorde) Use the field is_light_off in light.proto from
+      // Garden on.
+      bool isLightOn = true;
+      for (int i = 0; i < light.second.header().data_size(); ++i)
+      {
+        for (int j = 0;
+            j < light.second.header().data(i).value_size(); ++j)
+        {
+          if (light.second.header().data(i).key() ==
+              "isLightOn")
+          {
+            isLightOn = ignition::math::parseInt(
+              light.second.header().data(i).value(0));
+          }
+        }
+      }
+
+      if (isLightOn)
+      {
+        if (!ignition::math::equal(
+            l->Intensity(),
+            static_cast<double>(light.second.intensity())))
+        {
+          l->SetIntensity(light.second.intensity());
+        }
+      }
+      else
+      {
+        l->SetIntensity(0);
+      }
+      if (light.second.has_diffuse())
+      {
+        if (l->DiffuseColor() != msgs::Convert(light.second.diffuse()))
+          l->SetDiffuseColor(msgs::Convert(light.second.diffuse()));
+      }
+      if (light.second.has_specular())
+      {
+        if (l->SpecularColor() != msgs::Convert(light.second.specular()))
+        {
+          l->SetSpecularColor(msgs::Convert(light.second.specular()));
+        }
+      }
+      if (!ignition::math::equal(
+          l->AttenuationRange(),
+          static_cast<double>(light.second.range())))
+      {
+        l->SetAttenuationRange(light.second.range());
+      }
+      if (!ignition::math::equal(
+          l->AttenuationLinear(),
+          static_cast<double>(light.second.attenuation_linear())))
+      {
+        l->SetAttenuationLinear(light.second.attenuation_linear());
+      }
+      if (!ignition::math::equal(
+          l->AttenuationConstant(),
+          static_cast<double>(light.second.attenuation_constant())))
+      {
+        l->SetAttenuationConstant(light.second.attenuation_constant());
+      }
+      if (!ignition::math::equal(
+          l->AttenuationQuadratic(),
+          static_cast<double>(light.second.attenuation_quadratic())))
+      {
+        l->SetAttenuationQuadratic(light.second.attenuation_quadratic());
+      }
+      if (l->CastShadows() != light.second.cast_shadows())
+        l->SetCastShadows(light.second.cast_shadows());
+      auto lDirectional =
+        std::dynamic_pointer_cast<rendering::DirectionalLight>(node);
+      if (lDirectional)
+      {
+        if (light.second.has_direction())
+        {
+          if (lDirectional->Direction() !=
+              msgs::Convert(light.second.direction()))
+          {
+            lDirectional->SetDirection(
+              msgs::Convert(light.second.direction()));
+          }
+        }
+      }
+      auto lSpotLight =
+        std::dynamic_pointer_cast<rendering::SpotLight>(node);
+      if (lSpotLight)
+      {
+        if (light.second.has_direction())
+        {
+          if (lSpotLight->Direction() !=
+            msgs::Convert(light.second.direction()))
+          {
+            lSpotLight->SetDirection(
+              msgs::Convert(light.second.direction()));
+          }
+        }
+        if (lSpotLight->InnerAngle() != light.second.spot_inner_angle())
+          lSpotLight->SetInnerAngle(light.second.spot_inner_angle());
+        if (lSpotLight->OuterAngle() != light.second.spot_outer_angle())
+          lSpotLight->SetOuterAngle(light.second.spot_outer_angle());
+        if (!ignition::math::equal(
+            lSpotLight->Falloff(),
+            static_cast<double>(light.second.spot_falloff())))
+        {
+          lSpotLight->SetFalloff(light.second.spot_falloff());
+        }
+      }
+    }
+  }
+}
+
+/////////////////////////////////////////////////
+void RenderUtilPrivate::UpdateThermalCamera(const std::unordered_map<Entity,
+    std::tuple<double, components::TemperatureRangeInfo>> &_thermalCamData)
+{
+  for (const auto &thermal : _thermalCamData)
+  {
+    Entity id = thermal.first;
+    rendering::ThermalCameraPtr camera =
+        std::dynamic_pointer_cast<rendering::ThermalCamera>(
+        this->sceneManager.NodeById(id));
+    if (camera)
+    {
+      double resolution = std::get<0>(thermal.second);
+
+      if (resolution > 0.0)
+      {
+        camera->SetLinearResolution(resolution);
+      }
+      else
+      {
+        ignwarn << "Unable to set thermal camera temperature linear resolution."
+                << " Value must be greater than 0. Using the default value: "
+                << camera->LinearResolution() << ". " << std::endl;
+      }
+      double minTemp = std::get<1>(thermal.second).min.Kelvin();
+      double maxTemp = std::get<1>(thermal.second).max.Kelvin();
+      if (maxTemp >= minTemp)
+      {
+        camera->SetMinTemperature(minTemp);
+        camera->SetMaxTemperature(maxTemp);
+      }
+      else
+      {
+        ignwarn << "Unable to set thermal camera temperature range."
+                << "Max temperature must be greater or equal to min. "
+                << "Using the default values : [" << camera->MinTemperature()
+                << ", " << camera->MaxTemperature() << "]." << std::endl;
+      }
+    }
+  }
+}
+
+/////////////////////////////////////////////////
+void RenderUtilPrivate::UpdateAnimation(const std::unordered_map<Entity,
+    AnimationUpdateData> &_actorAnimationData,
+    const std::unordered_map<Entity, math::Pose3d> &_entityPoses,
+    const std::unordered_map<Entity, math::Pose3d> &_trajectoryPoses)
+{
+  for (auto &it : _actorAnimationData)
+  {
+    auto actorMesh = this->sceneManager.ActorMeshById(it.first);
+    auto actorVisual = this->sceneManager.NodeById(it.first);
+    auto actorSkel = this->sceneManager.ActorSkeletonById(
+        it.first);
+    if (!actorMesh || !actorVisual || !actorSkel)
+    {
+      ignerr << "Actor with Entity ID '" << it.first << "'. not found. "
+             << "Skipping skeleton animation update." << std::endl;
+      continue;
+    }
+
+    const AnimationUpdateData &animData = it.second;
+    if (!animData.valid)
+    {
+      ignerr << "invalid animation update data" << std::endl;
+      continue;
+    }
+    // Enable skeleton animation
+    if (!actorMesh->SkeletonAnimationEnabled(animData.animationName))
+    {
+      // disable all animations for this actor
+      for (unsigned int i = 0; i < actorSkel->AnimationCount(); ++i)
+      {
+        actorMesh->SetSkeletonAnimationEnabled(
+            actorSkel->Animation(i)->Name(), false, false, 0.0);
+      }
+
+      // enable requested animation
+      actorMesh->SetSkeletonAnimationEnabled(
+          animData.animationName, true, animData.loop);
+
+      // Set skeleton root node weight to zero so it is not affected by
+      // the animation being played. This is needed if trajectory animation
+      // is enabled. We need to let the trajectory animation set the
+      // position of the actor instead
+      common::SkeletonPtr skeleton =
+          this->sceneManager.ActorSkeletonById(it.first);
+      if (skeleton)
+      {
+        float rootBoneWeight = (animData.followTrajectory) ? 0.0 : 1.0;
+        std::unordered_map<std::string, float> weights;
+        weights[skeleton->RootNode()->Name()] = rootBoneWeight;
+        actorMesh->SetSkeletonWeights(weights);
+      }
+    }
+    // Update skeleton animation by setting animation time.
+    // Note that animation time is different from sim time. An actor can
+    // have multiple animations. Animation time is associated with
+    // current animation that is being played. It is also adjusted if
+    // interpotate_x is enabled.
+    actorMesh->UpdateSkeletonAnimation(animData.time);
+
+    // manually update root transform in order to sync with trajectory
+    // animation
+    if (animData.followTrajectory)
+    {
+      common::SkeletonPtr skeleton =
+          this->sceneManager.ActorSkeletonById(it.first);
+      std::map<std::string, math::Matrix4d> rootTf;
+      rootTf[skeleton->RootNode()->Name()] = animData.rootTransform;
+      actorMesh->SetSkeletonLocalTransforms(rootTf);
+    }
+
+    // update actor trajectory animation
+    math::Pose3d globalPose;
+    auto entityPosesIt = _entityPoses.find(it.first);
+    if (entityPosesIt != _entityPoses.end())
+    {
+      globalPose = entityPosesIt->second;
+    }
+
+    math::Pose3d trajPose;
+    // Trajectory from the ECS
+    auto trajectoryPosesIt = _trajectoryPoses.find(it.first);
+    if (trajectoryPosesIt != _trajectoryPoses.end())
+    {
+      trajPose = trajectoryPosesIt->second;
+    }
+    else
+    {
+      // trajectory from sdf script
+      common::PoseKeyFrame poseFrame(0.0);
+      if (animData.followTrajectory)
+        animData.trajectory.Waypoints()->InterpolatedKeyFrame(poseFrame);
+      trajPose.Pos() = poseFrame.Translation();
+      trajPose.Rot() = poseFrame.Rotation();
+    }
+
+    actorVisual->SetLocalPose(trajPose + globalPose);
+  }
+}
+
 /////////////////////////////////////////////////
 void RenderUtil::ViewCollisions(const Entity &_entity)
 {
   std::vector<Entity> colEntities;
+  std::vector<Entity> links;
+
   if (this->dataPtr->linkToCollisionEntities.find(_entity) !=
       this->dataPtr->linkToCollisionEntities.end())
   {
@@ -2114,13 +2421,40 @@ void RenderUtil::ViewCollisions(const Entity &_entity)
   else if (this->dataPtr->modelToLinkEntities.find(_entity) !=
            this->dataPtr->modelToLinkEntities.end())
   {
-    std::vector<Entity> links = this->dataPtr->modelToLinkEntities[_entity];
-    for (const auto &link : links)
-      colEntities.insert(colEntities.end(),
-          this->dataPtr->linkToCollisionEntities[link].begin(),
-          this->dataPtr->linkToCollisionEntities[link].end());
+    links.insert(links.end(),
+        this->dataPtr->modelToLinkEntities[_entity].begin(),
+        this->dataPtr->modelToLinkEntities[_entity].end());
   }
 
+  if (this->dataPtr->modelToModelEntities.find(_entity) !=
+      this->dataPtr->modelToModelEntities.end())
+  {
+    std::stack<Entity> modelStack;
+    modelStack.push(_entity);
+
+    std::vector<Entity> childModels;
+    while (!modelStack.empty())
+    {
+      Entity model = modelStack.top();
+      modelStack.pop();
+
+      links.insert(links.end(),
+          this->dataPtr->modelToLinkEntities[model].begin(),
+          this->dataPtr->modelToLinkEntities[model].end());
+
+      childModels = this->dataPtr->modelToModelEntities[model];
+      for (const auto &childModel : childModels)
+      {
+        modelStack.push(childModel);
+      }
+    }
+  }
+
+  for (const auto &link : links)
+    colEntities.insert(colEntities.end(),
+        this->dataPtr->linkToCollisionEntities[link].begin(),
+        this->dataPtr->linkToCollisionEntities[link].end());
+
   // create and/or toggle collision visuals
 
   bool showCol, showColInit = false;
