@@ -21,6 +21,7 @@
 
 #include <string>
 #include <unordered_map>
+#include <unordered_set>
 #include <utility>
 
 #include <ignition/plugin/Register.hh>
@@ -49,16 +50,35 @@ using namespace systems;
 /// \brief Private AirPressure data class.
 class ignition::gazebo::systems::AirPressurePrivate
 {
-  /// \brief A map of air pressure entity to its vertical reference
+  /// \brief A map of air pressure entity to its sensor
   public: std::unordered_map<Entity,
       std::unique_ptr<sensors::AirPressureSensor>> entitySensorMap;
 
   /// \brief Ign-sensors sensor factory for creating sensors
   public: sensors::SensorFactory sensorFactory;
 
+  /// \brief Keep list of sensors that were created during the previous
+  /// `PostUpdate`, so that components can be created during the next
+  /// `PreUpdate`.
+  public: std::unordered_set<Entity> newSensors;
+
+  /// True if the rendering component is initialized
+  public: bool initialized = false;
+
+  /// \brief Create sensor
+  /// \param[in] _ecm Immutable reference to ECM.
+  /// \param[in] _entity Entity of the IMU
+  /// \param[in] _airPressure AirPressureSensor component.
+  /// \param[in] _parent Parent entity component.
+  public: void AddAirPressure(
+    const EntityComponentManager &_ecm,
+    const Entity _entity,
+    const components::AirPressureSensor *_airPressure,
+    const components::ParentEntity *_parent);
+
   /// \brief Create air pressure sensor
-  /// \param[in] _ecm Mutable reference to ECM.
-  public: void CreateAirPressureEntities(EntityComponentManager &_ecm);
+  /// \param[in] _ecm Immutable reference to ECM.
+  public: void CreateSensors(const EntityComponentManager &_ecm);
 
   /// \brief Update air pressure sensor data based on physics data
   /// \param[in] _ecm Immutable reference to ECM.
@@ -84,7 +104,21 @@ void AirPressure::PreUpdate(const UpdateInfo &/*_info*/,
     EntityComponentManager &_ecm)
 {
   IGN_PROFILE("AirPressure::PreUpdate");
-  this->dataPtr->CreateAirPressureEntities(_ecm);
+
+  // Create components
+  for (auto entity : this->dataPtr->newSensors)
+  {
+    auto it = this->dataPtr->entitySensorMap.find(entity);
+    if (it == this->dataPtr->entitySensorMap.end())
+    {
+      ignerr << "Entity [" << entity
+             << "] isn't in sensor map, this shouldn't happen." << std::endl;
+      continue;
+    }
+    // Set topic
+    _ecm.CreateComponent(entity, components::SensorTopic(it->second->Topic()));
+  }
+  this->dataPtr->newSensors.clear();
 }
 
 //////////////////////////////////////////////////
@@ -102,6 +136,8 @@ void AirPressure::PostUpdate(const UpdateInfo &_info,
         << "s]. System may not work properly." << std::endl;
   }
 
+  this->dataPtr->CreateSensors(_ecm);
+
   if (!_info.paused)
   {
     this->dataPtr->UpdateAirPressures(_ecm);
@@ -109,9 +145,7 @@ void AirPressure::PostUpdate(const UpdateInfo &_info,
     for (auto &it : this->dataPtr->entitySensorMap)
     {
       // Update measurement time
-      auto time = math::durationToSecNsec(_info.simTime);
-      dynamic_cast<sensors::Sensor *>(it.second.get())->Update(
-          math::secNsecToDuration(time.first, time.second), false);
+      it.second.get()->sensors::Sensor::Update(_info.simTime, false);
     }
   }
 
@@ -119,55 +153,78 @@ void AirPressure::PostUpdate(const UpdateInfo &_info,
 }
 
 //////////////////////////////////////////////////
-void AirPressurePrivate::CreateAirPressureEntities(EntityComponentManager &_ecm)
+void AirPressurePrivate::AddAirPressure(
+  const EntityComponentManager &_ecm,
+  const Entity _entity,
+  const components::AirPressureSensor *_airPressure,
+  const components::ParentEntity *_parent)
+{
+  // create sensor
+  std::string sensorScopedName =
+      removeParentScope(scopedName(_entity, _ecm, "::", false), "::");
+  sdf::Sensor data = _airPressure->Data();
+  data.SetName(sensorScopedName);
+  // check topic
+  if (data.Topic().empty())
+  {
+    std::string topic = scopedName(_entity, _ecm) + "/air_pressure";
+    data.SetTopic(topic);
+  }
+  std::unique_ptr<sensors::AirPressureSensor> sensor =
+      this->sensorFactory.CreateSensor<
+      sensors::AirPressureSensor>(data);
+  if (nullptr == sensor)
+  {
+    ignerr << "Failed to create sensor [" << sensorScopedName << "]"
+           << std::endl;
+    return;
+  }
+
+  // set sensor parent
+  std::string parentName = _ecm.Component<components::Name>(
+      _parent->Data())->Data();
+  sensor->SetParent(parentName);
+
+  // The WorldPose component was just created and so it's empty
+  // We'll compute the world pose manually here
+  // set sensor world pose
+  math::Pose3d sensorWorldPose = worldPose(_entity, _ecm);
+  sensor->SetPose(sensorWorldPose);
+
+  this->entitySensorMap.insert(
+      std::make_pair(_entity, std::move(sensor)));
+  this->newSensors.insert(_entity);
+}
+
+//////////////////////////////////////////////////
+void AirPressurePrivate::CreateSensors(const EntityComponentManager &_ecm)
 {
   IGN_PROFILE("AirPressurePrivate::CreateAirPressureEntities");
-  // Create air pressure sensors
-  _ecm.EachNew<components::AirPressureSensor, components::ParentEntity>(
-    [&](const Entity &_entity,
-        const components::AirPressureSensor *_airPressure,
-        const components::ParentEntity *_parent)->bool
-      {
-        // create sensor
-        std::string sensorScopedName =
-            removeParentScope(scopedName(_entity, _ecm, "::", false), "::");
-        sdf::Sensor data = _airPressure->Data();
-        data.SetName(sensorScopedName);
-        // check topic
-        if (data.Topic().empty())
+  if (!this->initialized)
+  {
+    // Create air pressure sensors
+    _ecm.Each<components::AirPressureSensor, components::ParentEntity>(
+      [&](const Entity &_entity,
+          const components::AirPressureSensor *_airPressure,
+          const components::ParentEntity *_parent)->bool
         {
-          std::string topic = scopedName(_entity, _ecm) + "/air_pressure";
-          data.SetTopic(topic);
-        }
-        std::unique_ptr<sensors::AirPressureSensor> sensor =
-            this->sensorFactory.CreateSensor<
-            sensors::AirPressureSensor>(data);
-        if (nullptr == sensor)
-        {
-          ignerr << "Failed to create sensor [" << sensorScopedName << "]"
-                 << std::endl;
+          this->AddAirPressure(_ecm, _entity, _airPressure, _parent);
           return true;
-        }
-
-        // set sensor parent
-        std::string parentName = _ecm.Component<components::Name>(
-            _parent->Data())->Data();
-        sensor->SetParent(parentName);
-
-        // The WorldPose component was just created and so it's empty
-        // We'll compute the world pose manually here
-        // set sensor world pose
-        math::Pose3d sensorWorldPose = worldPose(_entity, _ecm);
-        sensor->SetPose(sensorWorldPose);
-
-        // Set topic
-        _ecm.CreateComponent(_entity, components::SensorTopic(sensor->Topic()));
-
-        this->entitySensorMap.insert(
-            std::make_pair(_entity, std::move(sensor)));
-
-        return true;
-      });
+        });
+    this->initialized = true;
+  }
+  else
+  {
+    // Create air pressure sensors
+    _ecm.EachNew<components::AirPressureSensor, components::ParentEntity>(
+      [&](const Entity &_entity,
+          const components::AirPressureSensor *_airPressure,
+          const components::ParentEntity *_parent)->bool
+        {
+          this->AddAirPressure(_ecm, _entity, _airPressure, _parent);
+          return true;
+        });
+  }
 }
 
 //////////////////////////////////////////////////
