@@ -20,6 +20,7 @@
 #include <ignition/msgs/double.pb.h>
 
 #include <string>
+#include <unordered_set>
 
 #include <ignition/common/Profiler.hh>
 #include <ignition/math/PID.hh>
@@ -206,8 +207,17 @@ void JointPositionController::PreUpdate(
         this->dataPtr->model.JointByName(_ecm, this->dataPtr->jointName);
   }
 
+  // If the joint is still not found then warn the user, they may have entered
+  // the wrong joint name.
   if (this->dataPtr->jointEntity == kNullEntity)
+  {
+    static bool warned = false;
+    if(!warned)
+      ignerr << "Could not find joint with name ["
+        << this->dataPtr->jointName <<"]\n";
+    warned = true;
     return;
+  }
 
   // Nothing left to do if paused.
   if (_info.paused)
@@ -221,21 +231,24 @@ void JointPositionController::PreUpdate(
     _ecm.CreateComponent(
         this->dataPtr->jointEntity, components::JointPosition());
   }
-  if (jointPosComp == nullptr)
+  // We just created the joint position component, give one iteration for the
+  // physics system to update its size
+  if (jointPosComp == nullptr || jointPosComp->Data().empty())
     return;
 
   // Sanity check: Make sure that the joint index is valid.
   if (this->dataPtr->jointIndex >= jointPosComp->Data().size())
   {
-    static bool invalidJointReported = false;
-    if (!invalidJointReported)
+    static std::unordered_set<Entity> reported;
+    if (reported.find(this->dataPtr->jointEntity) == reported.end())
     {
       ignerr << "[JointPositionController]: Detected an invalid <joint_index> "
              << "parameter. The index specified is ["
-             << this->dataPtr->jointIndex << "] but the joint only has ["
+             << this->dataPtr->jointIndex << "] but joint ["
+             << this->dataPtr->jointName << "] only has ["
              << jointPosComp->Data().size() << "] index[es]. "
              << "This controller will be ignored" << std::endl;
-      invalidJointReported = true;
+      reported.insert(this->dataPtr->jointEntity);
     }
     return;
   }
