@@ -18,7 +18,7 @@
 #include "LogRecord.hh"
 
 #include <sys/stat.h>
-#include <gz/msgs/stringmsg.pb.h>
+#include <ignition/msgs/stringmsg.pb.h>
 
 #include <string>
 #include <fstream>
@@ -26,17 +26,17 @@
 #include <set>
 #include <list>
 
-#include <gz/common/Console.hh>
-#include <gz/common/Filesystem.hh>
-#include <gz/common/Profiler.hh>
-#include <gz/common/SystemPaths.hh>
-#include <gz/common/Util.hh>
-#include <gz/fuel_tools/Zip.hh>
-#include <gz/msgs/Utility.hh>
-#include <gz/plugin/Register.hh>
-#include <gz/transport/Node.hh>
-#include <gz/transport/log/Log.hh>
-#include <gz/transport/log/Recorder.hh>
+#include <ignition/common/Console.hh>
+#include <ignition/common/Filesystem.hh>
+#include <ignition/common/Profiler.hh>
+#include <ignition/common/SystemPaths.hh>
+#include <ignition/common/Util.hh>
+#include <ignition/fuel_tools/Zip.hh>
+#include <ignition/msgs/Utility.hh>
+#include <ignition/plugin/Register.hh>
+#include <ignition/transport/Node.hh>
+#include <ignition/transport/log/Log.hh>
+#include <ignition/transport/log/Recorder.hh>
 
 #include <sdf/Collision.hh>
 #include <sdf/Element.hh>
@@ -48,20 +48,21 @@
 #include <sdf/Visual.hh>
 #include <sdf/World.hh>
 
-#include "gz/sim/components/Geometry.hh"
-#include "gz/sim/components/Light.hh"
-#include "gz/sim/components/Link.hh"
-#include "gz/sim/components/Material.hh"
-#include "gz/sim/components/Model.hh"
-#include "gz/sim/components/Name.hh"
-#include "gz/sim/components/Pose.hh"
-#include "gz/sim/components/SourceFilePath.hh"
-#include "gz/sim/components/Visual.hh"
-#include "gz/sim/components/World.hh"
+#include "ignition/gazebo/components/Geometry.hh"
+#include "ignition/gazebo/components/Light.hh"
+#include "ignition/gazebo/components/Link.hh"
+#include "ignition/gazebo/components/Material.hh"
+#include "ignition/gazebo/components/Model.hh"
+#include "ignition/gazebo/components/Name.hh"
+#include "ignition/gazebo/components/Pose.hh"
+#include "ignition/gazebo/components/SourceFilePath.hh"
+#include "ignition/gazebo/components/Visual.hh"
+#include "ignition/gazebo/components/World.hh"
 
-#include "gz/sim/Util.hh"
+#include "ignition/gazebo/Util.hh"
 
-using namespace gz;
+using namespace ignition;
+using namespace ignition::gazebo;
 using namespace ignition::gazebo::systems;
 
 // Private data class.
@@ -216,7 +217,7 @@ void LogRecord::Configure(const Entity &_entity,
   //   activate one recorder.
   if (!LogRecordPrivate::started)
   {
-    auto logPath = _sdf->Get<std::string>("path");
+    auto logPath = _sdf->Get<std::string>("record_path");
     // Path is initialized by server if record is set from command line options.
     //   Otherwise, path is loaded from SDF. If a path is not specified in
     //   SDF, initialize to default here.
@@ -283,11 +284,31 @@ bool LogRecordPrivate::Start(const std::string &_logPath,
 
   // Use directory basename as topic name, to be able to retrieve at playback
   std::string sdfTopic = "/" + common::basename(this->logPath) + "/sdf";
-  this->sdfPub = this->node.Advertise(sdfTopic, this->sdfMsg.GetTypeName());
+  auto validSdfTopic = transport::TopicUtils::AsValidTopic(sdfTopic);
+  if (!validSdfTopic.empty())
+  {
+    this->sdfPub = this->node.Advertise(validSdfTopic,
+        this->sdfMsg.GetTypeName());
+  }
+  else
+  {
+    ignerr << "Failed to generate valid topic to publish SDF. Tried ["
+           << sdfTopic << "]." << std::endl;
+  }
 
   // TODO(louise) Combine with SceneBroadcaster's state topic
   std::string stateTopic = "/world/" + this->worldName + "/changed_state";
-  this->statePub = this->node.Advertise<msgs::SerializedStateMap>(stateTopic);
+  auto validStateTopic = transport::TopicUtils::AsValidTopic(stateTopic);
+  if (!validStateTopic.empty())
+  {
+    this->statePub = this->node.Advertise<msgs::SerializedStateMap>(
+        validStateTopic);
+  }
+  else
+  {
+    ignerr << "Failed to generate valid topic to publish state. Tried ["
+           << stateTopic << "]." << std::endl;
+  }
 
   // Append file name
   std::string dbPath = common::joinPaths(this->logPath, "state.tlog");
@@ -344,7 +365,7 @@ bool LogRecordPrivate::Start(const std::string &_logPath,
 
   // This calls Log::Open() and loads sql schema
   if (this->recorder.Start(dbPath) ==
-      transport::log::RecorderError::SUCCESS)
+      ignition::transport::log::RecorderError::SUCCESS)
   {
     this->instStarted = true;
     return true;
@@ -527,44 +548,41 @@ bool LogRecordPrivate::SaveModels(const std::set<std::string> &_models)
 
       // Look for URIs in SDF and convert them to paths relative to the model
       // directory
-      for (uint64_t mi = 0; mi < root.ModelCount(); mi++)
+      const sdf::Model *model = root.Model();
+      for (uint64_t li = 0; li < model->LinkCount(); li++)
       {
-        const sdf::Model *model = root.ModelByIndex(mi);
-        for (uint64_t li = 0; li < model->LinkCount(); li++)
+        const sdf::Link *link = model->LinkByIndex(li);
+        for (uint64_t ci = 0; ci < link->CollisionCount(); ci++)
         {
-          const sdf::Link *link = model->LinkByIndex(li);
-          for (uint64_t ci = 0; ci < link->CollisionCount(); ci++)
+          const sdf::Collision *collision = link->CollisionByIndex(ci);
+          const sdf::Geometry *geometry = collision->Geom();
+          const sdf::Mesh *mesh = geometry->MeshShape();
+          if (mesh != nullptr)
           {
-            const sdf::Collision *collision = link->CollisionByIndex(ci);
-            const sdf::Geometry *geometry = collision->Geom();
-            const sdf::Mesh *mesh = geometry->MeshShape();
-            if (mesh != nullptr)
+            // Replace path with relative path
+            std::string relPath = convertToRelativePath(mesh->Uri(), srcPath);
+            sdf::ElementPtr meshElem = mesh->Element();
+            if (meshElem->HasElement("uri"))
             {
-              // Replace path with relative path
-              std::string relPath = convertToRelativePath(mesh->Uri(), srcPath);
-              sdf::ElementPtr meshElem = mesh->Element();
-              if (meshElem->HasElement("uri"))
-              {
-                sdf::ElementPtr uriElem = meshElem->GetElement("uri");
-                uriElem->Set(relPath);
-              }
+              sdf::ElementPtr uriElem = meshElem->GetElement("uri");
+              uriElem->Set(relPath);
             }
           }
-          for (uint64_t vi = 0; vi < link->VisualCount(); vi++)
+        }
+        for (uint64_t vi = 0; vi < link->VisualCount(); vi++)
+        {
+          const sdf::Visual *visual = link->VisualByIndex(vi);
+          const sdf::Geometry *geometry = visual->Geom();
+          const sdf::Mesh *mesh = geometry->MeshShape();
+          if (mesh != nullptr)
           {
-            const sdf::Visual *visual = link->VisualByIndex(vi);
-            const sdf::Geometry *geometry = visual->Geom();
-            const sdf::Mesh *mesh = geometry->MeshShape();
-            if (mesh != nullptr)
+            // Replace path with relative path
+            std::string relPath = convertToRelativePath(mesh->Uri(), srcPath);
+            sdf::ElementPtr meshElem = mesh->Element();
+            if (meshElem->HasElement("uri"))
             {
-              // Replace path with relative path
-              std::string relPath = convertToRelativePath(mesh->Uri(), srcPath);
-              sdf::ElementPtr meshElem = mesh->Element();
-              if (meshElem->HasElement("uri"))
-              {
-                sdf::ElementPtr uriElem = meshElem->GetElement("uri");
-                uriElem->Set(relPath);
-              }
+              sdf::ElementPtr uriElem = meshElem->GetElement("uri");
+              uriElem->Set(relPath);
             }
           }
         }
@@ -694,15 +712,11 @@ void LogRecord::PostUpdate(const UpdateInfo &_info,
     this->dataPtr->LogModelResources(_ecm);
 }
 
-IGNITION_ADD_PLUGIN(LogRecord,
-                    gz::sim::System,
+IGNITION_ADD_PLUGIN(ignition::gazebo::systems::LogRecord,
+                    ignition::gazebo::System,
                     LogRecord::ISystemConfigure,
                     LogRecord::ISystemPreUpdate,
                     LogRecord::ISystemPostUpdate)
 
-IGNITION_ADD_PLUGIN_ALIAS(LogRecord,
-                          "gz::sim::systems::LogRecord")
-
-// TODO(CH3): Deprecated, remove on version 8
-IGNITION_ADD_PLUGIN_ALIAS(LogRecord,
+IGNITION_ADD_PLUGIN_ALIAS(ignition::gazebo::systems::LogRecord,
                           "ignition::gazebo::systems::LogRecord")
