@@ -19,6 +19,7 @@
 
 #include <string>
 #include <unordered_map>
+#include <unordered_set>
 #include <utility>
 
 #include <ignition/plugin/Register.hh>
@@ -56,9 +57,30 @@ class ignition::gazebo::systems::MagnetometerPrivate
   /// \brief Ign-sensors sensor factory for creating sensors
   public: sensors::SensorFactory sensorFactory;
 
+  /// \brief Keep list of sensors that were created during the previous
+  /// `PostUpdate`, so that components can be created during the next
+  /// `PreUpdate`.
+  public: std::unordered_set<Entity> newSensors;
+
+  /// True if the rendering component is initialized
+  public: bool initialized = false;
+
+  /// \brief Create sensor
+  /// \param[in] _ecm Immutable reference to ECM.
+  /// \param[in] _entity Entity of the IMU
+  /// \param[in] _magnetometer Magnetometer component.
+  /// \param[in] _worldField MagneticField component.
+  /// \param[in] _parent Parent entity component.
+  public: void AddMagnetometer(
+    const EntityComponentManager &_ecm,
+    const Entity _entity,
+    const components::Magnetometer *_magnetometer,
+    const components::MagneticField *_worldField,
+    const components::ParentEntity *_parent);
+
   /// \brief Create magnetometer sensor
-  /// \param[in] _ecm Mutable reference to ECM.
-  public: void CreateMagnetometerEntities(EntityComponentManager &_ecm);
+  /// \param[in] _ecm Immutable reference to ECM.
+  public: void CreateSensors(const EntityComponentManager &_ecm);
 
   /// \brief Update magnetometer sensor data based on physics data
   /// \param[in] _ecm Immutable reference to ECM.
@@ -84,7 +106,21 @@ void Magnetometer::PreUpdate(const UpdateInfo &/*_info*/,
     EntityComponentManager &_ecm)
 {
   IGN_PROFILE("Magnetometer::PreUpdate");
-  this->dataPtr->CreateMagnetometerEntities(_ecm);
+
+  // Create components
+  for (auto entity : this->dataPtr->newSensors)
+  {
+    auto it = this->dataPtr->entitySensorMap.find(entity);
+    if (it == this->dataPtr->entitySensorMap.end())
+    {
+      ignerr << "Entity [" << entity
+             << "] isn't in sensor map, this shouldn't happen." << std::endl;
+      continue;
+    }
+    // Set topic
+    _ecm.CreateComponent(entity, components::SensorTopic(it->second->Topic()));
+  }
+  this->dataPtr->newSensors.clear();
 }
 
 //////////////////////////////////////////////////
@@ -101,6 +137,8 @@ void Magnetometer::PostUpdate(const UpdateInfo &_info,
         << "s]. System may not work properly." << std::endl;
   }
 
+  this->dataPtr->CreateSensors(_ecm);
+
   // Only update and publish if not paused.
   if (!_info.paused)
   {
@@ -109,9 +147,7 @@ void Magnetometer::PostUpdate(const UpdateInfo &_info,
     for (auto &it : this->dataPtr->entitySensorMap)
     {
       // Update measurement time
-      auto time = math::durationToSecNsec(_info.simTime);
-      dynamic_cast<sensors::Sensor *>(it.second.get())->Update(
-          math::secNsecToDuration(time.first, time.second), false);
+      it.second.get()->sensors::Sensor::Update(_info.simTime, false);
     }
   }
 
@@ -119,8 +155,56 @@ void Magnetometer::PostUpdate(const UpdateInfo &_info,
 }
 
 //////////////////////////////////////////////////
-void MagnetometerPrivate::CreateMagnetometerEntities(
-    EntityComponentManager &_ecm)
+void MagnetometerPrivate::AddMagnetometer(
+  const EntityComponentManager &_ecm,
+  const Entity _entity,
+  const components::Magnetometer *_magnetometer,
+  const components::MagneticField *_worldField,
+  const components::ParentEntity *_parent)
+{
+  // create sensor
+  std::string sensorScopedName =
+      removeParentScope(scopedName(_entity, _ecm, "::", false), "::");
+  sdf::Sensor data = _magnetometer->Data();
+  data.SetName(sensorScopedName);
+  // check topic
+  if (data.Topic().empty())
+  {
+    std::string topic = scopedName(_entity, _ecm) + "/magnetometer";
+    data.SetTopic(topic);
+  }
+  std::unique_ptr<sensors::MagnetometerSensor> sensor =
+      this->sensorFactory.CreateSensor<
+      sensors::MagnetometerSensor>(data);
+  if (nullptr == sensor)
+  {
+    ignerr << "Failed to create sensor [" << sensorScopedName << "]"
+           << std::endl;
+    return;
+  }
+
+  // set sensor parent
+  std::string parentName = _ecm.Component<components::Name>(
+      _parent->Data())->Data();
+  sensor->SetParent(parentName);
+
+  // set world magnetic field. Assume uniform in world and does not
+  // change throughout simulation
+  sensor->SetWorldMagneticField(_worldField->Data());
+
+  // Get initial pose of sensor and set the reference z pos
+  // The WorldPose component was just created and so it's empty
+  // We'll compute the world pose manually here
+  math::Pose3d p = worldPose(_entity, _ecm);
+  sensor->SetWorldPose(p);
+
+  this->entitySensorMap.insert(
+      std::make_pair(_entity, std::move(sensor)));
+  this->newSensors.insert(_entity);
+}
+
+//////////////////////////////////////////////////
+void MagnetometerPrivate::CreateSensors(const EntityComponentManager &_ecm)
 {
   IGN_PROFILE("MagnetometerPrivate::CreateMagnetometerEntities");
   auto worldEntity = _ecm.EntityByComponents(components::World());
@@ -138,56 +222,33 @@ void MagnetometerPrivate::CreateMagnetometerEntities(
     return;
   }
 
-  // Create magnetometers
-  _ecm.EachNew<components::Magnetometer, components::ParentEntity>(
-    [&](const Entity &_entity,
-        const components::Magnetometer *_magnetometer,
-        const components::ParentEntity *_parent)->bool
-      {
-        // create sensor
-        std::string sensorScopedName =
-            removeParentScope(scopedName(_entity, _ecm, "::", false), "::");
-        sdf::Sensor data = _magnetometer->Data();
-        data.SetName(sensorScopedName);
-        // check topic
-        if (data.Topic().empty())
+  if (!this->initialized)
+  {
+    // Create magnetometers
+    _ecm.Each<components::Magnetometer, components::ParentEntity>(
+      [&](const Entity &_entity,
+          const components::Magnetometer *_magnetometer,
+          const components::ParentEntity *_parent)->bool
         {
-          std::string topic = scopedName(_entity, _ecm) + "/magnetometer";
-          data.SetTopic(topic);
-        }
-        std::unique_ptr<sensors::MagnetometerSensor> sensor =
-            this->sensorFactory.CreateSensor<
-            sensors::MagnetometerSensor>(data);
-        if (nullptr == sensor)
-        {
-          ignerr << "Failed to create sensor [" << sensorScopedName << "]"
-                 << std::endl;
+          this->AddMagnetometer(_ecm, _entity, _magnetometer, worldField,
+              _parent);
           return true;
-        }
-
-        // set sensor parent
-        std::string parentName = _ecm.Component<components::Name>(
-            _parent->Data())->Data();
-        sensor->SetParent(parentName);
-
-        // set world magnetic field. Assume uniform in world and does not
-        // change throughout simulation
-        sensor->SetWorldMagneticField(worldField->Data());
-
-        // Get initial pose of sensor and set the reference z pos
-        // The WorldPose component was just created and so it's empty
-        // We'll compute the world pose manually here
-        math::Pose3d p = worldPose(_entity, _ecm);
-        sensor->SetWorldPose(p);
-
-        // Set topic
-        _ecm.CreateComponent(_entity, components::SensorTopic(sensor->Topic()));
-
-        this->entitySensorMap.insert(
-            std::make_pair(_entity, std::move(sensor)));
-
-        return true;
-      });
+        });
+    this->initialized = true;
+  }
+  else
+  {
+    // Create magnetometers
+    _ecm.EachNew<components::Magnetometer, components::ParentEntity>(
+      [&](const Entity &_entity,
+          const components::Magnetometer *_magnetometer,
+          const components::ParentEntity *_parent)->bool
+        {
+          this->AddMagnetometer(_ecm, _entity, _magnetometer, worldField,
+              _parent);
+          return true;
+        });
+  }
 }
 
 //////////////////////////////////////////////////
