@@ -24,6 +24,7 @@
 #include <ignition/common/Util.hh>
 #include <ignition/math/Pose3.hh>
 #include <ignition/transport/Node.hh>
+#include <ignition/utilities/ExtraTestMacros.hh>
 
 #include "ignition/gazebo/components/Altimeter.hh"
 #include "ignition/gazebo/components/LinearVelocity.hh"
@@ -36,20 +37,14 @@
 #include "ignition/gazebo/test_config.hh"
 
 #include "../helpers/Relay.hh"
+#include "../helpers/EnvTestFixture.hh"
 
 using namespace ignition;
 using namespace gazebo;
 
 /// \brief Test AltimeterTest system
-class AltimeterTest : public ::testing::Test
+class AltimeterTest : public InternalFixture<::testing::Test>
 {
-  // Documentation inherited
-  protected: void SetUp() override
-  {
-    ignition::common::Console::SetVerbosity(4);
-    ignition::common::setenv("IGN_GAZEBO_SYSTEM_PLUGIN_PATH",
-           (std::string(PROJECT_BINARY_PATH) + "/lib").c_str());
-  }
 };
 
 std::mutex mutex;
@@ -65,7 +60,8 @@ void altimeterCb(const msgs::Altimeter &_msg)
 
 /////////////////////////////////////////////////
 // The test checks the world pose and sensor readings of a falling altimeter
-TEST_F(AltimeterTest, ModelFalling)
+// See https://github.com/ignitionrobotics/ign-gazebo/issues/1175
+TEST_F(AltimeterTest, IGN_UTILS_TEST_DISABLED_ON_WIN32(ModelFalling))
 {
   // Start server
   ServerConfig serverConfig;
@@ -86,7 +82,7 @@ TEST_F(AltimeterTest, ModelFalling)
   test::Relay testSystem;
   std::vector<math::Pose3d> poses;
   std::vector<math::Vector3d> velocities;
-  testSystem.OnPostUpdate([&](const gazebo::UpdateInfo &,
+  testSystem.OnPostUpdate([&](const gazebo::UpdateInfo &_info,
                               const gazebo::EntityComponentManager &_ecm)
       {
         _ecm.Each<components::Altimeter, components::Name,
@@ -106,6 +102,10 @@ TEST_F(AltimeterTest, ModelFalling)
               auto sensorComp = _ecm.Component<components::Sensor>(_entity);
               EXPECT_NE(nullptr, sensorComp);
 
+              if (_info.iterations == 1)
+                return true;
+
+              // This component is created on the 2nd PreUpdate
               auto topicComp = _ecm.Component<components::SensorTopic>(_entity);
               EXPECT_NE(nullptr, topicComp);
               if (topicComp)
@@ -129,9 +129,10 @@ TEST_F(AltimeterTest, ModelFalling)
   EXPECT_EQ(iters100, poses.size());
 
   // Wait for messages to be received
-  double updateRate = 30;
+  size_t updateRate = 30;
   double stepSize = 0.001;
-  size_t waitForMsgs = poses.size() * stepSize * updateRate + 1;
+  size_t waitForMsgs =
+      static_cast<size_t>(poses.size() * stepSize * updateRate + 1);
   for (int sleep = 0; sleep < 30; ++sleep)
   {
     std::this_thread::sleep_for(std::chrono::milliseconds(100));
@@ -176,7 +177,7 @@ TEST_F(AltimeterTest, ModelFalling)
   EXPECT_EQ(iters100 + iters1000, poses.size());
 
   // Wait for messages to be received
-  waitForMsgs = poses.size() * stepSize * updateRate + 1;
+  waitForMsgs = static_cast<size_t>(poses.size() * stepSize * updateRate + 1);
   for (int sleep = 0; sleep < 30; ++sleep)
   {
     std::this_thread::sleep_for(std::chrono::milliseconds(100));
