@@ -19,9 +19,12 @@
 
 #include <ignition/common/Console.hh>
 #include <ignition/common/Util.hh>
+#include <ignition/utilities/ExtraTestMacros.hh>
 
+#include "ignition/gazebo/Util.hh"
 #include "ignition/gazebo/Server.hh"
 #include "ignition/gazebo/SystemLoader.hh"
+#include "ignition/gazebo/TestFixture.hh"
 #include "ignition/gazebo/components/CenterOfVolume.hh"
 #include "ignition/gazebo/components/Link.hh"
 #include "ignition/gazebo/components/Name.hh"
@@ -32,23 +35,18 @@
 
 #include "ignition/gazebo/test_config.hh"
 #include "../helpers/Relay.hh"
+#include "../helpers/EnvTestFixture.hh"
 
 using namespace ignition;
 using namespace gazebo;
 
-class BuoyancyTest : public ::testing::Test
+class BuoyancyTest : public InternalFixture<::testing::Test>
 {
-  // Documentation inherited
-  protected: void SetUp() override
-  {
-    ignition::common::Console::SetVerbosity(4);
-    ignition::common::setenv("IGN_GAZEBO_SYSTEM_PLUGIN_PATH",
-           (std::string(PROJECT_BINARY_PATH) + "/lib").c_str());
-  }
 };
 
 /////////////////////////////////////////////////
-TEST_F(BuoyancyTest, Movement)
+// See https://github.com/ignitionrobotics/ign-gazebo/issues/1175
+TEST_F(BuoyancyTest, IGN_UTILS_TEST_DISABLED_ON_WIN32(Movement))
 {
   // Start server
   ServerConfig serverConfig;
@@ -200,3 +198,77 @@ TEST_F(BuoyancyTest, Movement)
   server.Run(true, iterations, false);
   EXPECT_TRUE(finished);
 }
+
+/////////////////////////////////////////////////
+TEST_F(BuoyancyTest, IGN_UTILS_TEST_DISABLED_ON_WIN32(OffsetAndRotation))
+{
+  TestFixture fixture(common::joinPaths(std::string(PROJECT_SOURCE_PATH),
+    "test", "worlds", "center_of_volume.sdf"));
+
+  std::size_t iterations{0};
+  fixture.OnPostUpdate([&](
+      const gazebo::UpdateInfo &,
+      const gazebo::EntityComponentManager &_ecm)
+  {
+    // Get links
+    auto noOffsets = entitiesFromScopedName("no_offset::link", _ecm);
+    ASSERT_EQ(1u, noOffsets.size());
+    auto noOffset = *noOffsets.begin();
+    EXPECT_NE(kNullEntity, noOffset);
+
+    auto noOffsetRotateds = entitiesFromScopedName("no_offset_rotated::link",
+        _ecm);
+    ASSERT_EQ(1u, noOffsetRotateds.size());
+    auto noOffsetRotated = *noOffsetRotateds.begin();
+    EXPECT_NE(kNullEntity, noOffsetRotated);
+
+    auto withOffsets = entitiesFromScopedName("com_cov_offset::link", _ecm);
+    ASSERT_EQ(1u, withOffsets.size());
+    auto withOffset = *withOffsets.begin();
+    EXPECT_NE(kNullEntity, withOffset);
+
+    auto withOffsetRotateds = entitiesFromScopedName(
+        "com_cov_offset_rotated::link", _ecm);
+    ASSERT_EQ(1u, withOffsetRotateds.size());
+    auto withOffsetRotated = *withOffsetRotateds.begin();
+    EXPECT_NE(kNullEntity, withOffsetRotated);
+
+    // Check CoVs have correct offsets
+    auto noOffsetCoV = _ecm.Component<components::CenterOfVolume>(noOffset);
+    ASSERT_NE(noOffsetCoV, nullptr);
+    EXPECT_EQ(math::Vector3d::Zero, noOffsetCoV->Data());
+
+    auto noOffsetRotatedCoV = _ecm.Component<components::CenterOfVolume>(
+        noOffsetRotated);
+    ASSERT_NE(noOffsetRotatedCoV, nullptr);
+    EXPECT_EQ(math::Vector3d::Zero, noOffsetRotatedCoV->Data());
+
+    auto withOffsetCoV = _ecm.Component<components::CenterOfVolume>(withOffset);
+    ASSERT_NE(withOffsetCoV, nullptr);
+    EXPECT_EQ(math::Vector3d::One, withOffsetCoV->Data());
+
+    auto withOffsetRotatedCoV = _ecm.Component<components::CenterOfVolume>(
+        withOffsetRotated);
+    ASSERT_NE(withOffsetRotatedCoV, nullptr);
+    EXPECT_EQ(math::Vector3d::One, withOffsetRotatedCoV->Data());
+
+    // Check that all objects are neutrally buoyant and stay still
+    auto noOffsetPose = worldPose(noOffset, _ecm);
+    EXPECT_EQ(math::Pose3d(), noOffsetPose);
+
+    auto noOffsetRotatedPose = worldPose(noOffsetRotated, _ecm);
+    EXPECT_EQ(math::Pose3d(-3, 0, 0, 0.1, 0.2, 0.3), noOffsetRotatedPose);
+
+    auto withOffsetPose = worldPose(withOffset, _ecm);
+    EXPECT_EQ(math::Pose3d(0, 3, 0, 0, 0, 0), withOffsetPose);
+
+    auto withOffsetRotatedPose = worldPose(withOffsetRotated, _ecm);
+    EXPECT_EQ(math::Pose3d(-3, 3, 0, 0.1, 0.2, 0.3), withOffsetRotatedPose);
+
+    iterations++;
+  }).Finalize();
+
+  std::size_t targetIterations{1000};
+  fixture.Server()->Run(true, targetIterations, false);
+  EXPECT_EQ(targetIterations, iterations);
+}
