@@ -24,30 +24,27 @@
 #include <ignition/common/Console.hh>
 #include <ignition/common/Util.hh>
 #include <ignition/transport/Node.hh>
+#include <ignition/utilities/ExtraTestMacros.hh>
 
 #include "ignition/gazebo/Server.hh"
 #include "ignition/gazebo/SystemLoader.hh"
 #include "ignition/gazebo/test_config.hh"
 
 #include "plugins/MockSystem.hh"
+#include "../helpers/EnvTestFixture.hh"
 
 using namespace ignition;
 using namespace gazebo;
 
-class ContactSystemTest : public ::testing::Test
+class ContactSystemTest : public InternalFixture<::testing::Test>
 {
-  // Documentation inherited
-  protected: void SetUp() override
-  {
-    ignition::common::Console::SetVerbosity(4);
-    ignition::common::setenv("IGN_GAZEBO_SYSTEM_PLUGIN_PATH",
-           (std::string(PROJECT_BINARY_PATH) + "/lib").c_str());
-  }
 };
 
 /////////////////////////////////////////////////
 // The test checks that contacts are published by the contact system
-TEST_F(ContactSystemTest, MultipleCollisionsAsContactSensors)
+// See https://github.com/ignitionrobotics/ign-gazebo/issues/1175
+TEST_F(ContactSystemTest,
+       IGN_UTILS_TEST_DISABLED_ON_WIN32(MultipleCollisionsAsContactSensors))
 {
   // Start server
   ServerConfig serverConfig;
@@ -120,7 +117,8 @@ TEST_F(ContactSystemTest, MultipleCollisionsAsContactSensors)
   }
 }
 
-TEST_F(ContactSystemTest, RemoveContactSensor)
+TEST_F(ContactSystemTest,
+       IGN_UTILS_TEST_DISABLED_ON_WIN32(RemoveContactSensor))
 {
   // Start server
   ServerConfig serverConfig;
