@@ -21,8 +21,11 @@
 #include <ignition/common/Console.hh>
 #include <ignition/common/Util.hh>
 #include <ignition/common/Filesystem.hh>
+#include <ignition/math/SemanticVersion.hh>
+#include <ignition/utilities/ExtraTestMacros.hh>
 
 #include "ignition/gazebo/test_config.hh"
+#include "../helpers/EnvTestFixture.hh"
 
 // File copied from
 // https://github.com/ignitionrobotics/ign-gui/raw/ign-gui3/test/integration/ExamplesBuild_TEST.cc
@@ -120,7 +123,8 @@ bool createAndSwitchToTempDir(std::string &_newTempPath)
 #endif
 
 //////////////////////////////////////////////////
-class ExamplesBuild : public ::testing::TestWithParam<const char*>
+class ExamplesBuild
+  : public InternalFixture<::testing::TestWithParam<const char*>>
 {
   /// \brief Build code in a temporary build folder.
   /// \param[in] _type Type of example to build (plugins, standalone).
@@ -143,6 +147,14 @@ void ExamplesBuild::Build(const std::string &_type)
   {
     auto base = ignition::common::basename(*dirIter);
 
+    math::SemanticVersion cmakeVersion{std::string(CMAKE_VERSION)};
+    if (base == "gtest_setup" && cmakeVersion < math::SemanticVersion(3, 11, 0))
+    {
+      igndbg << "Skipping [gtest_setup] test, which requires CMake version >= "
+             << "3.11.0. Currently using CMake " << cmakeVersion << std::endl;
+      continue;
+    }
+
     // Source directory for this example
     auto sourceDir = examplesDir;
     sourceDir += "/" + base;
@@ -168,7 +180,8 @@ void ExamplesBuild::Build(const std::string &_type)
 }
 
 //////////////////////////////////////////////////
-TEST_P(ExamplesBuild, Build)
+// See https://github.com/ignitionrobotics/ign-gazebo/issues/1175
+TEST_P(ExamplesBuild, IGN_UTILS_TEST_DISABLED_ON_WIN32(Build))
 {
   Build(GetParam());
 }
