@@ -23,6 +23,7 @@
 #include <ignition/common/Util.hh>
 #include <ignition/math/Pose3.hh>
 #include <ignition/transport/Node.hh>
+#include <ignition/utilities/ExtraTestMacros.hh>
 
 #include "ignition/gazebo/components/AngularVelocity.hh"
 #include "ignition/gazebo/components/Gravity.hh"
@@ -37,6 +38,7 @@
 #include "ignition/gazebo/test_config.hh"
 
 #include "../helpers/Relay.hh"
+#include "../helpers/EnvTestFixture.hh"
 
 #define TOL 1e-4
 
@@ -44,15 +46,8 @@ using namespace ignition;
 using namespace gazebo;
 
 /// \brief Test ImuTest system
-class ImuTest : public ::testing::Test
+class ImuTest : public InternalFixture<::testing::Test>
 {
-  // Documentation inherited
-  protected: void SetUp() override
-  {
-    ignition::common::Console::SetVerbosity(4);
-    ignition::common::setenv("IGN_GAZEBO_SYSTEM_PLUGIN_PATH",
-           (std::string(PROJECT_BINARY_PATH) + "/lib").c_str());
-  }
 };
 
 std::mutex mutex;
@@ -68,7 +63,8 @@ void imuCb(const msgs::IMU &_msg)
 
 /////////////////////////////////////////////////
 // The test checks the world pose and sensor readings of a falling imu
-TEST_F(ImuTest, ModelFalling)
+// See https://github.com/ignitionrobotics/ign-gazebo/issues/1175
+TEST_F(ImuTest, IGN_UTILS_TEST_DISABLED_ON_WIN32(ModelFalling))
 {
   double z = 3;
   // TODO(anyone): get step size from sdf
@@ -95,7 +91,7 @@ TEST_F(ImuTest, ModelFalling)
   std::vector<math::Pose3d> poses;
   std::vector<math::Vector3d> accelerations;
   std::vector<math::Vector3d> angularVelocities;
-  testSystem.OnPostUpdate([&](const gazebo::UpdateInfo &,
+  testSystem.OnPostUpdate([&](const gazebo::UpdateInfo &_info,
                               const gazebo::EntityComponentManager &_ecm)
       {
         _ecm.Each<components::Imu,
@@ -119,6 +115,10 @@ TEST_F(ImuTest, ModelFalling)
               auto sensorComp = _ecm.Component<components::Sensor>(_entity);
               EXPECT_NE(nullptr, sensorComp);
 
+              if (_info.iterations == 1)
+                return true;
+
+              // This component is created on the 2nd PreUpdate
               auto topicComp = _ecm.Component<components::SensorTopic>(_entity);
               EXPECT_NE(nullptr, topicComp);
               if (topicComp)
@@ -185,7 +185,7 @@ TEST_F(ImuTest, ModelFalling)
   // Time to advance, allow 0.5 s settling time.
   // This assumes inelastic collisions with the ground.
   double dtHit = tHit + 0.5 - (iters200 + 1) * stepSize;
-  double steps = ceil(dtHit / stepSize);
+  int steps = static_cast<int>(ceil(dtHit / stepSize));
   ASSERT_GT(steps, 0);
   server.Run(true, steps, false);
 
@@ -212,3 +212,41 @@ TEST_F(ImuTest, ModelFalling)
   EXPECT_EQ(imuMsgs.back().entity_name(), scopedName);
   mutex.unlock();
 }
+
+/////////////////////////////////////////////////
+// The test checks to make sure orientation is not published if it is deabled
+TEST_F(ImuTest, IGN_UTILS_TEST_DISABLED_ON_WIN32(OrientationDisabled))
+{
+  imuMsgs.clear();
+
+  // Start server
+  ServerConfig serverConfig;
+  const auto sdfFile = common::joinPaths(std::string(PROJECT_SOURCE_PATH),
+    "test", "worlds", "imu_no_orientation.sdf");
+  serverConfig.SetSdfFile(sdfFile);
+
+  Server server(serverConfig);
+  EXPECT_FALSE(server.Running());
+  EXPECT_FALSE(*server.Running(0));
+
+  auto topic =
+      "world/imu_sensor/model/imu_model/link/link/sensor/imu_sensor/imu";
+
+  // subscribe to imu topic
+  transport::Node node;
+  node.Subscribe(topic, &imuCb);
+
+  // step world and verify imu's orientation is not published
+  // Run server
+  size_t iters200 = 200u;
+  server.Run(true, iters200, false);
+
+  // Check we received messages
+  EXPECT_GT(imuMsgs.size(), 0u);
+  mutex.lock();
+  for (const auto &msg : imuMsgs)
+  {
+    EXPECT_FALSE(msg.has_orientation());
+  }
+  mutex.unlock();
+}
