@@ -37,6 +37,7 @@
 #include <ignition/transport/log/Playback.hh>
 #include <ignition/transport/log/QualifiedTime.hh>
 #include <ignition/math/Pose3.hh>
+#include <ignition/utilities/ExtraTestMacros.hh>
 
 #include <sdf/Root.hh>
 #include <sdf/World.hh>
@@ -51,6 +52,7 @@
 #include "ignition/gazebo/test_config.hh"
 
 #include "../helpers/Relay.hh"
+#include "../helpers/EnvTestFixture.hh"
 
 using namespace ignition;
 using namespace gazebo;
@@ -147,16 +149,8 @@ void entryDiff(std::vector<std::string> &_paths1,
 #endif
 
 //////////////////////////////////////////////////
-class LogSystemTest : public ::testing::Test
+class LogSystemTest : public InternalFixture<::testing::Test>
 {
-  // Documentation inherited
-  protected: void SetUp() override
-  {
-    common::Console::SetVerbosity(4);
-    ignition::common::setenv("IGN_GAZEBO_SYSTEM_PLUGIN_PATH",
-           (std::string(PROJECT_BINARY_PATH) + "/lib").c_str());
-  }
-
   // Create a temporary directory in build path for recorded data
   public: void CreateLogsDir()
   {
@@ -269,7 +263,8 @@ class LogSystemTest : public ::testing::Test
 };
 
 /////////////////////////////////////////////////
-TEST_F(LogSystemTest, LogPlaybackStatistics)
+// See https://github.com/ignitionrobotics/ign-gazebo/issues/1175
+TEST_F(LogSystemTest, IGN_UTILS_TEST_DISABLED_ON_WIN32(LogPlaybackStatistics))
 {
   auto logPath = common::joinPaths(PROJECT_SOURCE_PATH, "test", "media",
       "rolling_shapes_log");
@@ -320,7 +315,7 @@ TEST_F(LogSystemTest, LogPlaybackStatistics)
 
 /////////////////////////////////////////////////
 // Logging behavior when no paths are specified
-TEST_F(LogSystemTest, LogDefaults)
+TEST_F(LogSystemTest, IGN_UTILS_TEST_DISABLED_ON_WIN32(LogDefaults))
 {
   // Create temp directory to store log
   this->CreateLogsDir();
@@ -424,7 +419,7 @@ TEST_F(LogSystemTest, LogDefaults)
 /////////////////////////////////////////////////
 // Logging behavior when a path is specified either via the C++ API, SDF, or
 // the command line.
-TEST_F(LogSystemTest, LogPaths)
+TEST_F(LogSystemTest, IGN_UTILS_TEST_DISABLED_ON_WIN32(LogPaths))
 {
   // Create temp directory to store log
   this->CreateLogsDir();
@@ -693,7 +688,7 @@ TEST_F(LogSystemTest, LogPaths)
 }
 
 /////////////////////////////////////////////////
-TEST_F(LogSystemTest, RecordAndPlayback)
+TEST_F(LogSystemTest, IGN_UTILS_TEST_DISABLED_ON_WIN32(RecordAndPlayback))
 {
   // Create temp directory to store log
   this->CreateLogsDir();
@@ -780,7 +775,27 @@ TEST_F(LogSystemTest, RecordAndPlayback)
   stateMsg.ParseFromString(recordedIter->Data());
   // entity size = 28 in dbl pendulum + 4 in nested model
   EXPECT_EQ(32, stateMsg.entities_size());
-  EXPECT_EQ(batch.end(), ++recordedIter);
+  EXPECT_NE(batch.end(), ++recordedIter);
+
+  // check rest of recordIter (state message) for poses
+  while (batch.end() != recordedIter)
+  {
+    EXPECT_EQ("ignition.msgs.SerializedStateMap", recordedIter->Type());
+    EXPECT_EQ(recordedIter->Topic(), "/world/log_pendulum/changed_state");
+
+    stateMsg.ParseFromString(recordedIter->Data());
+
+    for (const auto &entityIter : stateMsg.entities())
+    {
+      for (const auto &compIter : entityIter.second.components())
+      {
+        EXPECT_EQ(components::Pose::typeId, compIter.second.type());
+      }
+    }
+    ++recordedIter;
+  }
+
+  EXPECT_EQ(batch.end(), recordedIter);
 
   // Keep track of total number of pose comparisons
   int nTotal{0};
@@ -895,7 +910,7 @@ TEST_F(LogSystemTest, RecordAndPlayback)
 }
 
 /////////////////////////////////////////////////
-TEST_F(LogSystemTest, LogControl)
+TEST_F(LogSystemTest, IGN_UTILS_TEST_DISABLED_ON_WIN32(LogControl))
 {
   auto logPath = common::joinPaths(PROJECT_SOURCE_PATH, "test", "media",
       "rolling_shapes_log");
@@ -1009,7 +1024,7 @@ TEST_F(LogSystemTest, LogControl)
 }
 
 /////////////////////////////////////////////////
-TEST_F(LogSystemTest, LogOverwrite)
+TEST_F(LogSystemTest, IGN_UTILS_TEST_DISABLED_ON_WIN32(LogOverwrite))
 {
   // Create temp directory to store log
   this->CreateLogsDir();
@@ -1158,7 +1173,7 @@ TEST_F(LogSystemTest, LogOverwrite)
 }
 
 /////////////////////////////////////////////////
-TEST_F(LogSystemTest, LogControlLevels)
+TEST_F(LogSystemTest, IGN_UTILS_TEST_DISABLED_ON_WIN32(LogControlLevels))
 {
   auto logPath = common::joinPaths(PROJECT_SOURCE_PATH, "test", "media",
       "levels_log");
@@ -1189,8 +1204,8 @@ TEST_F(LogSystemTest, LogControlLevels)
   // verify there are entities at the beginning of the playback
   EXPECT_TRUE(!entitiesAtTime0.empty());
 
-  double timeA = 8;
-  double timeB = 18;
+  int timeA = 8;
+  int timeB = 18;
 
   transport::Node node;
 
@@ -1297,7 +1312,7 @@ TEST_F(LogSystemTest, LogControlLevels)
 }
 
 /////////////////////////////////////////////////
-TEST_F(LogSystemTest, LogCompress)
+TEST_F(LogSystemTest, IGN_UTILS_TEST_DISABLED_ON_WIN32(LogCompress))
 {
   // Create temp directory to store log
   this->CreateLogsDir();
@@ -1403,7 +1418,7 @@ TEST_F(LogSystemTest, LogCompress)
 }
 
 /////////////////////////////////////////////////
-TEST_F(LogSystemTest, LogCompressOverwrite)
+TEST_F(LogSystemTest, IGN_UTILS_TEST_DISABLED_ON_WIN32(LogCompressOverwrite))
 {
   // Create temp directory to store log
   this->CreateLogsDir();
@@ -1449,7 +1464,7 @@ TEST_F(LogSystemTest, LogCompressOverwrite)
 }
 
 /////////////////////////////////////////////////
-TEST_F(LogSystemTest, LogCompressCmdLine)
+TEST_F(LogSystemTest, IGN_UTILS_TEST_DISABLED_ON_WIN32(LogCompressCmdLine))
 {
 #ifndef __APPLE__
   // Create temp directory to store log
@@ -1529,7 +1544,7 @@ TEST_F(LogSystemTest, LogCompressCmdLine)
 }
 
 /////////////////////////////////////////////////
-TEST_F(LogSystemTest, LogResources)
+TEST_F(LogSystemTest, IGN_UTILS_TEST_DISABLED_ON_WIN32(LogResources))
 {
   // Create temp directory to store log
   this->CreateLogsDir();
@@ -1616,7 +1631,7 @@ TEST_F(LogSystemTest, LogResources)
 }
 
 /////////////////////////////////////////////////
-TEST_F(LogSystemTest, LogTopics)
+TEST_F(LogSystemTest, IGN_UTILS_TEST_DISABLED_ON_WIN32(LogTopics))
 {
   // Create temp directory to store log
   this->CreateLogsDir();
