@@ -25,6 +25,7 @@
 #include <ignition/common/Console.hh>
 #include <ignition/common/Util.hh>
 #include <ignition/transport/Node.hh>
+#include <ignition/utilities/ExtraTestMacros.hh>
 
 #include "ignition/gazebo/components/AngularVelocity.hh"
 #include "ignition/gazebo/components/Joint.hh"
@@ -40,21 +41,14 @@
 #include "ignition/gazebo/test_config.hh"
 
 #include "../helpers/Relay.hh"
+#include "../helpers/EnvTestFixture.hh"
 
 using namespace ignition;
 using namespace gazebo;
 using namespace std::chrono_literals;
 
-class MulticopterTest : public ::testing::Test
+class MulticopterTest : public InternalFixture<::testing::Test>
 {
-  // Documentation inherited
-  protected: void SetUp() override
-  {
-    ignition::common::Console::SetVerbosity(4);
-    ignition::common::setenv("IGN_GAZEBO_SYSTEM_PLUGIN_PATH",
-           (std::string(PROJECT_BINARY_PATH) + "/lib").c_str());
-  }
-
   protected: std::unique_ptr<Server> StartServer(const std::string &_filePath)
   {
     ServerConfig serverConfig;
@@ -73,7 +67,8 @@ class MulticopterTest : public ::testing::Test
 
 /////////////////////////////////////////////////
 // Test that commanded motor speed is applied
-TEST_F(MulticopterTest, CommandedMotorSpeed)
+// See https://github.com/ignitionrobotics/ign-gazebo/issues/1175
+TEST_F(MulticopterTest, IGN_UTILS_TEST_DISABLED_ON_WIN32(CommandedMotorSpeed))
 {
   // Start server
   auto server = this->StartServer("/test/worlds/quadcopter.sdf");
@@ -139,7 +134,8 @@ TEST_F(MulticopterTest, CommandedMotorSpeed)
 }
 
 /////////////////////////////////////////////////
-TEST_F(MulticopterTest, MulticopterVelocityControl)
+TEST_F(MulticopterTest,
+       IGN_UTILS_TEST_DISABLED_ON_WIN32(MulticopterVelocityControl))
 {
   // Start server
   auto server =
@@ -247,7 +243,8 @@ TEST_F(MulticopterTest, MulticopterVelocityControl)
 /////////////////////////////////////////////////
 // Test the interactions between MulticopterVelocityControl and
 // MulticopterMotorModel
-TEST_F(MulticopterTest, ModelAndVelocityControlInteraction)
+TEST_F(MulticopterTest,
+       IGN_UTILS_TEST_DISABLED_ON_WIN32(ModelAndVelocityControlInteraction))
 {
   // Start server
   auto server =
