@@ -18,10 +18,15 @@
 #include <gtest/gtest.h>
 
 #include <algorithm>
+#include <string>
 #include <vector>
 
+#ifdef HAVE_DART
+#include <dart/config.hpp>
+#endif
 #include <ignition/common/Console.hh>
 #include <ignition/common/Util.hh>
+#include <ignition/utilities/ExtraTestMacros.hh>
 #include <sdf/Collision.hh>
 #include <sdf/Cylinder.hh>
 #include <sdf/Geometry.hh>
@@ -31,8 +36,10 @@
 #include <sdf/Sphere.hh>
 #include <sdf/World.hh>
 
+#include "ignition/gazebo/Entity.hh"
 #include "ignition/gazebo/Server.hh"
 #include "ignition/gazebo/SystemLoader.hh"
+#include "ignition/gazebo/Util.hh"
 #include "ignition/gazebo/test_config.hh"  // NOLINT(build/include)
 
 #include "ignition/gazebo/components/AxisAlignedBox.hh"
@@ -41,9 +48,14 @@
 #include "ignition/gazebo/components/Geometry.hh"
 #include "ignition/gazebo/components/Inertial.hh"
 #include "ignition/gazebo/components/Joint.hh"
+#include "ignition/gazebo/components/JointEffortLimitsCmd.hh"
+#include "ignition/gazebo/components/JointForceCmd.hh"
 #include "ignition/gazebo/components/JointPosition.hh"
+#include "ignition/gazebo/components/JointPositionLimitsCmd.hh"
 #include "ignition/gazebo/components/JointPositionReset.hh"
 #include "ignition/gazebo/components/JointVelocity.hh"
+#include "ignition/gazebo/components/JointVelocityCmd.hh"
+#include "ignition/gazebo/components/JointVelocityLimitsCmd.hh"
 #include "ignition/gazebo/components/JointVelocityReset.hh"
 #include "ignition/gazebo/components/Link.hh"
 #include "ignition/gazebo/components/LinearVelocity.hh"
@@ -51,25 +63,34 @@
 #include "ignition/gazebo/components/Model.hh"
 #include "ignition/gazebo/components/Name.hh"
 #include "ignition/gazebo/components/ParentEntity.hh"
+#include "ignition/gazebo/components/Physics.hh"
 #include "ignition/gazebo/components/Pose.hh"
 #include "ignition/gazebo/components/Static.hh"
 #include "ignition/gazebo/components/Visual.hh"
 #include "ignition/gazebo/components/World.hh"
 
 #include "../helpers/Relay.hh"
+#include "../helpers/EnvTestFixture.hh"
 
 using namespace ignition;
 using namespace gazebo;
 using namespace std::chrono_literals;
 
-class PhysicsSystemFixture : public ::testing::Test
+class PhysicsSystemFixture : public InternalFixture<::testing::Test>
+{
+};
+
+class PhysicsSystemFixtureWithDart6_10 : public PhysicsSystemFixture
 {
   protected: void SetUp() override
   {
-    common::Console::SetVerbosity(4);
-    // Augment the system plugin path.  In SetUp to avoid test order issues.
-    ignition::common::setenv("IGN_GAZEBO_SYSTEM_PLUGIN_PATH",
-      (std::string(PROJECT_BINARY_PATH) + "/lib").c_str());
+#ifndef HAVE_DART
+    GTEST_SKIP();
+#elif !DART_VERSION_AT_LEAST(6, 10, 0)
+    GTEST_SKIP();
+#endif
+
+    PhysicsSystemFixture::SetUp();
   }
 };
 
@@ -94,8 +115,9 @@ TEST_F(PhysicsSystemFixture, CreatePhysicsWorld)
   // TODO(addisu) add useful EXPECT calls
 }
 
-/////////////////////////////////////////////////
-TEST_F(PhysicsSystemFixture, FallingObject)
+////////////////////////////////////////////////
+// See https://github.com/ignitionrobotics/ign-gazebo/issues/1175
+TEST_F(PhysicsSystemFixture, IGN_UTILS_TEST_DISABLED_ON_WIN32(FallingObject))
 {
   ignition::gazebo::ServerConfig serverConfig;
 
@@ -164,7 +186,7 @@ TEST_F(PhysicsSystemFixture, FallingObject)
 // This tests whether links with fixed joints keep their relative transforms
 // after physics. For that to work properly, the canonical link implementation
 // must be correct.
-TEST_F(PhysicsSystemFixture, CanonicalLink)
+TEST_F(PhysicsSystemFixture, IGN_UTILS_TEST_DISABLED_ON_WIN32(CanonicalLink))
 {
   ignition::gazebo::ServerConfig serverConfig;
 
@@ -235,7 +257,8 @@ TEST_F(PhysicsSystemFixture, CanonicalLink)
 
 /////////////////////////////////////////////////
 // Same as the CanonicalLink test, but with a non-default canonical link
-TEST_F(PhysicsSystemFixture, NonDefaultCanonicalLink)
+TEST_F(PhysicsSystemFixture,
+       IGN_UTILS_TEST_DISABLED_ON_WIN32(NonDefaultCanonicalLink))
 {
   ignition::gazebo::ServerConfig serverConfig;
 
@@ -290,7 +313,7 @@ TEST_F(PhysicsSystemFixture, NonDefaultCanonicalLink)
 
 /////////////////////////////////////////////////
 // Test physics integration with revolute joints
-TEST_F(PhysicsSystemFixture, RevoluteJoint)
+TEST_F(PhysicsSystemFixture, IGN_UTILS_TEST_DISABLED_ON_WIN32(RevoluteJoint))
 {
   ignition::gazebo::ServerConfig serverConfig;
 
@@ -369,7 +392,7 @@ TEST_F(PhysicsSystemFixture, RevoluteJoint)
 }
 
 /////////////////////////////////////////////////
-TEST_F(PhysicsSystemFixture, CreateRuntime)
+TEST_F(PhysicsSystemFixture, IGN_UTILS_TEST_DISABLED_ON_WIN32(CreateRuntime))
 {
   ignition::gazebo::ServerConfig serverConfig;
   gazebo::Server server(serverConfig);
@@ -452,7 +475,8 @@ TEST_F(PhysicsSystemFixture, CreateRuntime)
 }
 
 /////////////////////////////////////////////////
-TEST_F(PhysicsSystemFixture, SetFrictionCoefficient)
+TEST_F(PhysicsSystemFixture,
+       IGN_UTILS_TEST_DISABLED_ON_WIN32(SetFrictionCoefficient))
 {
   ignition::gazebo::ServerConfig serverConfig;
 
@@ -470,13 +494,11 @@ TEST_F(PhysicsSystemFixture, SetFrictionCoefficient)
 
   // Create a system that records the poses of the 3 boxes
   test::Relay testSystem;
-  double dt = 0.0;
 
   testSystem.OnPostUpdate(
-    [&boxParams, &poses, &dt](const gazebo::UpdateInfo &_info,
+    [&boxParams, &poses](const gazebo::UpdateInfo &,
     const gazebo::EntityComponentManager &_ecm)
     {
-      dt = _info.dt.count();
       _ecm.Each<components::Model, components::Name, components::Pose>(
         [&](const ignition::gazebo::Entity &, const components::Model *,
         const components::Name *_name, const components::Pose *_pose)->bool
@@ -539,7 +561,8 @@ TEST_F(PhysicsSystemFixture, SetFrictionCoefficient)
 
 /////////////////////////////////////////////////
 /// Test that joint position reported by the physics system include all axes
-TEST_F(PhysicsSystemFixture, MultiAxisJointPosition)
+TEST_F(PhysicsSystemFixture,
+       IGN_UTILS_TEST_DISABLED_ON_WIN32(MultiAxisJointPosition))
 {
   ignition::gazebo::ServerConfig serverConfig;
 
@@ -618,7 +641,8 @@ TEST_F(PhysicsSystemFixture, MultiAxisJointPosition)
 
 /////////////////////////////////////////////////
 /// Test joint position reset component
-TEST_F(PhysicsSystemFixture, ResetPositionComponent)
+TEST_F(PhysicsSystemFixture,
+       IGN_UTILS_TEST_DISABLED_ON_WIN32(ResetPositionComponent))
 {
   ignition::gazebo::ServerConfig serverConfig;
 
@@ -717,7 +741,8 @@ TEST_F(PhysicsSystemFixture, ResetPositionComponent)
 
 /////////////////////////////////////////////////
 /// Test joint veocity reset component
-TEST_F(PhysicsSystemFixture, ResetVelocityComponent)
+TEST_F(PhysicsSystemFixture,
+       IGN_UTILS_TEST_DISABLED_ON_WIN32(ResetVelocityComponent))
 {
   ignition::gazebo::ServerConfig serverConfig;
 
@@ -814,7 +839,382 @@ TEST_F(PhysicsSystemFixture, ResetVelocityComponent)
 }
 
 /////////////////////////////////////////////////
-TEST_F(PhysicsSystemFixture, GetBoundingBox)
+/// Test joint position limit command component
+TEST_F(PhysicsSystemFixtureWithDart6_10, JointPositionLimitsCommandComponent)
+{
+  ignition::gazebo::ServerConfig serverConfig;
+
+  const auto sdfFile = std::string(PROJECT_SOURCE_PATH) +
+    "/test/worlds/revolute_joint.sdf";
+
+  sdf::Root root;
+  root.Load(sdfFile);
+  const sdf::World *world = root.WorldByIndex(0);
+  ASSERT_TRUE(nullptr != world);
+
+  serverConfig.SetSdfFile(sdfFile);
+
+  gazebo::Server server(serverConfig);
+
+  server.SetUpdatePeriod(1ms);
+
+  const std::string rotatingJointName{"j2"};
+
+  test::Relay testSystem;
+
+  // cppcheck-suppress variableScope
+  size_t iteration = 0u;
+
+  // The system is not in equilibrium at the beginning, so normally, joint j2
+  // would move. For the first 50 ms, we set position limits to 1e-6 so that
+  // it can't freely move, and we check it did not. For the other 50 ms, we
+  // remove the position limit and check that the joint has moved at least by
+  // 1e-2. Between times 30 and 40 ms we also add a 100 N force to the joint to
+  // check that the limit is held even in presence of force commands. Between
+  // times 40 and 50 ms, we add a velocity command to check that velocity
+  // commands do not break the positional limit.
+
+  testSystem.OnPreUpdate(
+    [&](const gazebo::UpdateInfo &, gazebo::EntityComponentManager &_ecm)
+    {
+      _ecm.Each<components::Joint, components::Name>(
+        [&](const ignition::gazebo::Entity &_entity,
+            const components::Joint *, components::Name *_name) -> bool
+        {
+          if (_name->Data() == rotatingJointName)
+          {
+            if (iteration == 0u)
+            {
+              auto limitComp =
+                _ecm.Component<components::JointPositionLimitsCmd>(_entity);
+              EXPECT_EQ(nullptr, limitComp);
+              _ecm.CreateComponent(_entity,
+                components::JointPositionLimitsCmd ({{-1e-6, 1e-6}}));
+              _ecm.CreateComponent(_entity, components::JointPosition());
+            }
+            else if (iteration == 50u)
+            {
+              auto limitComp =
+                _ecm.Component<components::JointPositionLimitsCmd>(_entity);
+              EXPECT_NE(nullptr, limitComp);
+              if (limitComp)
+              {
+                limitComp->Data() = {{-1e6, 1e6}};
+              }
+            }
+            else
+            {
+              auto limitComp =
+                _ecm.Component<components::JointPositionLimitsCmd>(_entity);
+              EXPECT_NE(nullptr, limitComp);
+              if (limitComp)
+              {
+                EXPECT_EQ(0u, limitComp->Data().size());
+              }
+              if (iteration >= 30u && iteration < 40u)
+              {
+                _ecm.SetComponentData<components::JointForceCmd>(
+                  _entity, {100.0});
+              }
+              else if (iteration >= 40u && iteration < 50u)
+              {
+                _ecm.SetComponentData<components::JointVelocityCmd>(
+                  _entity, {1.0});
+              }
+            }
+            ++iteration;
+          }
+          return true;
+        });
+    });
+
+  std::vector<double> positions;
+
+  testSystem.OnPostUpdate([&](
+    const gazebo::UpdateInfo &, const gazebo::EntityComponentManager &_ecm)
+    {
+      _ecm.Each<components::Joint,
+                components::Name,
+                components::JointPosition>(
+        [&](const ignition::gazebo::Entity &,
+            const components::Joint *,
+            const components::Name *_name,
+            const components::JointPosition *_pos)
+        {
+          if (_name->Data() == rotatingJointName)
+          {
+            positions.push_back(_pos->Data()[0]);
+          }
+          return true;
+        });
+    });
+
+  server.AddSystem(testSystem.systemPtr);
+  server.Run(true, 100, false);
+
+  ASSERT_EQ(positions.size(), 100ul);
+  // The 1e-6 limit is slightly overcome, but not very much
+  EXPECT_NEAR(positions[0], positions[20], 2e-5);
+  EXPECT_NEAR(positions[0], positions[30], 3e-5);
+  EXPECT_NEAR(positions[0], positions[40], 3e-5);
+  EXPECT_NEAR(positions[0], positions[49], 3e-5);
+  EXPECT_LT(std::abs(positions[50]) + 1e-2, std::abs(positions[99]));
+}
+
+/////////////////////////////////////////////////
+/// Test joint velocity limit command component
+TEST_F(PhysicsSystemFixtureWithDart6_10, JointVelocityLimitsCommandComponent)
+{
+  ignition::gazebo::ServerConfig serverConfig;
+
+  const auto sdfFile = std::string(PROJECT_SOURCE_PATH) +
+    "/test/worlds/revolute_joint.sdf";
+
+  sdf::Root root;
+  root.Load(sdfFile);
+  const sdf::World *world = root.WorldByIndex(0);
+  ASSERT_TRUE(nullptr != world);
+
+  serverConfig.SetSdfFile(sdfFile);
+
+  gazebo::Server server(serverConfig);
+
+  server.SetUpdatePeriod(1ms);
+
+  const std::string rotatingJointName{"j2"};
+
+  test::Relay testSystem;
+
+  // cppcheck-suppress variableScope
+  size_t iteration = 0u;
+
+  // The system is not in equilibrium at the beginning, so normally, joint j2
+  // would move. For the first 50 ms, we set velocity limits to 0.1 so that
+  // it can't move very fast, and we check it does not. For the other 50 ms, we
+  // remove the velocity limit and check that the joint has moved faster.
+  // Between times 30 and 40 ms we also add a 100 N force to the joint to
+  // check that the limit is held even in presence of force commands. Between
+  // times 40 and 50 ms, we add a velocity command to check that velocity
+  // commands do not break the velocity limit.
+
+  testSystem.OnPreUpdate(
+    [&](const gazebo::UpdateInfo &, gazebo::EntityComponentManager &_ecm)
+    {
+      _ecm.Each<components::Joint, components::Name>(
+        [&](const ignition::gazebo::Entity &_entity,
+            const components::Joint *, components::Name *_name) -> bool
+        {
+          if (_name->Data() == rotatingJointName)
+          {
+            if (iteration == 0u)
+            {
+              auto limitComp =
+                _ecm.Component<components::JointVelocityLimitsCmd>(_entity);
+              EXPECT_EQ(nullptr, limitComp);
+              _ecm.CreateComponent(_entity,
+                components::JointVelocityLimitsCmd ({{-0.1, 0.1}}));
+              _ecm.CreateComponent(_entity, components::JointVelocity());
+            }
+            else if (iteration == 50u)
+            {
+              auto limitComp =
+                _ecm.Component<components::JointVelocityLimitsCmd>(_entity);
+              EXPECT_NE(nullptr, limitComp);
+              if (limitComp)
+              {
+                limitComp->Data() = {{-1e6, 1e6}};
+              }
+            }
+            else
+            {
+              auto limitComp =
+                _ecm.Component<components::JointVelocityLimitsCmd>(_entity);
+              EXPECT_NE(nullptr, limitComp);
+              if (limitComp)
+              {
+                EXPECT_EQ(0u, limitComp->Data().size());
+              }
+              if (iteration >= 30u && iteration < 40u)
+              {
+                _ecm.SetComponentData<components::JointForceCmd>(
+                  _entity, {100.0});
+              }
+              else if (iteration >= 40u && iteration < 50u)
+              {
+                _ecm.SetComponentData<components::JointVelocityCmd>(
+                  _entity, {1.0});
+              }
+            }
+            ++iteration;
+          }
+          return true;
+        });
+    });
+
+  std::vector<double> velocities;
+
+  testSystem.OnPostUpdate([&](
+    const gazebo::UpdateInfo &, const gazebo::EntityComponentManager &_ecm)
+    {
+      _ecm.Each<components::Joint,
+                components::Name,
+                components::JointVelocity>(
+        [&](const ignition::gazebo::Entity &,
+            const components::Joint *,
+            const components::Name *_name,
+            const components::JointVelocity *_vel)
+        {
+          if (_name->Data() == rotatingJointName)
+          {
+            velocities.push_back(_vel->Data()[0]);
+          }
+          return true;
+        });
+    });
+
+  server.AddSystem(testSystem.systemPtr);
+  server.Run(true, 100, false);
+
+  ASSERT_EQ(velocities.size(), 100ul);
+  // The 0.1 limit is slightly overcome, but not very much
+  EXPECT_NEAR(0.1, velocities[20], 1e-2);
+  EXPECT_NEAR(0.1, velocities[30], 1e-2);
+  EXPECT_NEAR(0.1, velocities[40], 1e-2);
+  EXPECT_NEAR(0.1, velocities[49], 1e-2);
+  EXPECT_LT(0.5, std::abs(velocities[99]));
+}
+
+
+/////////////////////////////////////////////////
+/// Test joint effort limit command component
+TEST_F(PhysicsSystemFixtureWithDart6_10, JointEffortLimitsCommandComponent)
+{
+  ignition::gazebo::ServerConfig serverConfig;
+
+  const auto sdfFile = std::string(PROJECT_SOURCE_PATH) +
+    "/test/worlds/revolute_joint_equilibrium.sdf";
+
+  sdf::Root root;
+  root.Load(sdfFile);
+  const sdf::World *world = root.WorldByIndex(0);
+  ASSERT_TRUE(nullptr != world);
+
+  serverConfig.SetSdfFile(sdfFile);
+
+  gazebo::Server server(serverConfig);
+
+  server.SetUpdatePeriod(1ms);
+
+  const std::string rotatingJointName{"j2"};
+
+  test::Relay testSystem;
+
+  // cppcheck-suppress variableScope
+  size_t iteration = 0u;
+
+  // The system is in equilibrium at the beginning.
+  // For the first 50 ms, we set effort limits to 1e-6 so that
+  // it can't move, and we check it does not. For the other 50 ms, we
+  // remove the effort limit and check that the joint has moved.
+  // Between times 30 and 40 ms we also add a 100 N force to the joint to
+  // check that the limit is held even in presence of force commands. Between
+  // times 40 and 50 ms, we add a velocity command to check that velocity
+  // commands do not break the effort limit.
+
+  testSystem.OnPreUpdate(
+    [&](const gazebo::UpdateInfo &, gazebo::EntityComponentManager &_ecm)
+    {
+      _ecm.Each<components::Joint, components::Name>(
+        [&](const ignition::gazebo::Entity &_entity,
+            const components::Joint *, components::Name *_name) -> bool
+        {
+          if (_name->Data() == rotatingJointName)
+          {
+            if (iteration == 0u)
+            {
+              auto limitComp =
+                _ecm.Component<components::JointEffortLimitsCmd>(_entity);
+              EXPECT_EQ(nullptr, limitComp);
+              _ecm.CreateComponent(_entity,
+                components::JointEffortLimitsCmd ({{-1e-6, 1e-6}}));
+              _ecm.CreateComponent(_entity, components::JointPosition());
+            }
+            else if (iteration == 50u)
+            {
+              auto limitComp =
+                _ecm.Component<components::JointEffortLimitsCmd>(_entity);
+              EXPECT_NE(nullptr, limitComp);
+              if (limitComp)
+              {
+                limitComp->Data() = {{-1e9, 1e9}};
+              }
+            }
+            else
+            {
+              auto limitComp =
+                _ecm.Component<components::JointEffortLimitsCmd>(_entity);
+              EXPECT_NE(nullptr, limitComp);
+              if (limitComp)
+              {
+                EXPECT_EQ(0u, limitComp->Data().size());
+              }
+              if (iteration >= 30u && iteration < 40u)
+              {
+                _ecm.SetComponentData<components::JointForceCmd>(
+                  _entity, {100.0});
+              }
+              else if (iteration >= 40u && iteration < 50u)
+              {
+                _ecm.SetComponentData<components::JointVelocityCmd>(
+                  _entity, {1.0});
+              }
+              else if (iteration >= 50u)
+              {
+                _ecm.Component<components::JointForceCmd>(_entity)->Data() =
+                  {1000.0};
+              }
+            }
+            ++iteration;
+          }
+          return true;
+        });
+    });
+
+  std::vector<double> positions;
+
+  testSystem.OnPostUpdate([&](
+    const gazebo::UpdateInfo &, const gazebo::EntityComponentManager &_ecm)
+    {
+    _ecm.Each<components::Joint,
+    components::Name,
+    components::JointPosition>(
+      [&](const ignition::gazebo::Entity &,
+        const components::Joint *,
+        const components::Name *_name,
+        const components::JointPosition *_pos)
+        {
+        if (_name->Data() == rotatingJointName)
+        {
+          positions.push_back(_pos->Data()[0]);
+        }
+        return true;
+        });
+    });
+
+  server.AddSystem(testSystem.systemPtr);
+  server.Run(true, 100, false);
+
+  ASSERT_EQ(positions.size(), 100ul);
+  // The 1e-6 limit is slightly overcome, but not very much
+  EXPECT_NEAR(positions[0], positions[20], 2e-5);
+  EXPECT_NEAR(positions[0], positions[30], 3e-5);
+  EXPECT_NEAR(positions[0], positions[40], 3e-5);
+  EXPECT_NEAR(positions[0], positions[49], 3e-5);
+  EXPECT_LT(std::abs(positions[50]) + 1e-2, std::abs(positions[99]));
+}
+
+/////////////////////////////////////////////////
+TEST_F(PhysicsSystemFixture, IGN_UTILS_TEST_DISABLED_ON_WIN32(GetBoundingBox))
 {
   ignition::gazebo::ServerConfig serverConfig;
 
@@ -885,7 +1285,7 @@ TEST_F(PhysicsSystemFixture, GetBoundingBox)
 
 /////////////////////////////////////////////////
 // This tests whether nested models can be loaded correctly
-TEST_F(PhysicsSystemFixture, NestedModel)
+TEST_F(PhysicsSystemFixture, IGN_UTILS_TEST_DISABLED_ON_WIN32(NestedModel))
 {
   ignition::gazebo::ServerConfig serverConfig;
 
@@ -990,7 +1390,8 @@ TEST_F(PhysicsSystemFixture, NestedModel)
 }
 
 // This tests whether nested models can be loaded correctly
-TEST_F(PhysicsSystemFixture, IncludeNestedModelDartsim)
+TEST_F(PhysicsSystemFixture,
+       IGN_UTILS_TEST_DISABLED_ON_WIN32(IncludeNestedModelDartsim))
 {
   std::string path = std::string(PROJECT_SOURCE_PATH) + "/test/worlds/models";
   ignition::common::setenv("IGN_GAZEBO_RESOURCE_PATH", path.c_str());
@@ -1132,7 +1533,8 @@ TEST_F(PhysicsSystemFixture, IncludeNestedModelDartsim)
 }
 
 // This tests whether nested models can be loaded correctly
-TEST_F(PhysicsSystemFixture, IncludeNestedModelTPE)
+TEST_F(PhysicsSystemFixture,
+       IGN_UTILS_TEST_DISABLED_ON_WIN32(IncludeNestedModelTPE))
 {
   std::string path = std::string(PROJECT_SOURCE_PATH) + "/test/worlds/models";
   ignition::common::setenv("IGN_GAZEBO_RESOURCE_PATH", path.c_str());
@@ -1274,7 +1676,8 @@ TEST_F(PhysicsSystemFixture, IncludeNestedModelTPE)
 }
 
 // This tests whether the poses of nested models are updated correctly
-TEST_F(PhysicsSystemFixture, NestedModelIndividualCanonicalLinks)
+TEST_F(PhysicsSystemFixture,
+       IGN_UTILS_TEST_DISABLED_ON_WIN32(NestedModelIndividualCanonicalLinks))
 {
   ignition::gazebo::ServerConfig serverConfig;
 
@@ -1295,11 +1698,17 @@ TEST_F(PhysicsSystemFixture, NestedModelIndividualCanonicalLinks)
   // Create a system that records the poses of the links after physics
   test::Relay testSystem;
 
+  // Store a pointer to the ECM used in testSystem so that it can be used to
+  // help verify poses at the end of this test
+  const gazebo::EntityComponentManager *ecm;
+
   std::unordered_map<std::string, ignition::math::Pose3d> postUpModelPoses;
   testSystem.OnPostUpdate(
-    [&postUpModelPoses](const gazebo::UpdateInfo &,
+    [&postUpModelPoses, &ecm](const gazebo::UpdateInfo &,
     const gazebo::EntityComponentManager &_ecm)
     {
+      ecm = &_ecm;
+
       _ecm.Each<components::Model, components::Name, components::Pose>(
         [&](const ignition::gazebo::Entity &, const components::Model *,
         const components::Name *_name, const components::Pose *_pose)->bool
@@ -1315,10 +1724,10 @@ TEST_F(PhysicsSystemFixture, NestedModelIndividualCanonicalLinks)
   server.AddSystem(testSystem.systemPtr);
   const size_t iters = 500;
   server.Run(true, iters, false);
-  EXPECT_EQ(3u, postUpModelPoses.size());
+  EXPECT_EQ(6u, postUpModelPoses.size());
 
   auto modelIt = postUpModelPoses.find("model_00");
-  EXPECT_NE(postUpModelPoses.end(), modelIt);
+  ASSERT_NE(postUpModelPoses.end(), modelIt);
 
   // link_00 is resting on the ground_box, so it remains stationary. And since
   // it is the canonical link of the parent model, model_00, the model frame
@@ -1332,6 +1741,333 @@ TEST_F(PhysicsSystemFixture, NestedModelIndividualCanonicalLinks)
   const double dt = 0.001;
   const double zExpected = 0.5 * world->Gravity().Z() * pow(dt * iters, 2);
   modelIt = postUpModelPoses.find("model_01");
-  EXPECT_NE(postUpModelPoses.end(), modelIt);
+  ASSERT_NE(postUpModelPoses.end(), modelIt);
   EXPECT_NEAR(zExpected, modelIt->second.Z(), 1e-2);
+
+  // model_10, model_11 and model_12 share the same canonical link (link_12),
+  // which is a floating link. So, link_12 should fall due to gravity, which
+  // also means that the frames for model_10, model_11 and model_12 should fall
+  auto model10It = postUpModelPoses.find("model_10");
+  ASSERT_NE(postUpModelPoses.end(), model10It);
+  EXPECT_NEAR(zExpected, model10It->second.Z(), 1e-2);
+  auto model11It = postUpModelPoses.find("model_11");
+  ASSERT_NE(postUpModelPoses.end(), model11It);
+  auto model12It = postUpModelPoses.find("model_12");
+  ASSERT_NE(postUpModelPoses.end(), model12It);
+  // (since model_11 and model_12 are nested, their poses are computed w.r.t.
+  // model_10)
+  EXPECT_DOUBLE_EQ(0.0, model11It->second.Z());
+  EXPECT_DOUBLE_EQ(0.0, model12It->second.Z());
+  // check the world pose of model_11 and model_12 to make sure the z components
+  // of these world poses match zExpected
+  ASSERT_NE(nullptr, ecm);
+  const auto model11Entity =
+    ecm->EntityByComponents(components::Name(model11It->first));
+  ASSERT_NE(kNullEntity, model11Entity);
+  const auto model11WorldPose = gazebo::worldPose(model11Entity, *ecm);
+  EXPECT_NEAR(zExpected, model11WorldPose.Z(), 1e-2);
+  const auto model12Entity =
+    ecm->EntityByComponents(components::Name(model12It->first));
+  ASSERT_NE(kNullEntity, model12Entity);
+  const auto model12WorldPose = gazebo::worldPose(model12Entity, *ecm);
+  EXPECT_NEAR(zExpected, model12WorldPose.Z(), 1e-2);
+  EXPECT_DOUBLE_EQ(model11WorldPose.Z(), model12WorldPose.Z());
+  // zExpected is also the z component of the world pose for model_10
+  EXPECT_DOUBLE_EQ(model11WorldPose.Z(), model10It->second.Z());
+  EXPECT_DOUBLE_EQ(model12WorldPose.Z(), model10It->second.Z());
+}
+
+/////////////////////////////////////////////////
+TEST_F(PhysicsSystemFixture,
+       IGN_UTILS_TEST_DISABLED_ON_WIN32(DefaultPhysicsOptions))
+{
+  ignition::gazebo::ServerConfig serverConfig;
+
+  bool checked{false};
+
+  // Create a system to check components
+  test::Relay testSystem;
+  testSystem.OnPostUpdate(
+    [&checked](const gazebo::UpdateInfo &,
+    const gazebo::EntityComponentManager &_ecm)
+    {
+      _ecm.Each<components::World, components::PhysicsCollisionDetector,
+                components::PhysicsSolver>(
+        [&](const ignition::gazebo::Entity &, const components::World *,
+            const components::PhysicsCollisionDetector *_collisionDetector,
+            const components::PhysicsSolver *_solver)->bool
+        {
+          EXPECT_NE(nullptr, _collisionDetector);
+          if (_collisionDetector)
+          {
+            EXPECT_EQ("ode", _collisionDetector->Data());
+          }
+          EXPECT_NE(nullptr, _solver);
+          if (_solver)
+          {
+            EXPECT_EQ("DantzigBoxedLcpSolver", _solver->Data());
+          }
+          checked = true;
+          return true;
+        });
+    });
+
+  gazebo::Server server(serverConfig);
+  server.AddSystem(testSystem.systemPtr);
+  server.Run(true, 1, false);
+
+  EXPECT_TRUE(checked);
+}
+
+/////////////////////////////////////////////////
+TEST_F(PhysicsSystemFixture, PhysicsOptions)
+{
+  ignition::gazebo::ServerConfig serverConfig;
+  serverConfig.SetSdfFile(common::joinPaths(std::string(PROJECT_SOURCE_PATH),
+    "test", "worlds", "physics_options.sdf"));
+
+  bool checked{false};
+
+  // Create a system to check components
+  test::Relay testSystem;
+  testSystem.OnPostUpdate(
+    [&checked](const gazebo::UpdateInfo &,
+    const gazebo::EntityComponentManager &_ecm)
+    {
+      _ecm.Each<components::World, components::PhysicsCollisionDetector,
+                components::PhysicsSolver>(
+        [&](const ignition::gazebo::Entity &, const components::World *,
+            const components::PhysicsCollisionDetector *_collisionDetector,
+            const components::PhysicsSolver *_solver)->bool
+        {
+          EXPECT_NE(nullptr, _collisionDetector);
+          if (_collisionDetector)
+          {
+            EXPECT_EQ("bullet", _collisionDetector->Data());
+          }
+          EXPECT_NE(nullptr, _solver);
+          if (_solver)
+          {
+            EXPECT_EQ("pgs", _solver->Data());
+          }
+          checked = true;
+          return true;
+        });
+    });
+
+  gazebo::Server server(serverConfig);
+  server.AddSystem(testSystem.systemPtr);
+  server.Run(true, 1, false);
+
+  EXPECT_TRUE(checked);
+}
+
+/////////////////////////////////////////////////
+// This tests whether pose updates are correct for a model whose canonical link
+// changes, but other links do not
+TEST_F(PhysicsSystemFixture,
+       IGN_UTILS_TEST_DISABLED_ON_WIN32(MovingCanonicalLinkOnly))
+{
+  ignition::gazebo::ServerConfig serverConfig;
+
+  const auto sdfFile = common::joinPaths(PROJECT_SOURCE_PATH, "test", "worlds",
+    "only_canonical_link_moves.sdf");
+
+  sdf::Root root;
+  root.Load(sdfFile);
+  const sdf::World *world = root.WorldByIndex(0);
+  ASSERT_TRUE(nullptr != world);
+
+  serverConfig.SetSdfFile(sdfFile);
+
+  gazebo::Server server(serverConfig);
+
+  server.SetUpdatePeriod(1us);
+
+  // Create a system that records the poses of the links after physics
+  test::Relay testSystem;
+
+  size_t numBaseLinkChecks = 0;
+  size_t numOuterLinkChecks = 0;
+  size_t numBaseLinkCustomChecks = 0;
+  size_t numOuterLinkCustomChecks = 0;
+  size_t numBaseLinkParentChecks = 0;
+  size_t numNestedModelLinkChecks = 0;
+  size_t numParentModelLinkChecks = 0;
+  size_t numBaseLinkChildChecks = 0;
+
+  size_t currIter = 0;
+  testSystem.OnPostUpdate(
+    [&](const gazebo::UpdateInfo &, const gazebo::EntityComponentManager &_ecm)
+    {
+      currIter++;
+      _ecm.Each<components::Link, components::Name>(
+          [&](const ignition::gazebo::Entity &_entity,
+          const components::Link *, const components::Name *_name)->bool
+          {
+            // ignore the link for the ground plane
+            if (_name->Data() != "surface")
+            {
+              const double dt = 0.001;
+              const double zExpected =
+                0.5 * world->Gravity().Z() * pow(dt * currIter, 2);
+
+              if (_name->Data() == "base_link")
+              {
+                // link "base_link" falls due to gravity, starting from rest
+                EXPECT_NEAR(zExpected,
+                    ignition::gazebo::worldPose(_entity, _ecm).Z(), 1e-2);
+                numBaseLinkChecks++;
+              }
+              else if (_name->Data() == "link0_outer")
+              {
+                // link "link0_outer" is resting on the ground and does not
+                // move, so it should always have a pose of (1 0 0 0 0 0)
+                EXPECT_EQ(ignition::math::Pose3d(1, 0, 0, 0, 0, 0),
+                    ignition::gazebo::worldPose(_entity, _ecm));
+                numOuterLinkChecks++;
+              }
+              else if (_name->Data() == "base_link_custom")
+              {
+                // same as link "base_link"
+                EXPECT_NEAR(zExpected,
+                    ignition::gazebo::worldPose(_entity, _ecm).Z(), 1e-2);
+                numBaseLinkCustomChecks++;
+              }
+              else if (_name->Data() == "link0_outer_custom")
+              {
+                // same as "link0_outer", but with an offset pose
+                EXPECT_EQ(ignition::math::Pose3d(1, 2, 0, 0, 0, 0),
+                    ignition::gazebo::worldPose(_entity, _ecm));
+                numOuterLinkCustomChecks++;
+              }
+              else if (_name->Data() == "base_link_parent")
+              {
+                // same as link "base_link"
+                EXPECT_NEAR(zExpected,
+                    ignition::gazebo::worldPose(_entity, _ecm).Z(), 1e-2);
+                numBaseLinkParentChecks++;
+              }
+              else if (_name->Data() == "nested_model_link")
+              {
+                // same as "link0_outer", but with an offset pose
+                EXPECT_EQ(ignition::math::Pose3d(1, -2, 0, 0, 0, 0),
+                    ignition::gazebo::worldPose(_entity, _ecm));
+                numNestedModelLinkChecks++;
+              }
+              else if (_name->Data() == "parent_model_link")
+              {
+                // same as "link0_outer", but with an offset pose
+                EXPECT_EQ(ignition::math::Pose3d(1, -4, 0, 0, 0, 0),
+                    ignition::gazebo::worldPose(_entity, _ecm));
+                numParentModelLinkChecks++;
+              }
+              else if (_name->Data() == "base_link_child")
+              {
+                // same as link "base_link"
+                EXPECT_NEAR(zExpected,
+                    ignition::gazebo::worldPose(_entity, _ecm).Z(), 1e-2);
+                numBaseLinkChildChecks++;
+              }
+            }
+
+            return true;
+          });
+
+      return true;
+    });
+
+  server.AddSystem(testSystem.systemPtr);
+  const size_t iters = 500;
+  server.Run(true, iters, false);
+
+  EXPECT_EQ(iters, currIter);
+  EXPECT_EQ(iters, numBaseLinkChecks);
+  EXPECT_EQ(iters, numOuterLinkChecks);
+  EXPECT_EQ(iters, numBaseLinkCustomChecks);
+  EXPECT_EQ(iters, numOuterLinkCustomChecks);
+  EXPECT_EQ(iters, numBaseLinkParentChecks);
+  EXPECT_EQ(iters, numNestedModelLinkChecks);
+  EXPECT_EQ(iters, numParentModelLinkChecks);
+  EXPECT_EQ(iters, numBaseLinkChildChecks);
+}
+
+/////////////////////////////////////////////////
+TEST_F(PhysicsSystemFixture, IGN_UTILS_TEST_DISABLED_ON_WIN32(Heightmap))
+{
+  ignition::gazebo::ServerConfig serverConfig;
+
+  const auto sdfFile = std::string(PROJECT_SOURCE_PATH) +
+    "/test/worlds/heightmap.sdf";
+  serverConfig.SetSdfFile(sdfFile);
+
+  sdf::Root root;
+  root.Load(sdfFile);
+
+  bool checked{false};
+  int maxIt{0};
+
+  test::Relay testSystem;
+  testSystem.OnPostUpdate(
+    [&](const gazebo::UpdateInfo &_info,
+    const gazebo::EntityComponentManager &_ecm)
+    {
+      double aboveHeight;
+      double farHeight;
+      bool checkedAbove{false};
+      bool checkedFar{false};
+      bool checkedHeightmap{false};
+
+      _ecm.Each<components::Model, components::Name, components::Pose>(
+        [&](const ignition::gazebo::Entity &, const components::Model *,
+        const components::Name *_name, const components::Pose *_pose)->bool
+        {
+          if (_name->Data() == "above_heightmap")
+          {
+            aboveHeight = _pose->Data().Pos().Z();
+            checkedAbove = true;
+          }
+          else if (_name->Data() == "far_from_heightmap")
+          {
+            farHeight = _pose->Data().Pos().Z();
+            checkedFar = true;
+          }
+          else
+          {
+            EXPECT_EQ("Heightmap Bowl", _name->Data());
+            EXPECT_EQ(math::Pose3d(), _pose->Data());
+            checkedHeightmap = true;
+          }
+
+          return true;
+        });
+
+      EXPECT_TRUE(checkedAbove);
+      EXPECT_TRUE(checkedFar);
+      EXPECT_TRUE(checkedHeightmap);
+
+      // Both models drop from 7m
+      EXPECT_GE(7.01, aboveHeight) << _info.iterations;
+      EXPECT_GE(7.01, farHeight) << _info.iterations;
+
+      // Model above heightmap hits it and never drops below 5.5m
+      EXPECT_LE(5.5, aboveHeight) << _info.iterations;
+
+      // Model far from heightmap keeps falling
+      if (_info.iterations > 600)
+      {
+        EXPECT_GT(5.5, farHeight) << _info.iterations;
+      }
+
+      checked = true;
+      maxIt = _info.iterations;
+      return true;
+    });
+
+  gazebo::Server server(serverConfig);
+  server.AddSystem(testSystem.systemPtr);
+  server.Run(true, 1000, false);
+
+  EXPECT_TRUE(checked);
+  EXPECT_EQ(1000, maxIt);
 }
