@@ -25,6 +25,7 @@
 #include <ignition/common/Util.hh>
 #include <ignition/math/Pose3.hh>
 #include <ignition/transport/Node.hh>
+#include <ignition/utilities/ExtraTestMacros.hh>
 
 #include "ignition/gazebo/components/Light.hh"
 #include "ignition/gazebo/components/Link.hh"
@@ -32,35 +33,32 @@
 #include "ignition/gazebo/components/Name.hh"
 #include "ignition/gazebo/components/Physics.hh"
 #include "ignition/gazebo/components/Pose.hh"
+#include "ignition/gazebo/components/WheelSlipCmd.hh"
 #include "ignition/gazebo/components/World.hh"
+#include "ignition/gazebo/Model.hh"
 #include "ignition/gazebo/Server.hh"
 #include "ignition/gazebo/SystemLoader.hh"
 #include "ignition/gazebo/test_config.hh"
 
 #include "../helpers/Relay.hh"
+#include "../helpers/EnvTestFixture.hh"
 
 using namespace ignition;
 using namespace gazebo;
 
 //////////////////////////////////////////////////
-class UserCommandsTest : public ::testing::Test
+class UserCommandsTest : public InternalFixture<::testing::Test>
 {
-  // Documentation inherited
-  protected: void SetUp() override
-  {
-    ignition::common::Console::SetVerbosity(4);
-    ignition::common::setenv("IGN_GAZEBO_SYSTEM_PLUGIN_PATH",
-           (std::string(PROJECT_BINARY_PATH) + "/lib").c_str());
-  }
 };
 
 /////////////////////////////////////////////////
-TEST_F(UserCommandsTest, Create)
+// See https://github.com/ignitionrobotics/ign-gazebo/issues/1175
+TEST_F(UserCommandsTest, IGN_UTILS_TEST_DISABLED_ON_WIN32(Create))
 {
   // Start server
   ServerConfig serverConfig;
   const auto sdfFile = std::string(PROJECT_SOURCE_PATH) +
-    "/examples/worlds/empty.sdf";
+    "/test/worlds/empty.sdf";
   serverConfig.SetSdfFile(sdfFile);
 
   Server server(serverConfig);
@@ -336,7 +334,7 @@ TEST_F(UserCommandsTest, Create)
 }
 
 /////////////////////////////////////////////////
-TEST_F(UserCommandsTest, Remove)
+TEST_F(UserCommandsTest, IGN_UTILS_TEST_DISABLED_ON_WIN32(Remove))
 {
   // Start server
   ServerConfig serverConfig;
@@ -524,7 +522,7 @@ TEST_F(UserCommandsTest, Remove)
 }
 
 /////////////////////////////////////////////////
-TEST_F(UserCommandsTest, Pose)
+TEST_F(UserCommandsTest, IGN_UTILS_TEST_DISABLED_ON_WIN32(Pose))
 {
   // Start server
   ServerConfig serverConfig;
@@ -696,8 +694,84 @@ TEST_F(UserCommandsTest, Pose)
   EXPECT_NEAR(500.0, poseComp->Data().Pos().Y(), 0.2);
 }
 
+
 /////////////////////////////////////////////////
-TEST_F(UserCommandsTest, Light)
+TEST_F(UserCommandsTest, IGN_UTILS_TEST_DISABLED_ON_WIN32(PoseVector))
+{
+  // Start server
+  ServerConfig serverConfig;
+  const auto sdfFile = std::string(PROJECT_SOURCE_PATH) +
+    "/test/worlds/shapes.sdf";
+  serverConfig.SetSdfFile(sdfFile);
+
+  Server server(serverConfig);
+  EXPECT_FALSE(server.Running());
+  EXPECT_FALSE(*server.Running(0));
+
+  // Create a system just to get the ECM
+  EntityComponentManager *ecm{nullptr};
+  test::Relay testSystem;
+  testSystem.OnPreUpdate([&](const gazebo::UpdateInfo &,
+                             gazebo::EntityComponentManager &_ecm)
+      {
+        ecm = &_ecm;
+      });
+
+  server.AddSystem(testSystem.systemPtr);
+
+  // Run server and check we have the ECM
+  EXPECT_EQ(nullptr, ecm);
+  server.Run(true, 1, false);
+  EXPECT_NE(nullptr, ecm);
+
+  // Entity move by name
+  msgs::Pose_V req;
+
+  auto poseBoxMsg = req.add_pose();
+  poseBoxMsg->set_name("box");
+  poseBoxMsg->mutable_position()->set_y(123.0);
+
+  auto poseSphereMsg = req.add_pose();
+  poseSphereMsg->set_name("sphere");
+  poseSphereMsg->mutable_position()->set_y(456.0);
+
+  msgs::Boolean res;
+  bool result;
+  unsigned int timeout = 5000;
+  std::string service{"/world/default/set_pose_vector"};
+
+  transport::Node node;
+  EXPECT_TRUE(node.Request(service, req, timeout, res, result));
+  EXPECT_TRUE(result);
+  EXPECT_TRUE(res.data());
+
+  // Box entity
+  auto boxEntity = ecm->EntityByComponents(components::Name("box"));
+  EXPECT_NE(kNullEntity, boxEntity);
+
+  // Check entity has not been moved yet
+  auto poseComp = ecm->Component<components::Pose>(boxEntity);
+  ASSERT_NE(nullptr, poseComp);
+  EXPECT_EQ(math::Pose3d(1, 2, 3, 0, 0, 1), poseComp->Data());
+
+  // Run an iteration and check it was moved
+  server.Run(true, 1, false);
+
+  poseComp = ecm->Component<components::Pose>(boxEntity);
+  ASSERT_NE(nullptr, poseComp);
+  EXPECT_NEAR(123.0, poseComp->Data().Pos().Y(), 0.2);
+
+  auto sphereEntity = ecm->EntityByComponents(components::Name("sphere"));
+  EXPECT_NE(kNullEntity, sphereEntity);
+
+  poseComp = ecm->Component<components::Pose>(sphereEntity);
+  ASSERT_NE(nullptr, poseComp);
+  EXPECT_NEAR(456, poseComp->Data().Pos().Y(), 0.2);
+}
+
+/////////////////////////////////////////////////
+// https://github.com/ignitionrobotics/ign-gazebo/issues/634
+TEST_F(UserCommandsTest, IGN_UTILS_TEST_ENABLED_ONLY_ON_LINUX(Light))
 {
   // Start server
   ServerConfig serverConfig;
@@ -741,8 +815,10 @@ TEST_F(UserCommandsTest, Light)
   ASSERT_NE(nullptr, pointLightComp);
   EXPECT_EQ(
     math::Pose3d(0, -1.5, 3, 0, 0, 0), pointLightComp->Data().RawPose());
-  EXPECT_EQ(math::Color(1, 0, 0, 1), pointLightComp->Data().Diffuse());
-  EXPECT_EQ(math::Color(0.1, 0.1, 0.1, 1), pointLightComp->Data().Specular());
+  EXPECT_EQ(math::Color(1.0f, 0.0f, 0.0f, 1.0f),
+      pointLightComp->Data().Diffuse());
+  EXPECT_EQ(math::Color(0.1f, 0.1f, 0.1f, 1.0f),
+      pointLightComp->Data().Specular());
   EXPECT_NEAR(4.0, pointLightComp->Data().AttenuationRange(), 0.1);
   EXPECT_NEAR(0.5, pointLightComp->Data().LinearAttenuationFactor(), 0.1);
   EXPECT_NEAR(0.2, pointLightComp->Data().ConstantAttenuationFactor(), 0.1);
@@ -751,16 +827,24 @@ TEST_F(UserCommandsTest, Light)
 
   req.Clear();
   ignition::msgs::Set(req.mutable_diffuse(),
-    ignition::math::Color(0, 1, 1, 0));
+    ignition::math::Color(0.0f, 1.0f, 1.0f, 0.0f));
   ignition::msgs::Set(req.mutable_specular(),
-    ignition::math::Color(0.2, 0.2, 0.2, 0.2));
-  req.set_range(2.6);
+    ignition::math::Color(0.2f, 0.2f, 0.2f, 0.2f));
+  req.set_range(2.6f);
   req.set_name("point");
   req.set_type(ignition::msgs::Light::POINT);
-  req.set_attenuation_linear(0.7);
-  req.set_attenuation_constant(0.6);
-  req.set_attenuation_quadratic(0.001);
+  req.set_attenuation_linear(0.7f);
+  req.set_attenuation_constant(0.6f);
+  req.set_attenuation_quadratic(0.001f);
   req.set_cast_shadows(true);
+
+  // todo(ahcorde) Use the field is_light_off in light.proto from
+  // Garden on.
+  auto header = req.mutable_header()->add_data();
+  header->set_key("isLightOn");
+  std::string *value = header->add_value();
+  *value = std::to_string(true);
+
   EXPECT_TRUE(node.Request(service, req, timeout, res, result));
   EXPECT_TRUE(result);
   EXPECT_TRUE(res.data());
@@ -773,9 +857,10 @@ TEST_F(UserCommandsTest, Light)
   pointLightComp = ecm->Component<components::Light>(pointLightEntity);
   ASSERT_NE(nullptr, pointLightComp);
 
-  EXPECT_EQ(math::Color(0, 1, 1, 0), pointLightComp->Data().Diffuse());
-  EXPECT_EQ(math::Color(0.2, 0.2, 0.2, 0.2),
-    pointLightComp->Data().Specular());
+  EXPECT_EQ(math::Color(0.0f, 1.0f, 1.0f, 0.0f),
+      pointLightComp->Data().Diffuse());
+  EXPECT_EQ(math::Color(0.2f, 0.2f, 0.2f, 0.2f),
+      pointLightComp->Data().Specular());
   EXPECT_NEAR(2.6, pointLightComp->Data().AttenuationRange(), 0.1);
   EXPECT_NEAR(0.7, pointLightComp->Data().LinearAttenuationFactor(), 0.1);
   EXPECT_NEAR(0.6, pointLightComp->Data().ConstantAttenuationFactor(), 0.1);
@@ -794,10 +879,10 @@ TEST_F(UserCommandsTest, Light)
 
   EXPECT_EQ(
     math::Pose3d(0, 0, 10, 0, 0, 0), directionalLightComp->Data().RawPose());
-  EXPECT_EQ(
-    math::Color(0.8, 0.8, 0.8, 1), directionalLightComp->Data().Diffuse());
-  EXPECT_EQ(
-    math::Color(0.2, 0.2, 0.2, 1), directionalLightComp->Data().Specular());
+  EXPECT_EQ(math::Color(0.8f, 0.8f, 0.8f, 1.0f),
+    directionalLightComp->Data().Diffuse());
+  EXPECT_EQ(math::Color(0.2f, 0.2f, 0.2f, 1.0f),
+    directionalLightComp->Data().Specular());
   EXPECT_NEAR(100, directionalLightComp->Data().AttenuationRange(), 0.1);
   EXPECT_NEAR(
     0.01, directionalLightComp->Data().LinearAttenuationFactor(), 0.01);
@@ -812,15 +897,15 @@ TEST_F(UserCommandsTest, Light)
 
   req.Clear();
   ignition::msgs::Set(req.mutable_diffuse(),
-    ignition::math::Color(0, 1, 1, 0));
+    ignition::math::Color(0.0f, 1.0f, 1.0f, 0.0f));
   ignition::msgs::Set(req.mutable_specular(),
-    ignition::math::Color(0.3, 0.3, 0.3, 0.3));
-  req.set_range(2.6);
+    ignition::math::Color(0.3f, 0.3f, 0.3f, 0.3f));
+  req.set_range(2.6f);
   req.set_name("directional");
   req.set_type(ignition::msgs::Light::DIRECTIONAL);
-  req.set_attenuation_linear(0.7);
-  req.set_attenuation_constant(0.6);
-  req.set_attenuation_quadratic(1);
+  req.set_attenuation_linear(0.7f);
+  req.set_attenuation_constant(0.6f);
+  req.set_attenuation_quadratic(1.0f);
   req.set_cast_shadows(false);
   ignition::msgs::Set(req.mutable_direction(),
     ignition::math::Vector3d(1, 2, 3));
@@ -837,8 +922,9 @@ TEST_F(UserCommandsTest, Light)
     ecm->Component<components::Light>(directionalLightEntity);
   ASSERT_NE(nullptr, directionalLightComp);
 
-  EXPECT_EQ(math::Color(0, 1, 1, 0), directionalLightComp->Data().Diffuse());
-  EXPECT_EQ(math::Color(0.3, 0.3, 0.3, 0.3),
+  EXPECT_EQ(math::Color(0.0f, 1.0f, 1.0f, 0.0f),
+    directionalLightComp->Data().Diffuse());
+  EXPECT_EQ(math::Color(0.3f, 0.3f, 0.3f, 0.3f),
     directionalLightComp->Data().Specular());
   EXPECT_NEAR(2.6, directionalLightComp->Data().AttenuationRange(), 0.1);
   EXPECT_NEAR(
@@ -863,8 +949,10 @@ TEST_F(UserCommandsTest, Light)
   ASSERT_NE(nullptr, spotLightComp);
 
   EXPECT_EQ(math::Pose3d(0, 1.5, 3, 0, 0, 0), spotLightComp->Data().RawPose());
-  EXPECT_EQ(math::Color(0, 1, 0, 1), spotLightComp->Data().Diffuse());
-  EXPECT_EQ(math::Color(0.2, 0.2, 0.2, 1), spotLightComp->Data().Specular());
+  EXPECT_EQ(math::Color(0.0f, 1.0f, 0.0f, 1.0f),
+    spotLightComp->Data().Diffuse());
+  EXPECT_EQ(math::Color(0.2f, 0.2f, 0.2f, 1.0f),
+    spotLightComp->Data().Specular());
   EXPECT_NEAR(5, spotLightComp->Data().AttenuationRange(), 0.1);
   EXPECT_NEAR(0.4, spotLightComp->Data().LinearAttenuationFactor(), 0.01);
   EXPECT_NEAR(0.3, spotLightComp->Data().ConstantAttenuationFactor(), 0.1);
@@ -879,21 +967,21 @@ TEST_F(UserCommandsTest, Light)
 
   req.Clear();
   ignition::msgs::Set(req.mutable_diffuse(),
-    ignition::math::Color(1, 0, 1, 0));
+    ignition::math::Color(1.0f, 0.0f, 1.0f, 0.0f));
   ignition::msgs::Set(req.mutable_specular(),
-    ignition::math::Color(0.3, 0.3, 0.3, 0.3));
-  req.set_range(2.6);
+    ignition::math::Color(0.3f, 0.3f, 0.3f, 0.3f));
+  req.set_range(2.6f);
   req.set_name("spot");
   req.set_type(ignition::msgs::Light::SPOT);
-  req.set_attenuation_linear(0.7);
-  req.set_attenuation_constant(0.6);
-  req.set_attenuation_quadratic(1);
+  req.set_attenuation_linear(0.7f);
+  req.set_attenuation_constant(0.6f);
+  req.set_attenuation_quadratic(1.0f);
   req.set_cast_shadows(true);
   ignition::msgs::Set(req.mutable_direction(),
     ignition::math::Vector3d(1, 2, 3));
-  req.set_spot_inner_angle(1.5);
-  req.set_spot_outer_angle(0.3);
-  req.set_spot_falloff(0.9);
+  req.set_spot_inner_angle(1.5f);
+  req.set_spot_outer_angle(0.3f);
+  req.set_spot_falloff(0.9f);
 
   EXPECT_TRUE(node.Request(service, req, timeout, res, result));
   EXPECT_TRUE(result);
@@ -907,8 +995,9 @@ TEST_F(UserCommandsTest, Light)
   spotLightComp = ecm->Component<components::Light>(spotLightEntity);
   ASSERT_NE(nullptr, spotLightComp);
 
-  EXPECT_EQ(math::Color(1, 0, 1, 0), spotLightComp->Data().Diffuse());
-  EXPECT_EQ(math::Color(0.3, 0.3, 0.3, 0.3),
+  EXPECT_EQ(math::Color(1.0f, 0.0f, 1.0f, 0.0f),
+    spotLightComp->Data().Diffuse());
+  EXPECT_EQ(math::Color(0.3f, 0.3f, 0.3f, 0.3f),
     spotLightComp->Data().Specular());
   EXPECT_NEAR(2.6, spotLightComp->Data().AttenuationRange(), 0.1);
   EXPECT_NEAR(0.7, spotLightComp->Data().LinearAttenuationFactor(), 0.1);
@@ -923,7 +1012,7 @@ TEST_F(UserCommandsTest, Light)
 }
 
 /////////////////////////////////////////////////
-TEST_F(UserCommandsTest, Physics)
+TEST_F(UserCommandsTest, IGN_UTILS_TEST_DISABLED_ON_WIN32(Physics))
 {
   // Start server
   ServerConfig serverConfig;
@@ -957,7 +1046,7 @@ TEST_F(UserCommandsTest, Physics)
   auto physicsComp = ecm->Component<components::Physics>(worldEntity);
   ASSERT_NE(nullptr, physicsComp);
   EXPECT_DOUBLE_EQ(0.001, physicsComp->Data().MaxStepSize());
-  EXPECT_DOUBLE_EQ(1.0, physicsComp->Data().RealTimeFactor());
+  EXPECT_DOUBLE_EQ(0.0, physicsComp->Data().RealTimeFactor());
 
   // Set physics properties
   msgs::Physics req;
@@ -982,4 +1071,121 @@ TEST_F(UserCommandsTest, Physics)
   physicsComp = ecm->Component<components::Physics>(worldEntity);
   EXPECT_DOUBLE_EQ(0.123, physicsComp->Data().MaxStepSize());
   EXPECT_DOUBLE_EQ(4.567, physicsComp->Data().RealTimeFactor());
+
+  // Send invalid values (not > 0) and make sure they are not updated
+  req.set_max_step_size(0.0);
+  req.set_real_time_factor(0.0);
+
+  EXPECT_TRUE(node.Request(service, req, timeout, res, result));
+  EXPECT_TRUE(result);
+  EXPECT_TRUE(res.data());
+
+  // Run two iterations, in the first one the PhysicsCmd component is created
+  // in the second one it is processed
+  server.Run(true, 2, false);
+
+  // Check updated physics properties
+  physicsComp = ecm->Component<components::Physics>(worldEntity);
+  EXPECT_DOUBLE_EQ(0.123, physicsComp->Data().MaxStepSize());
+  EXPECT_DOUBLE_EQ(4.567, physicsComp->Data().RealTimeFactor());
 }
+
+/////////////////////////////////////////////////
+TEST_F(UserCommandsTest, IGN_UTILS_TEST_DISABLED_ON_WIN32(WheelSlip))
+{
+  // Start server
+  ServerConfig serverConfig;
+  const auto sdfFile = std::string(PROJECT_SOURCE_PATH) +
+    "/test/worlds/trisphere_cycle_wheel_slip.sdf";
+  serverConfig.SetSdfFile(sdfFile);
+
+  Server server(serverConfig);
+  EXPECT_FALSE(server.Running());
+  EXPECT_FALSE(*server.Running(0));
+
+  // Create a system just to get the ECM
+  EntityComponentManager *ecm{nullptr};
+  test::Relay testSystem;
+  testSystem.OnPreUpdate([&](const gazebo::UpdateInfo &,
+                             gazebo::EntityComponentManager &_ecm)
+      {
+        ecm = &_ecm;
+      });
+
+  server.AddSystem(testSystem.systemPtr);
+
+  // Run server and check we have the ECM
+  EXPECT_EQ(nullptr, ecm);
+  server.Run(true, 1, false);
+  ASSERT_NE(nullptr, ecm);
+
+  // Check that the physics properties are the ones specified in the sdf
+  auto worldEntity = ecm->EntityByComponents(components::World());
+  ASSERT_NE(kNullEntity, worldEntity);
+  Entity tc0 = ecm->EntityByComponents(
+    components::Name("trisphere_cycle0"));
+  ASSERT_NE(kNullEntity, tc0);
+  Entity tc1 = ecm->EntityByComponents(
+    components::Name("trisphere_cycle1"));
+  ASSERT_NE(kNullEntity, tc1);
+
+  Model tcModel0{tc0};
+  Model tcModel1{tc1};
+  Entity wf0 = tcModel0.LinkByName(*ecm, "wheel_front");
+  ASSERT_NE(kNullEntity, wf0);
+  Entity wrl0 = tcModel0.LinkByName(*ecm, "wheel_rear_left");
+  ASSERT_NE(kNullEntity, wrl0);
+  Entity wrf0 = tcModel0.LinkByName(*ecm, "wheel_rear_right");
+  ASSERT_NE(kNullEntity, wrf0);
+  Entity wf1 = tcModel1.LinkByName(*ecm, "wheel_front");
+  ASSERT_NE(kNullEntity, wf1);
+  Entity wrl1 = tcModel1.LinkByName(*ecm, "wheel_rear_left");
+  ASSERT_NE(kNullEntity, wrl1);
+  Entity wrf1 = tcModel1.LinkByName(*ecm, "wheel_rear_right");
+  ASSERT_NE(kNullEntity, wrf1);
+
+  Entity links[] = {wf0, wrl0, wrf0, wf1, wrl1, wrf1};
+  for (auto link : links) {
+    EXPECT_EQ(nullptr, ecm->Component<components::WheelSlipCmd>(link));
+  }
+
+  // modify wheel slip parameters of one link of model 0
+  msgs::WheelSlipParametersCmd req;
+  auto * entityMsg = req.mutable_entity();
+  entityMsg->set_name("trisphere_cycle0::wheel_front");
+  entityMsg->set_type(msgs::Entity::LINK);
+  req.set_slip_compliance_lateral(1);
+  req.set_slip_compliance_longitudinal(1);
+
+  msgs::Boolean res;
+  bool result;
+  unsigned int timeout = 5000;
+  std::string service{"/world/wheel_slip/wheel_slip"};
+
+  transport::Node node;
+  EXPECT_TRUE(node.Request(service, req, timeout, res, result));
+  EXPECT_TRUE(result);
+  EXPECT_TRUE(res.data());
+
+  // Run two iterations, in the first one the WheelSlipCmd component is created
+  // and processed.
+  // The second one is just to check everything went fine.
+  server.Run(true, 2, false);
+
+  // modify wheel slip parameters of one link of model 1
+  entityMsg->set_name("trisphere_cycle1");
+  entityMsg->set_type(msgs::Entity::MODEL);
+  req.set_slip_compliance_lateral(2);
+  req.set_slip_compliance_longitudinal(1);
+
+  result = false;
+  res = msgs::Boolean{};
+
+  EXPECT_TRUE(node.Request(service, req, timeout, res, result));
+  EXPECT_TRUE(result);
+  EXPECT_TRUE(res.data());
+
+  // Run two iterations, in the first one the WheelSlipCmd component is created
+  // and processed.
+  // The second one is just to check everything went fine.
+  server.Run(true, 3, false);}
